#pragma once

#ifndef __OPENCV__
  #error 666: do not include this file manually include only 'opencv.bi' !
#endif

extern "C"


' #########################################################################
' # Array allocation, deallocation, initialization and access to elements #
' #########################################################################

' <malloc> wrapper.
' If there is no enough memory, the function raises an error.
declare function cvAlloc( size as size_t ) as any ptr

' <free> wrapper.
' Here and further all the memory releasing functions (that all call cvFree) 
' take double pointer in order to to clear pointer to the data after releasing it.
' Passing pointer to NULL pointer is Ok: nothing happens in this case
declare sub      cvFree_( _ptr as any ptr )
#define cvFree(_ptr) cvFree_(*(_ptr)) : *(_ptr)=NULL

'' Allocates and initializes IplImage header
declare function cvCreateImageHeader( size as CvSize, depth as long, channels as long ) as IplImage ptr

' Inializes IplImage header
declare function cvInitImageHeader( image as IplImage ptr, size as CvSize, depth as long, _
                                    channels as long, origin as long CV_DEFAULT(0), _
                                    align as long CV_DEFAULT(4)) as IplImage ptr

' Creates IPL image (header and data)
declare function  cvCreateImage( size as CvSize, depth as long, channels as long ) as IplImage ptr

' Releases (i.e. deallocates) IPL image header/
declare sub      cvReleaseImageHeader( image as IplImage ptr ptr )

' Releases IPL image header and data
declare sub      cvReleaseImage( image as IplImage ptr ptr )

' Creates a copy of IPL image (widthStep may differ)
declare function cvCloneImage( image as const IplImage ptr ) as IplImage ptr

' Sets a Channel Of Interest (only a few functions support COI) -
' use cvCopy to extract the selected channel and/or put it back
declare sub      cvSetImageCOI( image as IplImage ptr, coi as long)

' Retrieves image Channel Of Interest
declare function cvGetImageCOI( image as const IplImage ptr ) as long

' Sets image ROI (region of interest) (COI is not changed)
declare sub      cvSetImageROI( image as IplImage ptr, rect as CvRect )

' Resets image ROI and COI
declare sub      cvResetImageROI( image as IplImage ptr )

' Retrieves image ROI
declare function cvGetImageROI( image as const IplImage ptr ) as CvRect

' Allocates and initializes CvMat header
declare function cvCreateMatHeader( rows as long, cols as long, _type as long ) as CvMat ptr

#define CV_AUTOSTEP  &H7fffffff

' Initializes CvMat header
declare function cvInitMatHeader( mat as CvMat ptr, rows as long, cols as long, _
                                  _type as long, data as any ptr CV_DEFAULT(NULL), _
                                  _step as long CV_DEFAULT(CV_AUTOSTEP) ) as CvMat ptr

' Allocates and initializes CvMat header and allocates data
declare function cvCreateMat( rows as long, cols as long, _type as long ) as CvMat ptr

' Releases CvMat header and deallocates matrix data
' (reference counting is used for data)
declare sub      cvReleaseMat( mat as CvMat ptr ptr )

' Decrements CvMat data reference counter and deallocates the data if it reaches 0
sub cvDecRefData( arr as CvArr ptr )
  if ( CV_IS_MAT( arr )) then
    dim as CvMat ptr mat = cast(CvMat ptr, arr)
    mat->data._ptr = NULL
    if ( mat->refcount <> NULL ) then 
      *mat->refcount-=1  
      if *mat->refcount=0 then cvFree( @mat->refcount )
    end if   
    mat->refcount = NULL
  elseif ( CV_IS_MATND( arr )) then
    dim as CvMatND ptr mat = cast(CvMatND ptr, arr)
    mat->data._ptr = NULL
    if( mat->refcount <> NULL) then 
      *mat->refcount-=1
      if *mat->refcount=0 then cvFree( @mat->refcount )
    endif    
    mat->refcount = NULL
  end if  
end sub

' Increments array data reference counter.
' The function increments CvMat or CvMatND data reference counter and returns the new counter value 
' if the reference counter pointer is not NULL, otherwise it returns zero.
'  param arr Array header
function cvIncRefData( arr as CvArr ptr ) as long
  dim as long refcount
  if ( CV_IS_MAT( arr )) then
    dim as CvMat ptr mat = cptr(CvMat ptr, arr)
    if( mat->refcount <> NULL ) then
      *mat->refcount+=1
      refcount = *mat->refcount
    end if  
  elseif( CV_IS_MATND( arr )) then
    dim as CvMatND ptr mat = cptr(CvMatND ptr, arr)
    if( mat->refcount <> NULL ) then
      *mat->refcount+=1
      refcount = *mat->refcount
    end if        
  end if
  return refcount
end function

' Creates an exact copy of the input matrix (except, may be, step value)
declare function cvCloneMat( mat as const CvMat ptr ) as CvMat ptr


' Makes a new matrix from <rect> subrectangle of input array.
' No data is copied
declare function cvGetSubRect( arr as const CvArr ptr, submat as CvMat ptr, rect as CvRect ) as CvMat ptr
#define cvGetSubArr cvGetSubRect

' Selects row span of the input array: arr(start_row:delta_row:end_row,:)
' (end_row is not included into the span).
declare function cvGetRows( arr as const CvArr ptr, submat as CvMat ptr, _
                            start_row as long, end_row as long, _
                            delta_row as long CV_DEFAULT(1)) as CvMat ptr

function cvGetRow( arr as const CvArr ptr, submat as CvMat ptr, row as long ) as CvMat ptr
  return cvGetRows( arr, submat, row, row + 1, 1 )
end function

' Selects column span of the input array: arr(:,start_col:end_col)
' (end_col is not included into the span)
declare function cvGetCols( arr as const CvArr ptr, submat as CvMat ptr, _
                            start_col as long, end_col as long ) as CvMat ptr

function cvGetCol( arr as const CvArr ptr, submat as CvMat ptr, col as long ) as CvMat ptr
  return cvGetCols( arr, submat, col, col + 1 )
end function

' Select a diagonal of the input array.
' (diag = 0 means the main diagonal, >0 means a diagonal above the main one,
'  <0 - below the main one).
' The diagonal will be represented as a column (nx1 matrix).
declare function cvGetDiag( arr as const CvArr ptr, submat as CvMat ptr, _
                            diag as long CV_DEFAULT(0)) as CvMat ptr

' low-level scalar <-> raw data conversion functions
declare sub      cvScalarToRawData( scalar as const CvScalar ptr, _data as any ptr, _type as long, _
                                    extend_to_12 as long CV_DEFAULT(0) )

declare sub      cvRawDataToScalar( _datat as const any ptr, _type as long, scalar as CvScalar ptr)

' Allocates and initializes CvMatND header
declare function cvCreateMatNDHeader( dims as long, sizes as const long ptr, _type as long) as CvMatND ptr

' Allocates and initializes CvMatND header and allocates data
declare function cvCreateMatND( dims as long, sizes as const long ptr, _type as long ) as CvMatND ptr

' Initializes preallocated CvMatND header
declare function cvInitMatNDHeader( mat as CvMatND ptr, dims as long, sizes as const long ptr, _
                                    _type as long, _data as any ptr CV_DEFAULT(NULL) ) as CvMatND ptr

' Releases CvMatND
sub cvReleaseMatND( mat as CvMatND ptr ptr )
  cvReleaseMat( cptr(CvMat ptr ptr, mat ) )
end sub

' Creates a copy of CvMatND (except, may be, steps)
declare function cvCloneMatND( mat as const CvMatND ptr ) as CvMatND ptr

' Allocates and initializes CvSparseMat header and allocates data
declare function cvCreateSparseMat( dims as long, sizes as const long ptr, _type as long ) as CvSparseMat ptr

' Releases CvSparseMat
declare sub      cvReleaseSparseMat( mat as CvSparseMat ptr ptr )

' Creates a copy of CvSparseMat (except, may be, zero items)
declare function cvCloneSparseMat( mat as const CvSparseMat ptr ) as CvSparseMat ptr

' Initializes sparse array iterator
' (returns the first node or NULL if the array is empty)
declare function cvInitSparseMatIterator( mat as const CvSparseMat ptr, _
                                          it as CvSparseMatIterator ptr ) as CvSparseNode ptr

' returns next sparse array node (or NULL if there is no more nodes)
function cvGetNextSparseNode( it as CvSparseMatIterator ptr ) as CvSparseNode ptr
  if (it->node->next ) then
    it->node = it->node->next
    return it->node
  else
    it->curidx+=1
    dim as long idx = it->curidx
    while idx < it->mat->hashsize
      dim as CvSparseNode ptr node = cptr(CvSparseNode ptr,it->mat->hashtable[idx])
      if ( node ) then
        it->curidx = idx
        it->node = node
        return it->node
      end if
      idx+=1
    wend
    return NULL
  end if
end function

' ###############################################################
' #  matrix iterator: used for n-ary operations on dense arrays #
' ###############################################################
#define CV_MAX_ARR 10

type CvNArrayIterator
  as long        count               ' number of arrays
  as long        dims                ' number of dimensions to iterate
  as CvSize      size                ' maximal common linear size: { width = size, height = 1 }
  as ubyte ptr   _ptr(CV_MAX_ARR-1)  ' renamed: ptr pointers to the array slices
  as long        stack(CV_MAX_DIM-1) ' for internal use
  as CvMatND ptr hdr(CV_MAX_ARR-1)   ' pointers to the headers of the matrices that are processed
end type


#define CV_NO_DEPTH_CHECK     1
#define CV_NO_CN_CHECK        2
#define CV_NO_SIZE_CHECK      4

' initializes iterator that traverses through several arrays simulteneously
' (the function together with cvNextArraySlice is used for N-ari element-wise operations)
declare function cvInitNArrayIterator( count as long, arrs as CvArr ptr ptr, _
                                       mask as const CvArr ptr,  stubs as CvMatND ptr, _
                                       it as CvNArrayIterator ptr, _
                                       flags as long CV_DEFAULT(0) ) as long

' returns zero value if iteration is finished, non-zero (slice length) otherwise
declare function cvNextNArraySlice( it as CvNArrayIterator ptr ) as long


' Returns type of array elements:
' CV_8UC1 ... CV_64FC4 ...
declare function cvGetElemType( arr as const CvArr ptr ) as long

' Retrieves number of an array dimensions and optionally sizes of the dimensions
declare function cvGetDims( arr as const CvArr ptr, sizes as long ptr CV_DEFAULT(NULL) ) as long


' Retrieves size of a particular array dimension.
' For 2d arrays cvGetDimSize(arr,0) returns number of rows (image height)
' and cvGetDimSize(arr,1) returns number of columns (image width)
declare function cvGetDimSize( arr as const CvArr ptr, index as long ) as long


' ptr = &arr(idx0,idx1,...). All indexes are zero-based,
' the major dimensions go first (e.g. (y,x) for 2D, (z,y,x) for 3D
declare function cvPtr1D( arr as const CvArr ptr, idx0 as long, _type as long ptr CV_DEFAULT(NULL)) as ubyte ptr
declare function cvPtr2D( arr as const CvArr ptr, idx0 as long, idx1 as long, _type as long ptr CV_DEFAULT(NULL) ) as ubyte ptr
declare function cvPtr3D( arr as const CvArr ptr, idx0 as long, idx1 as long, idx2 as long, _type as long ptr CV_DEFAULT(NULL)) as ubyte ptr

' For CvMat or IplImage number of indices should be 2
' (row index (y) goes first, column index (x) goes next).
' For CvMatND or CvSparseMat number of infices should match number of <dims> and
' indices order should match the array dimension order.
declare function cvPtrND( arr as const CvArr ptr, idx as const long ptr, _type as long ptr CV_DEFAULT(NULL), _
                          create_node as long CV_DEFAULT(1), _
                          precalc_hashval as ulong ptr CV_DEFAULT(NULL)) as ubyte ptr

' value = arr(idx0,idx1,...)
declare function cvGet1D( arr as const CvArr ptr, idx0 as long ) as CvScalar
declare function cvGet2D( arr as const CvArr ptr, idx0 as long, idx1 as long ) as CvScalar
declare function cvGet3D( arr as const CvArr ptr, idx0 as long, idx1 as long, idx2 as long ) as CvScalar
declare function cvGetND( arr as const CvArr ptr, idx as const long ptr ) as CvScalar

' for 1-channel arrays
declare function cvGetReal1D( arr as const CvArr ptr, idx0 as long ) as double
declare function cvGetReal2D( arr as const CvArr ptr, idx0 as long, idx1 as long ) as double
declare function cvGetReal3D( arr as const CvArr ptr, idx0 as long, idx1 as long, idx2 as long ) as double
declare function cvGetRealND( arr as const CvArr ptr, idx as const long ptr ) as double

' arr(idx0,idx1,...) = value
declare sub      cvSet1D( arr as CvArr ptr, idx0 as long, value as CvScalar )
declare sub      cvSet2D( arr as CvArr ptr, idx0 as long, idx1 as long, value as CvScalar )
declare sub      cvSet3D( arr as CvArr ptr, idx0 as long, idx1 as long, idx2 as long, value as CvScalar )
declare sub      cvSetND( arr as CvArr ptr, idx as const long ptr, value as CvScalar )

' for 1-channel arrays
declare sub      cvSetReal1D( arr as CvArr ptr, idx0 as long, value as double )
declare sub      cvSetReal2D( arr as CvArr ptr, idx0 as long, idx1 as long, value as double )
declare sub      cvSetReal3D( arr as CvArr ptr, idx0 as long, idx1 as long, idx2 as long, value as double )
declare sub      cvSetRealND( arr as CvArr ptr, idx as const long ptr, value as double )

' clears element of ND dense array, in case of sparse arrays it deletes the specified node
declare sub      cvClearND( arr as CvArr ptr, idx as const long ptr )

' Converts CvArr (IplImage or CvMat,...) to CvMat.
' If the last parameter is non-zero, function can convert multi(>2)-dimensional array 
' to CvMat as long as the last array's dimension is continous.
' The resultant matrix will be have appropriate (a huge) number of rows
declare function cvGetMat( arr as const CvArr ptr, header as CvMat ptr, _
                           coi as long ptr CV_DEFAULT(NULL), _
                           allowND as long CV_DEFAULT(0)) as CvMat ptr

' Converts CvArr (IplImage or CvMat) to IplImage
declare function cvGetImage( arr as const CvArr ptr, image_header as IplImage ptr ) as IplImage ptr


' Changes a shape of multi-dimensional array.
' new_cn = 0 means that number of channels remains unchanged.
' new_dims = 0 means that number and sizes of dimensions remain the same
' (unless they need to be changed to set the new number of channels)
' if new_dims = 1, there is no need to specify new dimension sizes
' The resultant configuration should be achievable w/o data copying.
' If the resultant array is sparse, CvSparseMat header should be passed
' to the function else if the result is 1 or 2 dimensional,
' CvMat header should be passed to the function
' else CvMatND header should be passed
declare function cvReshapeMatND( arr as const CvArr ptr, _
                                 sizeof_header as long, header as CvArr ptr, _
                                 new_cn as long, new_dims as long, new_sizes as long ptr) as CvArr ptr 

#define cvReshapeND( arr, header, new_cn, new_dims, new_sizes ) _
      cvReshapeMatND( (arr), sizeof(*(header)), (header), (new_cn), (new_dims), (new_sizes) )

declare function cvReshape( arr as const CvArr ptr, header as CvMat ptr, _
                            new_cn as long, new_rows as long CV_DEFAULT(0) ) as CvMat ptr

' Repeats source 2d array several times in both horizontal and
' vertical direction to fill destination array
declare sub      cvRepeat( src as const CvArr ptr, dst as CvArr ptr )

' Allocates array data
declare sub      cvCreateData( arr as CvArr ptr )

' Releases array data
declare sub      cvReleaseData( arr as CvArr ptr )

' Attaches user data to the array header. The step is reffered to
' the pre-last dimension. That is, all the planes of the array
' must be joint (w/o gaps)
declare sub      cvSetData( arr as CvArr ptr, _data as any ptr, _step as long )

' Retrieves raw data of CvMat, IplImage or CvMatND.
' In the latter case the function raises an error if
' the array can not be represented as a matrix
declare sub      cvGetRawData( arr as const CvArr ptr, _data as ubyte ptr ptr , _
                               _step as long ptr CV_DEFAULT(NULL), _
                               roi_size as CvSize ptr CV_DEFAULT(NULL))

' Returns width and height of array in elements
declare function cvGetSize( arr as const CvArr ptr ) as CvSize

' Copies source array to destination array
declare sub      cvCopy(src as const CvArr ptr, dst as CvArr ptr, _
                        mask as const CvArr ptr CV_DEFAULT(NULL) )

' Sets all or "masked" elements of input array to the same value
declare sub      cvSet( arr as CvArr ptr, value as CvScalar, _
                        mask as const CvArr ptr CV_DEFAULT(NULL) )

' Clears all the array elements (sets them to 0)
declare sub      cvSetZero( arr as CvArr ptr )
#define cvZero  cvSetZero


' Splits a multi-channel array into the set of single-channel arrays
' or extracts particular [color] plane
declare sub      cvSplit( src as const CvArr ptr, dst0 as CvArr ptr, dst1 as CvArr ptr, _
                          dst2 as CvArr ptr, dst3 as CvArr ptr )

' Merges a set of single-channel arrays into the single multi-channel array
' or inserts one particular [color] plane to the array '/
declare sub      cvMerge( src0 as const CvArr ptr, src1 as const CvArr ptr, _
                          src2 as const CvArr ptr, src3 as const CvArr ptr, _
                          dst as CvArr ptr )

' Copies several channels from input arrays to
' certain channels of output arrays
declare sub      cvMixChannels( src as const CvArr ptr ptr, src_count as long, _
                                dst as CvArr ptr ptr, dst_count as long, _
                                from_to as const long ptr, pair_count as long)

' Performs linear transformation on every source array element:
' dst(x,y,c) = scale*src(x,y,c)+shift.
' Arbitrary combination of input and output array depths are allowed
' (number of channels must be the same), thus the function can be used for type conversion
declare sub      cvConvertScale( src as const CvArr ptr, dst as CvArr ptr, _
                                 scale as double CV_DEFAULT(1), _
                                 shift as double  CV_DEFAULT(0) )
#define cvCvtScale cvConvertScale
#define cvScale  cvConvertScale
#define cvConvert( src, dst )  cvConvertScale( (src), (dst), 1, 0 )


' Performs linear transformation on every source array element,
' stores absolute value of the result:
' dst(x,y,c) = abs(scale*src(x,y,c)+shift).
' destination array must have 8u type.
' In other cases one may use cvConvertScale + cvAbsDiffS
declare sub      cvConvertScaleAbs( src as const CvArr ptr, dst as CvArr ptr, _
                                    scale as double CV_DEFAULT(1), _
                                    shift as double CV_DEFAULT(0) )
#define cvCvtScaleAbs  cvConvertScaleAbs


' checks termination criteria validity and sets eps to default_eps (if it is not set),
' max_iter to default_max_iters (if it is not set)
declare function cvCheckTermCriteria( criteria as CvTermCriteria, eps as double ptr, _
                                       default_max_iters as long ) as CvTermCriteria

' ###############################################
' # Arithmetic, logic and comparison operations #
' ###############################################
' dst(mask) = src1(mask) + src2(mask)
declare sub      cvAdd( src1 as const CvArr ptr, src2 as const CvArr ptr, dst as CvArr ptr, _
                        mask as const CvArr ptr CV_DEFAULT(NULL) )

' dst(mask) = src(mask) + value
declare sub      cvAddS( src as const CvArr ptr, value as CvScalar, dst as CvArr ptr, _
                         mask as const CvArr ptr CV_DEFAULT(NULL) )

' dst(mask) = src1(mask) - src2(mask)
declare sub      cvSub( src1 as const CvArr ptr, src2 as const CvArr ptr, dst as CvArr ptr, _
                        mask as const CvArr ptr CV_DEFAULT(NULL) )

' dst(mask) = src(mask) - value = src(mask) + (-value)
sub cvSubS( src as const CvArr ptr, value as CvScalar, dst as CvArr ptr, _
            mask as const CvArr ptr CV_DEFAULT(NULL))
  cvAddS( src, cv_Scalar( -value.val(0), -value.val(1), -value.val(2), -value.val(3) ), dst, mask )
end sub

' dst(mask) = value - src(mask)
declare sub      cvSubRS( src as const CvArr ptr, value as CvScalar, dst as CvArr ptr, _
                          mask as const CvArr ptr CV_DEFAULT(NULL))

' dst(idx) = src1(idx) * src2(idx) * scale
' (scaled element-wise multiplication of 2 arrays)
declare sub      cvMul( src1 as const CvArr ptr, src2 as const CvArr ptr, _
                        dst as CvArr ptr, scale as double CV_DEFAULT(1) )

' element-wise division/inversion with scaling:
' dst(idx) = src1(idx) * scale / src2(idx)
' or dst(idx) = scale / src2(idx) if src1 == 0 
declare sub      cvDiv( src1 as const CvArr ptr, src2 as const CvArr ptr, _
                    dst as CvArr ptr, scale as double CV_DEFAULT(1) )

' dst = src1 * scale + src2
declare sub      cvScaleAdd( src1 as const CvArr ptr, scale as CvScalar, _
                             src2 as const CvArr ptr, dst as CvArr ptr )
#define cvAXPY( A, real_scalar, B, C ) cvScaleAdd(A, cvRealScalar(real_scalar), B, C)

' dst = src1 * alpha + src2 * beta + gamma
declare sub      cvAddWeighted( src1 as const CvArr ptr, alpha as double, _
                                src2 as const CvArr ptr, beta as double, _
                                gamma as double, dst as CvArr ptr )

' result = sum_i(src1(i) * src2(i)) (results for all channels are accumulated together)
declare function  cvDotProduct( src1 as const CvArr ptr, src2 as const CvArr ptr ) as double

' dst(idx) = src1(idx) & src2(idx)
declare sub      cvAnd( src11 as const CvArr ptr, src2 as const CvArr ptr, _
                        dst as CvArr ptr, mask as const CvArr ptr CV_DEFAULT(NULL) )

' dst(idx) = src(idx) & value
declare sub      cvAndS( src as const CvArr ptr, value as CvScalar, _
                         dst as CvArr ptr, mask as const CvArr ptr CV_DEFAULT(NULL) )

' dst(idx) = src1(idx) | src2(idx)
declare sub      cvOr( src1 as const CvArr ptr, src2 as const CvArr ptr, _
                       dst as CvArr ptr, mask as const CvArr ptr CV_DEFAULT(NULL) )

' dst(idx) = src(idx) | value
declare sub      cvOrS( src as const CvArr ptr, value as CvScalar, _
                        dst as CvArr ptr, mask as const CvArr ptr CV_DEFAULT(NULL) )

' dst(idx) = src1(idx) ^ src2(idx)
declare sub      cvXor( src1 as const CvArr ptr, src2 as const CvArr ptr, _
                        dst as CvArr ptr, mask as const CvArr ptr CV_DEFAULT(NULL) )

' dst(idx) = src(idx) ^ value
declare sub      cvXorS( src as const CvArr ptr, value as CvScalar, _
                         dst as CvArr ptr, mask as const CvArr ptr CV_DEFAULT(NULL) ) 

' dst(idx) = ~src(idx)
declare sub      cvNot( src as const CvArr ptr, dst as CvArr ptr )

' dst(idx) = lower(idx) <= src(idx) < upper(idx)
declare sub      cvInRange( src as const CvArr ptr, lower as const CvArr ptr, _
                            upper as const CvArr ptr, dst as CvArr ptr )

' dst(idx) = lower <= src(idx) < upper
declare sub      cvInRangeS( src as const CvArr ptr, lower as CvScalar, _
                             upper as CvScalar, dst as CvArr ptr )

#define CV_CMP_EQ   0
#define CV_CMP_GT   1
#define CV_CMP_GE   2
#define CV_CMP_LT   3
#define CV_CMP_LE   4
#define CV_CMP_NE   5

' The comparison operation support single-channel arrays only.
' Destination image should be 8uC1 or 8sC1 '/

' dst(idx) = src1(idx) _cmp_op_ src2(idx)
declare sub      cvCmp( src1 as const CvArr ptr, src2 as const CvArr ptr, dst as CvArr ptr, cmp_op as long)

' dst(idx) = src1(idx) _cmp_op_ value
declare sub      cvCmpS( src as const CvArr ptr, value as double, dst as CvArr ptr, cmp_op as long )

' dst(idx) = min(src1(idx),src2(idx))
declare sub      cvMin( src1 as const CvArr ptr, src2 as const CvArr ptr, dst as CvArr ptr )

' dst(idx) = max(src1(idx),src2(idx))
declare sub      cvMax( src as const CvArr ptr, src2 as const CvArr ptr, dst as CvArr ptr )

' dst(idx) = min(src(idx),value)
declare sub      cvMinS( src as const CvArr ptr, value as double, dst as CvArr ptr )

' dst(idx) = max(src(idx),value)
declare sub      cvMaxS( src as const CvArr ptr, value as double, dst as CvArr ptr )

' dst(x,y,c) = abs(src1(x,y,c) - src2(x,y,c))
declare sub      cvAbsDiff( src1 as const CvArr ptr, src2 as const CvArr ptr, dst as CvArr ptr )

' dst(x,y,c) = abs(src(x,y,c) - value(c))
declare sub      cvAbsDiffS( src as const CvArr ptr, dst as CvArr ptr, value as CvScalar )
#define cvAbs( src, dst ) cvAbsDiffS( (src), (dst), cvScalarAll(0))

' ###################
' # Math operations #
' ###################

' Does cartesian->polar coordinates conversion.
' Either of output components (magnitude or angle) is optional
declare sub      cvCartToPolar( x as const CvArr ptr, y as const CvArr ptr, _
                                magnitude as CvArr ptr, angle as CvArr ptr CV_DEFAULT(NULL), _
                                angle_in_degrees as long CV_DEFAULT(0) )

' Does polar->cartesian coordinates conversion.
' Either of output components (magnitude or angle) is optional.
' If magnitude is missing it is assumed to be all 1's
declare sub      cvPolarToCart( magnitude as const CvArr ptr, angle as const CvArr ptr, _
                                x as CvArr ptr, y as CvArr ptr, _
                                angle_in_degrees as long CV_DEFAULT(0) )

' Does powering: dst(idx) = src(idx)^power
declare sub      cvPow( src as const CvArr ptr, dst as CvArr ptr, power as double )

' Does exponention: dst(idx) = exp(src(idx)).
' Overflow is not handled yet. Underflow is handled.
' Maximal relative error is ~7e-6 for single-precision input
declare sub      cvExp( src as const CvArr ptr, dst as CvArr ptr )

' Calculates natural logarithms: dst(idx) = log(abs(src(idx))).
' Logarithm of 0 gives large negative number(~-700)
' Maximal relative error is ~3e-7 for single-precision output
declare sub      cvLog( src as const CvArr ptr, dst as CvArr ptr )

' Fast arctangent calculation
declare function cvFastArctan( y as single, x as single ) as single

' Fast cubic root calculation
declare function cvCbrt( value as single ) as single

' Checks array values for NaNs, Infs or simply for too large numbers
' (if CV_CHECK_RANGE is set). If CV_CHECK_QUIET is set,
' no runtime errors is raised (function returns zero value in case of "bad" values).
' Otherwise cvError is called
#define  CV_CHECK_RANGE    1
#define  CV_CHECK_QUIET    2
declare function cvCheckArr( arr as const CvArr ptr, flags as long CV_DEFAULT(0), _
                             min_val as double CV_DEFAULT(0),  max_val as double CV_DEFAULT(0) ) as long
#define cvCheckArray cvCheckArr

#define CV_RAND_UNI      0
#define CV_RAND_NORMAL   1
declare sub      cvRandArr( rng as CvRNG ptr, arr as CvArr ptr, dist_type as long, _
                            param1 as CvScalar, oaram2 as CvScalar )

declare sub      cvRandShuffle( mat as CvArr ptr, rng as CvRNG ptr, _
                                iter_factor as double CV_DEFAULT(1.0) )

#define CV_SORT_EVERY_ROW 0
#define CV_SORT_EVERY_COLUMN 1
#define CV_SORT_ASCENDING 0
#define CV_SORT_DESCENDING 16

declare sub      cvSort( src as const CvArr ptr, dst as CvArr ptr CV_DEFAULT(NULL), _
                         idxmat as CvArr ptr CV_DEFAULT(NULL), _
                         flags as long CV_DEFAULT(0) )

' Finds real roots of a cubic equation
declare function cvSolveCubic( coeffs as const CvMat ptr, roots as CvMat ptr ) as long

' Finds all real and complex roots of a polynomial equation
declare sub      cvSolvePoly(coeffs as const CvMat ptr, roots2 as CvMat ptr, _
                             maxiter as long CV_DEFAULT(20), fig as long CV_DEFAULT(100) )

' #####################
' # Matrix operations #
' #####################

' Calculates cross product of two 3d vectors
declare sub      cvCrossProduct( src1 as const CvArr ptr, src2 as const CvArr ptr, dst as CvArr ptr )

' Matrix transform: dst = A*B + C, C is optional
#define cvMatMulAdd( src1, src2, src3, dst ) cvGEMM( (src1), (src2), 1., (src3), 1., (dst), 0 )
#define cvMatMul( src1, src2, dst )  cvMatMulAdd( (src1), (src2), NULL, (dst) )

#define CV_GEMM_A_T 1
#define CV_GEMM_B_T 2
#define CV_GEMM_C_T 4
' Extended matrix transform:
' dst = alpha*op(A)*op(B) + beta*op(C), where op(X) is X or X^T
declare sub      cvGEMM( src1 as const CvArr ptr, _
                         src2 as const CvArr ptr, alpha as double, _
                         src3 as const CvArr ptr, beta as double, _
                         dst as CvArr ptr, tABC as long CV_DEFAULT(0) )
#define cvMatMulAddEx cvGEMM

' Transforms each element of source array and stores
' resultant vectors in destination array
declare sub      cvTransform( src as const CvArr ptr, dst as CvArr ptr, _
                              transmat as const CvMat ptr, _
                              shiftvec as const CvMat ptr CV_DEFAULT(NULL) )
#define cvMatMulAddS cvTransform

' Does perspective transform on every element of input array
declare sub      cvPerspectiveTransform( src as const CvArr ptr, dst as CvArr ptr, _
                                         mat as const CvMat ptr )

' Calculates (A-delta)*(A-delta)^T (order=0) or (A-delta)^T*(A-delta) (order=1)
declare sub      cvMulTransposed( src as const CvArr ptr, dst as CvArr ptr, order as long, _
                                  delata as const CvArr ptr CV_DEFAULT(NULL), scale as double CV_DEFAULT(1.0) )

' Tranposes matrix. Square matrices can be transposed in-place
declare sub      cvTranspose( src as const CvArr ptr, dst as CvArr ptr )
#define cvT cvTranspose

' Completes the symmetric matrix from the lower (LtoR=0) or from the upper (LtoR!=0) part
declare sub      cvCompleteSymm( matrix as CvMat ptr, LtoR as long CV_DEFAULT(0) )

' Mirror array data around horizontal (flip=0),
' vertical (flip=1) or both(flip=-1) axises:
' cvFlip(src) flips images vertically and sequences horizontally (inplace)
declare sub      cvFlip( src as const CvArr ptr, dst as CvArr ptr CV_DEFAULT(NULL), _
                         flip_mode as long CV_DEFAULT(0) )
#define cvMirror cvFlip

#define CV_SVD_MODIFY_A   1
#define CV_SVD_U_T        2
#define CV_SVD_V_T        4

' Performs Singular Value Decomposition of a matrix
declare sub       cvSVD( A as CvArr ptr, W as CvArr ptr, U as CvArr ptr CV_DEFAULT(NULL), _
                         V as CvArr ptr CV_DEFAULT(NULL), flags as long CV_DEFAULT(0) )

' Performs Singular Value Back Substitution (solves A*X = B):
' flags must be the same as in cvSVD
declare sub       cvSVBkSb( W as const CvArr ptr, U as const CvArr ptr, _
                            V as const CvArr ptr, B as const CvArr ptr, _
                            X as CvArr ptr, flags as long )

#define CV_LU  0
#define CV_SVD 1
#define CV_SVD_SYM 2
#define CV_CHOLESKY 3
#define CV_QR  4
#define CV_NORMAL 16

' Inverts matrix
declare function  cvInvert( src as const CvArr ptr, dst as CvArr ptr, _
                             method as long CV_DEFAULT(CV_LU)) as double
#define cvInv cvInvert

' Solves linear system (src1)*(dst) = (src2)
' (returns 0 if src1 is a singular and CV_LU method is used)
declare function cvSolve( src1 as const CvArr ptr, src2 as const CvArr ptr, dst as CvArr ptr, _
                          method as long CV_DEFAULT(CV_LU) ) as long

' Calculates determinant of input matrix
declare function cvDet( mat as const CvArr ptr ) as double

' Calculates trace of the matrix (sum of elements on the main diagonal)
declare function cvTrace( mat as const CvArr ptr ) as CvScalar

' Finds eigen values and vectors of a symmetric matrix
declare sub      cvEigenVV( mat as CvArr ptr, evects as CvArr ptr, evals as CvArr ptr, _
                            eps as double CV_DEFAULT(0), lowindex as long CV_DEFAULT(-1), _
                            highindex as long CV_DEFAULT(-1) )

' Makes an identity matrix (mat_ij = i == j)
declare sub      cvSetIdentity( mat as CvArr ptr, value as CvScalar CV_DEFAULT( cvRealScalar(1)) )

' Fills matrix with given range of numbers
declare function  cvRange( mat as CvArr ptr, start as double, _end as double ) as CvArr ptr

' Calculates covariation matrix for a set of vectors
' transpose([v1-avg, v2-avg,...]) * [v1-avg,v2-avg,...]
#define CV_COVAR_SCRAMBLED 0

' [v1-avg, v2-avg,...] * transpose([v1-avg,v2-avg,...])
#define CV_COVAR_NORMAL    1

' do not calc average (i.e. mean vector) - use the input vector instead
' (useful for calculating covariance matrix by parts)
#define CV_COVAR_USE_AVG   2

' scale the covariance matrix coefficients by number of the vectors
#define CV_COVAR_SCALE     4

' all the input vectors are stored in a single matrix, as its rows
#define CV_COVAR_ROWS      8

' all the input vectors are stored in a single matrix, as its columns
#define CV_COVAR_COLS     16

declare sub      cvCalcCovarMatrix( vects as const CvArr ptr ptr, count as long, _
                                    cov_mat as CvArr ptr, avg as CvArr ptr, flags as long )

#define CV_PCA_DATA_AS_ROW 0
#define CV_PCA_DATA_AS_COL 1
#define CV_PCA_USE_AVG 2
declare sub      cvCalcPCA( _data as const CvArr ptr, mean as CvArr ptr, _
                            eigenvals as CvArr ptr, eigenvects as CvArr ptr, flags as long )

declare sub      cvProjectPCA( _data as const CvArr ptr, mean as const CvArr ptr, _
                               eigenvects as const CvArr ptr, result as CvArr ptr )

declare sub      cvBackProjectPCA( proj as const CvArr ptr, mean as const CvArr ptr, _
                                   eigenvects as const CvArr ptr, result as CvArr ptr )

' Calculates Mahalanobis(weighted) distance
declare function cvMahalanobis( vec1 as const CvArr ptr, vec2 as const CvArr ptr, mat as const CvArr ptr ) as double
#define cvMahalonobis  cvMahalanobis

' ####################
' # Array Statistics #
' ####################

' Finds sum of array elements
declare function cvSum( arr as const CvArr ptr ) as CvScalar

' Calculates number of non-zero pixels
declare function cvCountNonZero( arr as const CvArr ptr ) as long

' Calculates mean value of array elements
declare function cvAvg( arr as const CvArr ptr, mask as const CvArr ptr CV_DEFAULT(NULL) ) as CvScalar

' Calculates mean and standard deviation of pixel values
declare sub      cvAvgSdv( arr as const CvArr ptr, mean as CvScalar ptr, std_dev as CvScalar ptr, _
                           mask as const CvArr ptr CV_DEFAULT(NULL) )

' Finds global minimum, maximum and their positions
declare sub      cvMinMaxLoc( arr as const CvArr ptr, min_val as double ptr, max_val as double ptr, _
                              min_loc as CvPoint ptr CV_DEFAULT(NULL), _
                              max_loc as CvPoint ptr CV_DEFAULT(NULL), _
                              mask as const CvArr ptr CV_DEFAULT(NULL) )

' types of array norm
#define CV_C            1
#define CV_L1           2
#define CV_L2           4
#define CV_NORM_MASK    7
#define CV_RELATIVE     8
#define CV_DIFF         16
#define CV_MINMAX       32

#define CV_DIFF_C       (CV_DIFF | CV_C)
#define CV_DIFF_L1      (CV_DIFF | CV_L1)
#define CV_DIFF_L2      (CV_DIFF | CV_L2)
#define CV_RELATIVE_C   (CV_RELATIVE | CV_C)
#define CV_RELATIVE_L1  (CV_RELATIVE | CV_L1)
#define CV_RELATIVE_L2  (CV_RELATIVE | CV_L2)

' Finds norm, difference norm or relative difference norm for an array (or two arrays)
declare function cvNorm( arr1 as const CvArr ptr, arr2 as const CvArr ptr CV_DEFAULT(NULL), _
                         norm_type as long CV_DEFAULT(CV_L2), _
                         mask as const CvArr ptr CV_DEFAULT(NULL) ) as double

declare sub      cvNormalize( src as const CvArr ptr, dst as CvArr ptr, _
                              a as double CV_DEFAULT(1.), b as double CV_DEFAULT(0.), _
                              norm_type as long CV_DEFAULT(CV_L2), _
                              mask as const CvArr ptr CV_DEFAULT(NULL) )


#define CV_REDUCE_SUM 0
#define CV_REDUCE_AVG 1
#define CV_REDUCE_MAX 2
#define CV_REDUCE_MIN 3

declare sub      cvReduce( src as const CvArr ptr, dst as CvArr ptr, _dim as long CV_DEFAULT(-1), _
                           op as long CV_DEFAULT(CV_REDUCE_SUM) )

' ####################################################
' # Discrete Linear Transforms and Related Functions #
' ####################################################

#define CV_DXT_FORWARD  0
#define CV_DXT_INVERSE  1
#define CV_DXT_SCALE    2 ' divide result by size of array
#define CV_DXT_INV_SCALE (CV_DXT_INVERSE + CV_DXT_SCALE)
#define CV_DXT_INVERSE_SCALE CV_DXT_INV_SCALE
#define CV_DXT_ROWS     4 ' transform each row individually
#define CV_DXT_MUL_CONJ 8 ' conjugate the second argument of cvMulSpectrums

' Discrete Fourier Transform:
' complex->complex, real->ccs (forward), ccs->real (inverse)
declare sub      cvDFT( src as const CvArr ptr, dst as CvArr ptr, flags as long, _
                        nonzero_rows as long CV_DEFAULT(0) )
#define cvFFT cvDFT

' Multiply results of DFTs: DFT(X)*DFT(Y) or DFT(X)*conj(DFT(Y))
declare sub      cvMulSpectrums( src1 as const CvArr ptr, src2 as const CvArr ptr, _
                                 dst as CvArr ptr, flags as long )

' Finds optimal DFT vector size >= size0
declare function cvGetOptimalDFTSize( size0 as long) as long

' Discrete Cosine Transform
declare sub      cvDCT( src as const CvArr ptr, dst as CvArr ptr, flags as long )

' ###########################
' # Dynamic data structures #
' ###########################

' Calculates length of sequence slice (with support of negative indices).
declare function cvSliceLength( slice as CvSlice, seq as const CvSeq ptr ) as long


' Creates new memory storage.
' block_size == 0 means that default,
' somewhat optimal size, is used (currently, it is 64K)
declare function cvCreateMemStorage( block_size as long CV_DEFAULT(0) ) as CvMemStorage ptr

' Creates a memory storage that will borrow memory blocks from parent storage
declare function cvCreateChildMemStorage( parent as CvMemStorage ptr ) as CvMemStorage ptr

' Releases memory storage. All the children of a parent must be released before the parent.
' A child storage returns all the blocks to parent when it is released
declare sub      cvReleaseMemStorage( storage as CvMemStorage ptr ptr )

' Clears memory storage. 
' This is the only way(!!!)  (besides cvRestoreMemStoragePos)
' to reuse memory allocated for the storage cvClearSeq, cvClearSet ... do not free any memory.
' A child storage returns all the blocks to the parent when it is cleared
declare sub      cvClearMemStorage( storage as CvMemStorage ptr )

' Remember a storage "free memory" position
declare sub      cvSaveMemStoragePos( storage as const CvMemStorage ptr, _pos as CvMemStoragePos ptr )

' Restore a storage "free memory" position
declare sub      cvRestoreMemStoragePos( storage as CvMemStorage ptr, _pos as CvMemStoragePos ptr )

' Allocates continuous buffer of the specified size in the storage
declare function cvMemStorageAlloc( storage as CvMemStorage ptr, size as size_t ) as any ptr

' Allocates string in memory storage
declare function cvMemStorageAllocString( storage as CvMemStorage ptr, _ptr as const zstring ptr, _
                                          _len as long CV_DEFAULT(-1) ) as CvString

' Creates new empty sequence that will reside in the specified storage
declare function cvCreateSeq( seq_flags as long, header_size as size_t, _
                              elem_size as size_t, storage as CvMemStorage ptr ) as CvSeq ptr

' Changes default size (granularity) of sequence blocks.
' The default size is ~1Kbyte
declare sub      cvSetSeqBlockSize( seq as CvSeq ptr, delta_elems as long )

' Adds new element to the end of sequence. Returns pointer to the element
declare function cvSeqPush( seq as CvSeq ptr, element as const any ptr CV_DEFAULT(NULL) ) as byte ptr

' Adds new element to the beginning of sequence. Returns pointer to it
declare function cvSeqPushFront( seq as CvSeq ptr, element as const any ptr CV_DEFAULT(NULL) ) as byte ptr

' Removes the last element from sequence and optionally saves it
declare sub      cvSeqPop( seq as CvSeq ptr, element as any ptr CV_DEFAULT(NULL) )

' Removes the first element from sequence and optioanally saves it
declare sub      cvSeqPopFront( seq as CvSeq ptr, element as any ptr CV_DEFAULT(NULL) )

#define CV_FRONT 1
#define CV_BACK  0
' Adds several new elements to the end of sequence
declare sub      cvSeqPushMulti( seq as CvSeq ptr, elements as const any ptr , _
                                 count as long, in_front as long CV_DEFAULT(0) )

' Removes several elements from the end of sequence and optionally saves them
declare sub      cvSeqPopMulti( seq as CvSeq ptr, elements as const any ptr, _
                                count as long, in_front as long CV_DEFAULT(0) )

' Inserts a new element in the middle of sequence.
' cvSeqInsert(seq,0,elem) == cvSeqPushFront(seq,elem)
declare function cvSeqInsert( seq as CvSeq ptr, before_index as long, _
                              element as const any ptr CV_DEFAULT(NULL) ) as byte ptr

' Removes specified sequence element
declare sub      cvSeqRemove( seq as CvSeq ptr, index as long )

' Removes all the elements from the sequence.
' The freed memory can be reused later only by the same sequence
' unless cvClearMemStorage or cvRestoreMemStoragePos is called
declare sub      cvClearSeq( seq as CvSeq ptr )

' Retrieves pointer to specified sequence element.
' Negative indices are supported and mean counting from the end
' (e.g -1 means the last sequence element)
declare function cvGetSeqElem( seq as const CvSeq ptr, index as long ) as byte ptr

' Calculates index of the specified sequence element.
' Returns -1 if element does not belong to the sequence
declare function cvSeqElemIdx( seq as const CvSeq ptr,  element as const any ptr, _
                               block as CvSeqBlock ptr ptr CV_DEFAULT(NULL) ) as long

' Initializes sequence writer. The new elements will be added to the end of sequence
declare sub      cvStartAppendToSeq( seq as CvSeq ptr, writer as CvSeqWriter ptr )

' Combination of cvCreateSeq and cvStartAppendToSeq
declare sub      cvStartWriteSeq( seq_flags as long, header_size as long, _
                                  elem_size as long, storage as CvMemStorage ptr, _
                                  writer as CvSeqWriter ptr )

' Closes sequence writer, updates sequence header and returns pointer to the resultant sequence
' (which may be useful if the sequence was created using cvStartWriteSeq)
declare function cvEndWriteSeq( writer as CvSeqWriter ptr ) as CvSeq ptr

' Updates sequence header. May be useful to get access to some of previously
' written elements via cvGetSeqElem or sequence reader
declare sub      cvFlushSeqWriter( writer as CvSeqWriter ptr )

' Initializes sequence reader.
' The sequence can be read in forward or backward direction
declare sub      cvStartReadSeq( seq as const CvSeq ptr, reader as CvSeqReader ptr, reverse as long CV_DEFAULT(0) )

' Returns current sequence reader position (currently observed sequence element)
declare function cvGetSeqReaderPos( reader as CvSeqReader ptr ) as long

' Changes sequence reader position. 
' It may seek to an absolute or to relative to the current position
declare sub       cvSetSeqReaderPos( reader as CvSeqReader ptr, index as long, _
                                     is_relative as long CV_DEFAULT(0) )

' Copies sequence content to a continuous piece of memory
declare function  cvCvtSeqToArray( seq as const CvSeq ptr, elements as const any ptr, _
                                   slice as CvSlice CV_DEFAULT(CV_WHOLE_SEQ) ) as any ptr

' Creates sequence header for array.
' After that all the operations on sequences that do not alter the content
' can be applied to the resultant sequence
declare function cvMakeSeqHeaderForArray( seq_type as long, header_size as long, _
                                          elem_size as long, elements as const any ptr, total as long, _
                                          seq as CvSeq ptr, block as CvSeqBlock ptr ) as CvSeq ptr

' Extracts sequence slice (with or without copying sequence elements)
declare function cvSeqSlice( seq as const CvSeq ptr, slice as CvSlice, _
                             storage as CvMemStorage ptr CV_DEFAULT(NULL), _
                             copy_data as long CV_DEFAULT(0) ) as CvSeq ptr

function cvCloneSeq( seq as const CvSeq ptr, storage as CvMemStorage ptr CV_DEFAULT(NULL) ) as CvSeq ptr
  return cvSeqSlice( seq, CV_WHOLE_SEQ, storage, 1 )
end function

' Removes sequence slice
declare sub      cvSeqRemoveSlice( seq as CvSeq ptr, slice as CvSlice )

' Inserts a sequence or array into another sequence
declare sub      cvSeqInsertSlice( seq as CvSeq ptr, before_index as long, from_arr as const CvArr ptr )

' a < b ? -1 : a > b ? 1 : 0
type CvCmpFunc as function CV_CDECL(a as const any ptr, b as const any ptr, userdata as any ptr) as long

' Sorts sequence in-place given element comparison function
declare sub      cvSeqSort( seq as CvSeq ptr, func as CvCmpFunc, userdata as any ptr CV_DEFAULT(NULL) )

' Finds element in a [sorted] sequence
declare function cvSeqSearch( seq as CvSeq ptr, elem as const any ptr, func as CvCmpFunc ptr, _
                              is_sorted as long, elem_idx as long ptr, _
                              userdata as any ptr CV_DEFAULT(NULL) ) as byte ptr

' Reverses order of sequence elements in-place
declare sub      cvSeqInvert( seq as CvSeq ptr )

' Splits sequence into one or more equivalence classes using the specified criteria
declare function cvSeqPartition( seq as const CvSeq ptr, storage as CvMemStorage ptr, _
                                 labels as CvSeq ptr ptr, is_equal as CvCmpFunc, userdata as any ptr ) as long

' ###############################
' # Internal sequence functions #
' ###############################
declare sub      cvChangeSeqBlock( reader as any ptr, direction as long )
declare sub      cvCreateSeqBlock( writer as CvSeqWriter ptr )

' Creates a new set
declare function cvCreateSet( set_flags as long, header_size as long, _
                              elem_size as long, storage as CvMemStorage ptr ) as CvSet ptr

' Adds new element to the set and returns pointer to it
declare function cvSetAdd( set_header as CvSet ptr, elem as CvSetElem ptr CV_DEFAULT(NULL), _
                           inserted_elem as CvSetElem  ptr ptr CV_DEFAULT(NULL) ) as long

' Fast variant of cvSetAdd
function cvSetNew( set_header as CvSet ptr ) as CvSetElem ptr
  dim as CvSetElem ptr elem = set_header->free_elems
  if ( elem ) then
    set_header->free_elems = elem->next_free
    elem->flags = elem->flags and CV_SET_ELEM_IDX_MASK
    set_header->active_count+=1
  else
    cvSetAdd( set_header, NULL, @elem )
  end if  
  return elem
end function

' Removes set element given its pointer
sub cvSetRemoveByPtr( set_header as CvSet ptr, elem as any ptr )
  dim as CvSetElem ptr _elem = cptr(CvSetElem ptr, elem)
  _elem->next_free = set_header->free_elems
  _elem->flags = (_elem->flags and CV_SET_ELEM_IDX_MASK) or CV_SET_ELEM_FREE_FLAG
  set_header->free_elems = _elem
  set_header->active_count-=1
end sub

' Removes element from the set by its index
declare sub       cvSetRemove( set_header as CvSet ptr, index as long )

' Returns a set element by index. If the element doesn't belong to the set, NULL is returned
function cvGetSetElem( set_header as const CvSet ptr, index as long ) as CvSetElem ptr
  dim as CvSetElem ptr elem = cptr(CvSetElem ptr,cptr(any ptr,cvGetSeqElem( cptr(CvSeq ptr,set_header), index )))
  return iif(elem,iif(CV_IS_SET_ELEM( elem ),elem,0),0)
end function

' Removes all the elements from the set
declare sub      cvClearSet( set_header as CvSet ptr )

' Creates new graph
declare function cvCreateGraph( graph_flags as long, header_size as long, _
                                vtx_size as long, edge_size as long, _
                                storage as CvMemStorage ptr ) as CvGraph ptr

' Adds new vertex to the graph
declare function cvGraphAddVtx( graph as CvGraph ptr, vtx as const CvGraphVtx ptr CV_DEFAULT(NULL), _
                                inserted_vtx as CvGraphVtx ptr ptr CV_DEFAULT(NULL) ) as long


' Removes vertex from the graph together with all incident edges
declare function cvGraphRemoveVtx( graph as CvGraph ptr, index as long ) as long
declare function cvGraphRemoveVtxByPtr( graph as CvGraph ptr, vtx as CvGraphVtx ptr ) as long


' Link two vertices specifed by indices or pointers if they are not connected 
' or return pointer to already existing edge connecting the vertices.
' Functions return 1 if a new edge was created, 0 otherwise 
declare function cvGraphAddEdge( graph as CvGraph ptr, _
                                 start_idx as long, end_idx as long, _
                                 edge as const CvGraphEdge ptr CV_DEFAULT(NULL), _
                                 inserted_edge as CvGraphEdge ptr ptr CV_DEFAULT(NULL) ) as long

declare function cvGraphAddEdgeByPtr( graph as CvGraph ptr, _
                                      start_vtx as CvGraphVtx ptr, end_vtx as CvGraphVtx ptr, _
                                      edge as const CvGraphEdge ptr CV_DEFAULT(NULL), _
                                      inserted_edge as CvGraphEdge ptr ptr CV_DEFAULT(NULL) ) as long

' Remove edge connecting two vertices
declare sub      cvGraphRemoveEdge( graph as CvGraph ptr, start_idx as long, end_idx as long )
declare sub      cvGraphRemoveEdgeByPtr( graph as CvGraph ptr, start_vtx as CvGraphVtx ptr, _
                                         end_vtx as CvGraphVtx ptr )

' Find edge connecting two vertices
declare function cvFindGraphEdge( graph as const CvGraph ptr, start_idx as long, end_idx as long ) as CvGraphEdge ptr
declare function cvFindGraphEdgeByPtr( graph as const CvGraph ptr, _
                                       start_vtx as const CvGraphVtx ptr, _
                                       end_vtx as const CvGraphVtx ptr ) as CvGraphEdge ptr
#define cvGraphFindEdge cvFindGraphEdge
#define cvGraphFindEdgeByPtr cvFindGraphEdgeByPtr

' Remove all vertices and edges from the graph
declare sub      cvClearGraph( graph as CvGraph ptr )


' Count number of edges incident to the vertex
declare function cvGraphVtxDegree( graph as const CvGraph ptr, vtx_idx as long ) as long
declare function cvGraphVtxDegreeByPtr( graph as const CvGraph ptr, vtx as const CvGraphVtx ptr ) as long

' Retrieves graph vertex by given index
#define cvGetGraphVtx( graph, idx ) cptr( CvGraphVtx ptr,cvGetSetElem(cptr(CvSet ptr,graph), (idx)) )

' Retrieves index of a graph vertex given its pointer
#define cvGraphVtxIdx( graph, vtx ) ((vtx)->flags and CV_SET_ELEM_IDX_MASK)

' Retrieves index of a graph edge given its pointer
#define cvGraphEdgeIdx( graph, edge ) ((edge)->flags and CV_SET_ELEM_IDX_MASK)

#define cvGraphGetVtxCount( graph ) ((graph)->active_count)
#define cvGraphGetEdgeCount( graph ) ((graph)->edges->active_count)

#define  CV_GRAPH_VERTEX        1
#define  CV_GRAPH_TREE_EDGE     2
#define  CV_GRAPH_BACK_EDGE     4
#define  CV_GRAPH_FORWARD_EDGE  8
#define  CV_GRAPH_CROSS_EDGE    16
#define  CV_GRAPH_ANY_EDGE      30
#define  CV_GRAPH_NEW_TREE      32
#define  CV_GRAPH_BACKTRACKING  64
#define  CV_GRAPH_OVER          -1

#define  CV_GRAPH_ALL_ITEMS    -1

' flags for graph vertices and edges
#define  CV_GRAPH_ITEM_VISITED_FLAG  (1 shl 30)
#define  CV_IS_GRAPH_VERTEX_VISITED(vtx) _
    (cptr(CvGraphVtx ptr,vtx)->flags and CV_GRAPH_ITEM_VISITED_FLAG)
#define  CV_IS_GRAPH_EDGE_VISITED(edge) _
    (cptr(CvGraphVtx ptr,vtx)->flags and CV_GRAPH_ITEM_VISITED_FLAG)
#define  CV_GRAPH_SEARCH_TREE_NODE_FLAG   (1 shl 29)
#define  CV_GRAPH_FORWARD_EDGE_FLAG       (1 shl 28)

type CvGraphScanner
  as CvGraphVtx  ptr vtx   ' current graph vertex (or current edge origin)
  as CvGraphVtx  ptr dst   ' current graph edge destination vertex
  as CvGraphEdge ptr edge  ' current edge
  as CvGraph     ptr graph ' the graph
  as CvSeq       ptr stack ' the graph vertex stack
  as long            index ' the lower bound of certainly visited vertices
  as long            mask  ' event mask
end type

' Creates new graph scanner.
declare function cvCreateGraphScanner( graph as CvGraph ptr, _
                                       vtx as CvGraphVtx ptr  CV_DEFAULT(NULL), _
                                       mask as long CV_DEFAULT(CV_GRAPH_ALL_ITEMS)) as CvGraphScanner ptr

' Releases graph scanner.
declare sub      cvReleaseGraphScanner( scanner as CvGraphScanner ptr ptr )

' Get next graph element
declare function cvNextGraphItem( scanner as CvGraphScanner ptr ) as long

' Creates a copy of graph
declare function cvCloneGraph( graph as const CvGraph ptr, storage as CvMemStorage ptr ) as CvGraph ptr

' ###########
' # Drawing #
' ###########

' Drawing functions work with images/matrices of arbitrary type.
' For color images the channel order is BGR[A]
' Antialiasing is supported only for 8-bit image now.
' All the functions include parameter color that means rgb value
' (that may be constructed with CV_RGB macro) for color images and brightness for grayscale images.                                                            *
' If a drawn figure is partially or completely outside of the image, it is clipped.

#define CV_RGB( r, g, b )  cv_Scalar( (b), (g), (r), 0 )
#define CV_FILLED -1

#define CV_AA 16

' Draws 4-connected, 8-connected or antialiased line segment connecting two points
declare sub      cvLine( image as CvArr ptr, p1 as CvPoint, p2 as CvPoint, _
                         colour as CvScalar, thickness as long CV_DEFAULT(1), _
                         line_type as long CV_DEFAULT(8), shift as long CV_DEFAULT(0) )

' Draws a rectangle given two opposite corners of the rectangle (pt1 & pt2),
' if thickness<0 (e.g. thickness = CV_FILLED), the filled box is drawn
declare sub      cvRectangle( image as CvArr ptr, p1 as CvPoint, p2 as CvPoint, _
                              colour as CvScalar, thickness as long CV_DEFAULT(1), _
                              line_type as long CV_DEFAULT(8), _
                              shift as long CV_DEFAULT(0) )

' Draws a rectangle specified by a CvRect structure
declare sub      cvRectangleR( image as CvArr ptr, r as CvRect, _
                               colour as CvScalar, thickness as long CV_DEFAULT(1), _
                               line_type as long CV_DEFAULT(8), _
                               shift as long CV_DEFAULT(0) )

' Draws a circle with specified center and radius.
' Thickness works in the same way as with cvRectangle
declare sub      cvCircle( image as CvArr ptr, center as CvPoint, radius as long, _
                           colour as CvScalar, thickness as long CV_DEFAULT(1), _
                           line_type as long CV_DEFAULT(8), shift as long CV_DEFAULT(0) )

' Draws ellipse outline, filled ellipse, elliptic arc or filled elliptic sector,
' depending on <thickness>, <start_angle> and <end_angle> parameters.
' The resultant figure is rotated by <angle>. 
' All the angles are in degrees
declare sub      cvEllipse( image as CvArr ptr, center as CvPoint, axes as CvSize, _
                            angle as double, start_angle as double, end_angle as double, _
                            colour as CvScalar, thickness as long CV_DEFAULT(1), _
                            line_type as long CV_DEFAULT(8), shift as long CV_DEFAULT(0) )

sub cvEllipseBox( image as CvArr ptr, box as CvBox2D, colour as CvScalar, _
                  thickness as long CV_DEFAULT(1), _
                  line_type as long CV_DEFAULT(8), _
                  shift as long CV_DEFAULT(0) )
  dim as CvSize axes
  axes.width = cvRound(box.size.width*0.5)
  axes.height = cvRound(box.size.height*0.5)
  cvEllipse( image, cvPointFrom32f( box.center ), axes, box.angle, 0, 360, colour, thickness, line_type, shift )
end sub

' Fills convex or monotonous polygon.
declare sub      cvFillConvexPoly( image as CvArr ptr, pts as const CvPoint ptr, npts as long, colour as CvScalar, _
                                   line_type as long CV_DEFAULT(8), shift as long CV_DEFAULT(0) )

' Fills an area bounded by one or more arbitrary polygons
declare sub      cvFillPoly( image as CvArr ptr, pts as CvPoint ptr ptr, npts as const long ptr, _
                             contours as long, colour as CvScalar, _
                             line_type as long CV_DEFAULT(8), shift as long CV_DEFAULT(0) )

' Draws one or more polygonal curves
declare sub      cvPolyLine( image as CvArr ptr, pts as CvPoint ptr ptr, npts as const long ptr, _
                             contours as long, is_closed as long, colour as CvScalar, thickness as long CV_DEFAULT(1), _
                             line_type as long CV_DEFAULT(8), shift as long CV_DEFAULT(0) )

#define cvDrawRect cvRectangle
#define cvDrawLine cvLine
#define cvDrawCircle cvCircle
#define cvDrawEllipse cvEllipse
#define cvDrawPolyLine cvPolyLine

' Clips the line segment connecting *pt1 and *pt2 by the rectangular window
' (0<=x<img_size.width, 0<=y<img_size.height).
declare function cvClipLine( img_size as CvSize, p1 as CvPoint ptr, p2 as CvPoint ptr) as long

' Initializes line iterator.
' Initially, line_iterator->ptr will point to pt1 (or pt2, see left_to_right description) location in the image.
' Returns the number of pixels on the line between the ending points.
declare function cvInitLineIterator( image as const CvArr ptr, p1 as CvPoint ptr, p2 as CvPoint ptr, _
                                     line_iterator as CvLineIterator ptr, _
                                     connectivity as long CV_DEFAULT(8), _
                                     left_to_right as long CV_DEFAULT(0) ) as long

' Moves iterator to the next line point
#macro CV_NEXT_LINE_POINT( line_iterator )
  dim as long _line_iterator_mask = iif(line_iterator).err < 0 , -1 , 0)
  (line_iterator).err += (line_iterator).minus_delta + ((line_iterator).plus_delta and _line_iterator_mask)
  (line_iterator).ptr += (line_iterator).minus_step  + ((line_iterator).plus_step  and _line_iterator_mask)
#endmacro

' basic font types
#define CV_FONT_HERSHEY_SIMPLEX         0
#define CV_FONT_HERSHEY_PLAIN           1
#define CV_FONT_HERSHEY_DUPLEX          2
#define CV_FONT_HERSHEY_COMPLEX         3
#define CV_FONT_HERSHEY_TRIPLEX         4
#define CV_FONT_HERSHEY_COMPLEX_SMALL   5
#define CV_FONT_HERSHEY_SCRIPT_SIMPLEX  6
#define CV_FONT_HERSHEY_SCRIPT_COMPLEX  7

' font flags
#define CV_FONT_ITALIC                 16
#define CV_FONT_VECTOR0    CV_FONT_HERSHEY_SIMPLEX

' Font structure
type CvFont
  as const zstring ptr nameFont  ' Qt:nameFont
  as CvScalar          color     ' Qt:ColorFont -> cvScalar(blue, green, red[, alpha])
  as long              font_face ' Qt:bool italic =CV_FONT_*
  as const long ptr    ascii     ' font data and metrics
  as const long ptr    greek
  as const long ptr    cyrillic
  as single            hscale
  as single            vscale
  as single            shear      ' slope coefficient: 0 - normal, >0 - italic
  as long              thickness  ' Qt: weight letters thickness
  as single            dx         ' horizontal interval between letters
  as long              line_type  ' Qt: PointSize
end type

' Initializes font structure used further in cvPutText
declare sub      cvInitFont( font as CvFont ptr, font_face as long, _
                             hscale as double, vscale as double, shear as double CV_DEFAULT(0), _
                             thickness as long CV_DEFAULT(1), line_type as long CV_DEFAULT(8) )
' rnamed cv_Font
function cv_Font( scale as double, thickness as long CV_DEFAULT(1) ) as CvFont
  dim as CvFont font
  cvInitFont( @font, CV_FONT_HERSHEY_PLAIN, scale, scale, 0, thickness, CV_AA )
  return font
end function

' Renders text stroke with specified font and color at specified location.
' CvFont should be initialized with cvInitFont
declare sub      cvPutText( image as CvArr ptr, _
                            text as const zstring ptr, _
                            org as CvPoint, _
                            font as const CvFont ptr, _
                            colour as CvScalar )

' Calculates bounding box of text stroke (useful for alignment)
declare sub      cvGetTextSize( text_string as const zstring ptr, _
                                font as const CvFont ptr, _
                                text_size as CvSize ptr, _
                                baseline as long ptr )



' Unpacks color value, if arrtype is CV_8UC?, 
' <color> is treated as packed color value, 
' otherwise the first channels (depending on arrtype)
' of destination scalar are set to the same value = <color>
declare function  cvColorToScalar( packed_color as double, arrtype as long ) as CvScalar

' Returns the polygon points which make up the given ellipse.
' The ellipse is define by the box of size 'axes' rotated 'angle' around the 'center'.
' A partial sweep of the ellipse arc can be done 
' by spcifying arc_start and arc_end to be something other than 0 and 360, respectively.
' The input array 'pts' must be large enough to hold the result.
' The total number of points stored into 'pts' is returned by this function.
declare function cvEllipse2Poly( center as CvPoint, axes as CvSize, _
                                 angle as long, arc_start as long, arc_end as long, _
                                 pts as CvPoint ptr, delta as long ) as long

' Draws contour outlines or filled interiors on the image
declare sub      cvDrawContours( image as CvArr ptr, contour as CvSeq ptr, _
                                 external_color as CvScalar, hole_color as CvScalar, _
                                 max_level as long, thickness as long CV_DEFAULT(1), _
                                 line_type as long CV_DEFAULT(8), _
                                 offset as CvPoint CV_DEFAULT(cv_Point(0,0)) )

' Does look-up transformation.
' Elements of the source array (that should be 8uC1 or 8sC1) are used as indexes in lutarr 256-element table
declare sub      cvLUT( src as const CvArr ptr, dst as CvArr ptr, lut as const CvArr ptr )

' #######################################
' # Iteration through the sequence tree #
' #######################################
type CvTreeNodeIterator
  as const any ptr node
  as long level
  as long max_level
end type

declare sub      cvInitTreeNodeIterator( it as CvTreeNodeIterator ptr, _
                                         first as const any ptr, max_level as long)
declare function cvNextTreeNode( it as CvTreeNodeIterator ptr ) as any ptr
declare function cvPrevTreeNode( it as CvTreeNodeIterator ptr ) as any ptr

' Inserts sequence into tree with specified "parent" sequence.
' If parent is equal to frame (e.g. the most external contour),
' then added contour will have null pointer to parent.
declare sub      cvInsertNodeIntoTree( node as any ptr, parent as any ptr, frame as any ptr )

' Removes contour from tree (together with the contour children).
declare sub      cvRemoveNodeFromTree( node as any ptr, frame as any ptr )

' Gathers pointers to all the sequences,
' accessible from the <first>, to the single sequence
declare function cvTreeToNodeSeq( first as const any ptr, header_size as long, _
                                  storage as CvMemStorage ptr ) as CvSeq ptr

' The function implements the K-means algorithm 
' for clustering an array of sample vectors in a specified number of classes
#define CV_KMEANS_USE_INITIAL_LABELS    1
declare function cvKMeans2( samples as const CvArr ptr, cluster_count as long, labels as CvArr ptr, _
                            termcrit as CvTermCriteria, attempts as long CV_DEFAULT(1), _
                            rng as CvRNG ptr CV_DEFAULT(0), flags as long CV_DEFAULT(0), _
                            centers as CvArr ptr CV_DEFAULT(NULL), _
                            compactness as double CV_DEFAULT(0) ) as long

' ####################
' # System functions #
' ####################

' Add the function pointers table with associated information to the IPP primitives list
declare function cvRegisterModule( module_info as const CvModuleInfo ptr ) as long

' Loads optimized functions from IPP, MKL etc. or switches back to pure C code
declare function cvUseOptimized( on_off as long ) as long

' Retrieves information about the registered modules and loaded optimized plugins
declare sub      cvGetModuleInfo( module_name as const zstring ptr, _
                                  version as const zstring ptr ptr, _
                                  loaded_addon_plugins as const zstring ptr ptr )

type CvAllocFunc as function CV_CDECL ( size as size_t, userdata as any ptr ) as any ptr
type CvFreeFunc as function  CV_CDECL ( pptr as any ptr, userdata as any ptr ) as long

' Set user-defined memory managment functions (substitutors for malloc and free)
' that will be called by cvAlloc, cvFree and higher-level functions (e.g. cvCreateImage)
declare sub      cvSetMemoryManager( alloc    as CvAllocFunc CV_DEFAULT(NULL), _
                                     free     as CvFreeFunc  CV_DEFAULT(NULL), _
                                     userdata as any ptr     CV_DEFAULT(NULL) )

type Cv_iplCreateImageHeader as function CV_STDCALL (as long,as long,as long, _
                                                     as zstring ptr,as zstring ptr, _
                                                     as long,as long,as long,as long,as long, _
                                                     as IplROI ptr, as IplImage ptr, as any ptr, as IplTileInfo ptr) as IplImage ptr
type Cv_iplAllocateImageData as sub      CV_STDCALL (as IplImage ptr, as long, as long)
type Cv_iplDeallocate        as sub      CV_STDCALL (as IplImage ptr,as long)
type Cv_iplCreateROI         as function CV_STDCALL (as long,as long,as long,as long,as long) as IplROI ptr
type Cv_iplCloneImage        as function CV_STDCALL (as const IplImage ptr) as IplImage ptr

' Makes OpenCV use IPL functions for IplImage allocation/deallocation
declare sub      cvSetIPLAllocators(create_header   as Cv_iplCreateImageHeader, _
                                    allocate_data   as Cv_iplAllocateImageData, _
                                    deallocate_data as Cv_iplDeallocate, _
                                    create_roi      as Cv_iplCreateROI, _
                                    clone_image     as Cv_iplCloneImage )

#define CV_TURN_ON_IPL_COMPATIBILITY() _
  cvSetIPLAllocators( iplCreateImageHeader, iplAllocateImage, iplDeallocate, iplCreateROI, iplCloneImage )

' ####################
' # Data Persistence #
' ####################

' High-level functions 

' opens existing or creates new file storage
declare function cvOpenFileStorage( filename   as const zstring ptr, _
                                    memstorage as CvMemStorage ptr, _
                                    flags      as long, _
                                    pencoding  as const zstring ptr CV_DEFAULT(NULL) ) as CvFileStorage ptr

' closes file storage and deallocates buffers
declare sub      cvReleaseFileStorage( fs as CvFileStorage ptr ptr )

' returns attribute value or 0 (NULL) if there is no such attribute
declare function cvAttrValue( attr as const CvAttrList ptr, attr_name as const zstring ptr) as const zstring ptr

' starts writing compound structure (map or sequence)
declare sub      cvStartWriteStruct( fs           as CvFileStorage ptr, _
                                     pname        as const zstring ptr, _
                                     struct_flags as long, _
                                     type_name    as const zstring ptr CV_DEFAULT(NULL), _
                                     attributes   as CvAttrList CV_DEFAULT(cv_AttrList()) )

' finishes writing compound structure
declare sub      cvEndWriteStruct( fs as CvFileStorage ptr )

' writes an integer
declare sub      cvWriteInt( fs as CvFileStorage ptr, pname as const zstring ptr, value as long )

' writes a floating-point number
declare sub      cvWriteReal( fas as CvFileStorage ptr, pname as const zstring ptr, value as double )

' writes a string
declare sub      cvWriteString( fs as CvFileStorage ptr, pname as const zstring ptr, _
                                pStr as const zstring ptr, quote as long CV_DEFAULT(0) )

' writes a comment
declare sub      cvWriteComment( fs as CvFileStorage ptr, comment as const zstring ptr, eol_comment as long )

' writes instance of a standard type (matrix, image, sequence, graph etc.) or user-defined type
declare sub      cvWrite( fs as CvFileStorage ptr, pname as const zstring ptr, pdata as const any ptr, _
                          attributes as CvAttrList CV_DEFAULT(cv_AttrList()) )

' starts the next stream
declare sub      cvStartNextStream( fs as CvFileStorage ptr )

' helper function: writes multiple integer or floating-point numbers
declare sub      cvWriteRawData( fs   as CvFileStorage ptr, _
                                 src  as const any ptr, _
                                 len_ as long, _
                                 dt   as const zstring ptr )

' returns the hash entry corresponding to the specified literal key string 
' or 0 if there is no such a key in the storage
declare function cvGetHashedKey( fs as CvFileStorage ptr, pname as const zstring ptr, _ 
                                 _len as long CV_DEFAULT(-1), _
                                 create_missing as long CV_DEFAULT(0) ) as CvStringHashNode ptr

' returns file node with the specified key within the specified map (collection of named nodes)
declare function cvGetRootFileNode( fs as const CvFileStorage ptr, _
                                    stream_index as long CV_DEFAULT(0) ) as CvFileNode ptr

' returns file node with the specified key within the specified map (collection of named nodes)
declare function cvGetFileNode( fs as CvFileStorage ptr, map as CvFileNode ptr, _
                                key as const CvStringHashNode ptr, _
                                create_missing as long CV_DEFAULT(0) ) as CvFileNode ptr

' this is a slower version of cvGetFileNode that takes the key as a literal string
declare function cvGetFileNodeByName( fs as const CvFileStorage ptr, _
                                      map as const CvFileNode ptr, _
                                      pname as const zstring ptr ) as CvFileNode ptr

function cvReadInt( node as const CvFileNode ptr, _
                    default_value as long CV_DEFAULT(0) ) as long
  return iif(node=NULL, default_value, _
         iif(CV_NODE_IS_INT(node->tag),node->data.i, _
         iif(CV_NODE_IS_REAL(node->tag),cvRound(node->data.f),&H7fffffff)))
end function

function cvReadIntByName( fs as const CvFileStorage ptr, map as const CvFileNode ptr, _
                          pname as const zstring ptr, default_value as long CV_DEFAULT(0) ) as long
  return cvReadInt( cvGetFileNodeByName( fs, map, pname ), default_value )
end function

function cvReadReal( node as const CvFileNode ptr, default_value as double CV_DEFAULT(0.0) ) as double
  return iif(node=NULL,default_value, _
         iif(CV_NODE_IS_INT(node->tag), node->data.i, _
         iif(CV_NODE_IS_REAL(node->tag), node->data.f, &H1e300)))
end function

function cvReadRealByName( fs as const CvFileStorage ptr, map as const CvFileNode ptr, _
                           pname as const zstring ptr, default_value as double CV_DEFAULT(0.0) ) as double
  return cvReadReal( cvGetFileNodeByName( fs, map, pname ), default_value )
end function

function cvReadString( node as const CvFileNode ptr, _
                       default_value as const zstring ptr CV_DEFAULT(NULL) ) as const zstring ptr
  return iif(node=NULL,default_value, iif(CV_NODE_IS_STRING(node->tag),node->data.str._ptr,0))
end function

function cvReadStringByName( fs as const CvFileStorage ptr, map as const CvFileNode ptr, _
                             pname as const zstring ptr, default_value as const zstring ptr CV_DEFAULT(NULL) ) as const zstring ptr
  return cvReadString( cvGetFileNodeByName( fs, map, pname ), default_value )
end function

' decodes standard or user-defined object and returns it
declare function cvRead( fs as CvFileStorage ptr, node as CvFileNode ptr, _
                         attributes as CvAttrList ptr CV_DEFAULT(NULL) ) as any ptr

' decodes standard or user-defined object and returns it
function cvReadByName( fs as CvFileStorage ptr, map as const CvFileNode ptr, _
                       pname as const zstring ptr, attributes as CvAttrList ptr CV_DEFAULT(NULL) ) as any ptr
  return cvRead( fs, cvGetFileNodeByName( fs, map, pname ), attributes )
end function

' starts reading data from sequence or scalar numeric node
declare sub      cvStartReadRawData( fs as const CvFileStorage ptr, src as const CvFileNode ptr, _
                                     reader as CvSeqReader ptr )

' reads multiple numbers and stores them to array
declare sub      cvReadRawDataSlice( fs as const CvFileStorage ptr, reader as CvSeqReader ptr, _
                                     count as long, dst as any ptr , dt as const zstring ptr )

' combination of two previous functions for easier reading of whole sequences
declare sub      cvReadRawData( fs as const CvFileStorage ptr, src as const CvFileNode ptr, _
                                dst as any ptr, dt as const zstring ptr )

' writes a copy of file node to file storage
declare sub      cvWriteFileNode( fs as CvFileStorage ptr, new_node_name as const zstring ptr, _
                                  node as const CvFileNode ptr, embed as long )

' returns name of file node
declare function cvGetFileNodeName( node as const CvFileNode ptr ) as const zstring ptr

' ####################
' # Adding own types #
' ####################
declare sub      cvRegisterType( infgo as const CvTypeInfo ptr )
declare sub      cvUnregisterType( type_name as const zstring ptr )
declare function cvFirstType() as CvTypeInfo ptr
declare function cvFindType( type_name as const zstring ptr ) as CvTypeInfo ptr
declare function cvTypeOf( struct_ptr as const any ptr ) as CvTypeInfo ptr

' universal functions
declare sub      cvRelease( struct_ptr as any ptr ptr )
declare function cvClone( struct_ptr as const any ptr ) as any ptr

' simple API for reading/writing data
declare sub      cvSave( filename as const zstring ptr,  struct_ptr as const any ptr, _
                         pname as const zstring ptr CV_DEFAULT(NULL), _
                         comment as const zstring ptr CV_DEFAULT(NULL), _
                         attributes as CvAttrList CV_DEFAULT(cv_AttrList()) )
                         
declare function cvLoad( filename as const zstring ptr, _
                         memstorage as CvMemStorage ptr CV_DEFAULT(NULL), _
                         pname as const zstring ptr CV_DEFAULT(NULL), _
                         real_name as const zstring ptr ptr CV_DEFAULT(NULL) ) as any ptr

' ############################
' # Measuring Execution Time #
' ############################

' helper functions for RNG initialization and accurate time measurement:
' uses internal clock counter on x86
declare function cvGetTickCount() as longint
declare function cvGetTickFrequency() as double

' CPU capabilities 
#define CV_CPU_NONE    0
#define CV_CPU_MMX     1
#define CV_CPU_SSE     2
#define CV_CPU_SSE2    3
#define CV_CPU_SSE3    4
#define CV_CPU_SSSE3   5
#define CV_CPU_SSE4_1  6
#define CV_CPU_SSE4_2  7
#define CV_CPU_POPCNT  8
#define CV_CPU_AVX    10
#define CV_CPU_AVX2   11
#define CV_HARDWARE_MAX_FEATURE 255

declare function cvCheckHardwareSupport( feature as long ) as long

' Multi-Threading

' retrieve/set the number of threads used in OpenMP implementations
declare function cvGetNumThreads() as long
declare sub  cvSetNumThreads( threads as long = 0 )
' get index of the thread being executed
declare function cvGetThreadNum() as long

' ##################
' # Error Handling #
' ##################

' Get current OpenCV error status
declare function cvGetErrStatus() as long

' Sets error status silently
declare sub  cvSetErrStatus( status as long )

#define CV_ErrModeLeaf     0 ' Print error and exit program
#define CV_ErrModeParent   1 ' Print error and continue
#define CV_ErrModeSilent   2 ' Don't print and continue

' Retrieves current error processing mode
declare function cvGetErrMode() as long

' Sets error processing mode, returns previously used mode
declare function cvSetErrMode( mode as long ) as long

' Sets error status and performs some additional actions
' (displaying message box, writing message to stderr, terminating application etc.)
' depending on the current error mode
declare sub cvError( status as long, func_name as const zstring ptr, _
                     err_msg as const zstring ptr, file_name as const zstring ptr, _line as long)

' Retrieves textual description of the error given its code
declare function cvErrorStr( status as long) as const zstring ptr

' Retrieves detailed information about the last error occurred
declare function cvGetErrInfo( errcode_desc as const zstring ptr ptr, _
                               description  as const zstring ptr ptr, _
                               filename     as const zstring ptr ptr, _
                               _line        as long ptr ) as long 

' Maps IPP error codes to the counterparts from OpenCV
declare function cvErrorFromIppStatus(ipp_status as long) as long

type CvErrorCallback as function cdecl ( status as long, func_name as const zstring ptr, _
                                         err_msg as const zstring ptr, _
                                         file_name as const zstring ptr, _
                                         _line as long, _
                                         userdata as any ptr) as long

' Assigns a new error-handling function
declare function cvRedirectError( error_handler as CvErrorCallback, _
                                  userdata as any ptr = NULL, _
                                  prev_userdata as any ptr ptr = NULL ) as CvErrorCallback

' Output nothing
declare function cvNulDevReport( status as long, func_name as const zstring ptr, err_msg  as const zstring ptr, _
                                 file_name as const zstring ptr, _line as long, userdata as any ptr) as long

' Output to console(fprintf(stderr,...))
declare function cvStdErrReport( status as long, func_name as const zstring ptr, err_msg as const zstring ptr, _
                                 file_name as const zstring ptr, _line as long, userdata as any ptr ) as long

' Output to MessageBox(WIN32)
declare function cvGuiBoxReport( status as long, func_name as const zstring ptr, err_msg as const zstring ptr, _
                                 file_name as const zstring ptr, _line as long, userdata as any ptr ) as long

end extern ' "C"
