#pragma once

#ifndef __OPENCV__
  #error 666: do not include this file manually include only 'opencv.bi' !
#endif

extern "C"

#define CV_ATTR_UNUSED
#define CV_ATTR_USED

#define CV_CDECL cdecl

#ifdef __FB_WIN32__
 #define CV_STDCALL stdcall
#else
 #define CV_STDCALL CV_CDECL
#endif

#define CV_DEFAULT(x) = x

#define CV_INLINE

#define CV_BIG_INT(n)   n##LL
#define CV_BIG_UINT(n)  n##ULL

' CvArr is used to pass arbitrary array-like data structures into functions 
' where the particular array type is recognized at runtime:
type CvArr as any

union Cv32suf
  as long   i
  as ulong  u
  as single f
end union

union Cv64suf
  as longint  i
  as ulongint u
  as double   f
end union

type CVStatus as long

enum 
 CV_StsOk=                        0, ' everithing is ok
 CV_StsBackTrace=                -1, ' pseudo error for back trace
 CV_StsError=                    -2, ' unknown /unspecified error
 CV_StsInternal=                 -3, ' internal error (bad state)
 CV_StsNoMem=                    -4, ' insufficient memory
 CV_StsBadArg=                   -5, ' function arg/param is bad
 CV_StsBadFunc=                  -6, ' unsupported function
 CV_StsNoConv=                   -7, ' iter. didn't converge
 CV_StsAutoTrace=                -8, ' tracing 
 CV_HeaderIsNull=                -9, ' image header is NULL
 CV_BadImageSize=               -10, ' image size is invalid
 CV_BadOffset=                  -11, ' offset is invalid
 CV_BadDataPtr=                 -12, 
 CV_BadStep=                    -13, 
 CV_BadModelOrChSeq=            -14, 
 CV_BadNumChannels=             -15, 
 CV_BadNumChannel1U=            -16, 
 CV_BadDepth=                   -17, 
 CV_BadAlphaChannel=            -18, 
 CV_BadOrder=                   -19, 
 CV_BadOrigin=                  -20, 
 CV_BadAlign=                   -21, 
 CV_BadCallBack=                -22, 
 CV_BadTileSize=                -23, 
 CV_BadCOI=                     -24, 
 CV_BadROISize=                 -25, 
 CV_MaskIsTiled=                -26, 
 CV_StsNullPtr=                 -27, ' null pointer
 CV_StsVecLengthErr=            -28, ' incorrect vector length
 CV_StsFilterStructContentErr=  -29, ' incorr. filter structure content
 CV_StsKernelStructContentErr=  -30, ' incorr. transform kernel content
 CV_StsFilterOffsetErr=         -31, ' incorrect filter offset value
 CV_StsBadSize=                -201, ' the input/output structure size is incorrect
 CV_StsDivByZero=              -202, ' division by zero
 CV_StsInplaceNotSupported=    -203, ' in-place operation is not supported
 CV_StsObjectNotFound=         -204, ' request can't be completed
 CV_StsUnmatchedFormats=       -205, ' formats of input/output arrays differ
 CV_StsBadFlag=                -206, ' flag is wrong or not supported
 CV_StsBadPoint=               -207, ' bad CvPoint
 CV_StsBadMask=                -208, ' bad format of mask (neither 8uC1 nor 8sC1)
 CV_StsUnmatchedSizes=         -209, ' sizes of input/output structures do not match
 CV_StsUnsupportedFormat=      -210, ' the data format/type is not supported by the function
 CV_StsOutOfRange=             -211, ' some of parameters are out of range
 CV_StsParseError=             -212, ' invalid syntax/structure of the parsed file
 CV_StsNotImplemented=         -213, ' the requested function/feature is not implemented
 CV_StsBadMemBlock=            -214, ' an allocated block has been corrupted
 CV_StsAssert=                 -215, ' assertion failed
 CV_GpuNotSupported=           -216,
 CV_GpuApiCallError=           -217,
 CV_OpenGlNotSupported=        -218,
 CV_OpenGlApiCallError=        -219,
 CV_OpenCLDoubleNotSupported=  -220,
 CV_OpenCLInitError=           -221,
 CV_OpenCLNoAMDBlasFft=        -222
end enum

' ######################################
' # Common macros and inline functions #
' ######################################

#define CV_PI   3.1415926535897932384626433832795
#define CV_LOG2 0.69314718055994530941723212145818

#define CV_SWAP(a,b,t) ((t) = (a) : (a) = (b) : (b) = (t))

#define CV_IMIN(a, b)  ((a) xor (((a) xor (b)) and (iif(((a) < (b),1,0)) - 1)))

#define CV_IMAX(a, b)  ((a) xor (((a) xor (b)) and (iif(((a) > (b),1,0)) - 1)))

#define CV_IABS(a)     (iif((a) xor ((a) < 0 , -1 , 0)) - iif((a) < 0 , -1 , 0))

#define  CV_CMP(a,b)    (iif((a) > (b),1,0) - iif((a) < (b),1,0))
#define  CV_SIGN(a)     CV_CMP((a),0)

function cvFloor(value as double) as long
  dim as long i = clng(value)
  return i - iif(i > value,1,0)
end function

function cvCeil(value as double) as long
  dim as long i = clng(value)
  return i + iif(i < value,1,0)
end function

function cvRound(value as single) as long
  return clng(value + iif (value >= 0 , 0.5f , -0.5f))
end function

#define cvInvSqrt(value) csng(1./sqr(value))
#define cvSqrt(value)  csng(sqrt(value))

function cvIsNaN( value as double ) as boolean
  dim as Cv64suf ieee754
  ieee754.f = value
  return (((culng(ieee754.u shr 32) and &H7fffffff) + iif(culng(ieee754.u)<>0,1,0)) > &H7ff00000)
end function

function cvIsInf( value as double ) as double
  dim as Cv64suf ieee754
  ieee754.f = value
  return ( ((culng(ieee754.u shr 32) and &H7fffffff) = &H7ff00000) andalso (culng(ieee754.u) = 0) )
end function

' ############################
' # Random number generation #
' ############################

type CvRNG as ulongint

#define CV_RNG_COEFF 4164903690U

' brief Initializes a random number generator state.

' The function initializes a random number generator and returns the state.
' The pointer to the state can be then passed to the cvRandInt, cvRandReal and cvRandArr functions.
' In the current implementation a multiply-with-carry generator is used.
' param seed 64-bit value used to initiate a random sequence
' renamed: cvRNG
function cv_RNG(seed as longint = -1) as CvRNG
  dim as CvRNG rng = iif(seed , culngint(seed), culngint(clngint(-1)))
  return rng
end function

' Returns a 32-bit unsigned integer and updates RNG.

' The function returns a uniformly-distributed random 32-bit unsigned integer and updates the RNG state.
' It is similar to the rand() function from the C runtime library, except that OpenCV functions
' always generates a 32-bit random number, regardless of the platform.
' param rng CvRNG state initialized by cvRNG.
function cvRandInt( rng as CvRNG ptr ) as ulong
  dim as ulongint temp = *rng
  temp = culngint(culng(temp*CV_RNG_COEFF) + (temp shr 32))
  *rng = temp
  return culng(temp)
end function

'  Returns a floating-point random number and updates RNG.

' The function returns a uniformly-distributed random floating-point number between 0 and 1 (1 is not included).
' param rng RNG state initialized by cvRNG
function cvRandReal( rng as CvRNG ptr ) as double
  return cvRandInt(rng)*2.3283064365386962890625e-10 ' 2^-32
end function  

' #########################
' # Image type (IplImage) #
' #########################

#ifndef HAVE_IPL

' The following definitions (until #endif) is an extract from IPL headers.'
' Copyright (c) 1995 Intel Corporation.

#define IPL_DEPTH_SIGN &H80000000

#define IPL_DEPTH_1U   1
#define IPL_DEPTH_8U   8
#define IPL_DEPTH_16U 16
#define IPL_DEPTH_32F 32

#define IPL_DEPTH_8S  (IPL_DEPTH_SIGN or  8)
#define IPL_DEPTH_16S (IPL_DEPTH_SIGN or 16)
#define IPL_DEPTH_32S (IPL_DEPTH_SIGN or 32)

#define IPL_DATA_ORDER_PIXEL  0
#define IPL_DATA_ORDER_PLANE  1

#define IPL_ORIGIN_TL 0
#define IPL_ORIGIN_BL 1

#define IPL_ALIGN_4BYTES   4
#define IPL_ALIGN_8BYTES   8
#define IPL_ALIGN_16BYTES 16
#define IPL_ALIGN_32BYTES 32

#define IPL_ALIGN_DWORD   IPL_ALIGN_4BYTES
#define IPL_ALIGN_QWORD   IPL_ALIGN_8BYTES

#define IPL_BORDER_CONSTANT  0
#define IPL_BORDER_REPLICATE 1
#define IPL_BORDER_REFLECT   2
#define IPL_BORDER_WRAP      3

type IplTileInfo as any

type _IplROI
  as long coi               ' 0 - no COI (all channels are selected), 1 - 0th channel is selected ...
  as long xOffset
  as long yOffset
  as long width
  as long height
end type
type IplROI as _IplROI

type _IplImage
  as long            nSize           ' sizeof(IplImage)
  as long            ID              ' version (=0)
  as long            nChannels       ' Most of OpenCV functions support 1,2,3 or 4 channels
  as long            alphaChannel    ' Ignored by OpenCV
  as long            depth           ' Pixel depth in bits: IPL_DEPTH_8U, IPL_DEPTH_8S, IPL_DEPTH_16S,
                                     ' IPL_DEPTH_32S, IPL_DEPTH_32F and IPL_DEPTH_64F are supported.
  as ubyte           colorModel(3)   ' Ignored by OpenCV
  as ubyte           channelSeq(3)   ' Ignored by OpenCV 
  as long            dataOrder       ' 0 - interleaved color channels, 1 - separate color channels.
                                     ' cvCreateImage can only create interleaved images
  as long            origin          ' 0 - top-left origin, 1 - bottom-left origin (Windows bitmaps style).
  as long            align           ' Alignment of image rows (4 or 8).
                                     ' OpenCV ignores it and uses widthStep instead.
  as long            width           ' Image width in pixels.
  as long            height          ' Image height in pixels.
  as _IplROI     ptr roi             ' Image ROI. If NULL, the whole image is selected.
  as _IplImage   ptr maskROI         ' Must be NULL.
  as any         ptr imageId         ' Must be NULL.
  as IplTileInfo ptr tileInfo        ' Must be NULL.
  as long            imageSize       ' Image data size in bytes =image->height * image->widthStep in case of interleaved data)
  as ubyte       ptr imageData       ' Pointer to aligned image data.
  as long            widthStep       ' Size of aligned image row in bytes.
  as long            BorderMode(3)   ' Ignored by OpenCV.
  as long            BorderConst(3)  ' Ignored by OpenCV.
  as ubyte       ptr imageDataOrigin ' Pointer to very origin of image data (not necessarily aligned) needed for correct deallocation '/
end type
type IplImage as _IplImage

type _IplConvKernel
  as long     nCols
  as long     nRows
  as long     anchorX
  as long     anchorY
  as long ptr values
  as long     nShiftR
end type
type IplConvKernel as _IplConvKernel 

type _IplConvKernelFP
  as long       nCols
  as long       nRows
  as long       anchorX
  as long       anchorY
  as single ptr values
end type
type IplConvKernelFP as _IplConvKernelFP

#define IPL_IMAGE_HEADER 1
#define IPL_IMAGE_DATA   2
#define IPL_IMAGE_ROI    4

#endif 'HAVE_IPL

' extra border mode
#define IPL_BORDER_REFLECT_101    4
#define IPL_BORDER_TRANSPARENT    5

#define IPL_IMAGE_MAGIC_VAL  clng(sizeof(IplImage))
#define CV_TYPE_NAME_IMAGE "opencv-image"

#define CV_IS_IMAGE_HDR(img) ((img <> NULL) andalso (cptr(as const IplImage ptr,img)->nSize = sizeof(IplImage)))

#define CV_IS_IMAGE(img) (CV_IS_IMAGE_HDR(img) andalso (cptr(const IplImage ptr,img)->imageData <> NULL)

' for storing as double-precision floating point data in IplImage's
#define IPL_DEPTH_64F  64

' get reference to pixel at (col,row), for multi-channel images (col) should be multiplied by number of channels
#define CV_IMAGE_ELEM( image, elemtype, row, col ) cptr(elemtype ptr,((image)->imageData + (image)->widthStep*(row)))([(col)])

' #######################
' # Matrix type (CvMat) #
' #######################
#define CV_CN_MAX     512
#define CV_CN_SHIFT   3
#define CV_DEPTH_MAX  (1 shl CV_CN_SHIFT)

#define CV_8U   0
#define CV_8S   1
#define CV_16U  2
#define CV_16S  3
#define CV_32S  4
#define CV_32F  5
#define CV_64F  6
#define CV_USRTYPE1 7

#define CV_MAT_DEPTH_MASK       (CV_DEPTH_MAX - 1)
#define CV_MAT_DEPTH(flags)     ((flags) and CV_MAT_DEPTH_MASK)

#define CV_MAKETYPE(depth,cn) (CV_MAT_DEPTH(depth) + (((cn)-1) shl CV_CN_SHIFT))
#define CV_MAKE_TYPE CV_MAKETYPE

#define CV_8UC1 CV_MAKETYPE(CV_8U,1)
#define CV_8UC2 CV_MAKETYPE(CV_8U,2)
#define CV_8UC3 CV_MAKETYPE(CV_8U,3)
#define CV_8UC4 CV_MAKETYPE(CV_8U,4)
#define CV_8UC(n) CV_MAKETYPE(CV_8U,(n))

#define CV_8SC1 CV_MAKETYPE(CV_8S,1)
#define CV_8SC2 CV_MAKETYPE(CV_8S,2)
#define CV_8SC3 CV_MAKETYPE(CV_8S,3)
#define CV_8SC4 CV_MAKETYPE(CV_8S,4)
#define CV_8SC(n) CV_MAKETYPE(CV_8S,(n))

#define CV_16UC1 CV_MAKETYPE(CV_16U,1)
#define CV_16UC2 CV_MAKETYPE(CV_16U,2)
#define CV_16UC3 CV_MAKETYPE(CV_16U,3)
#define CV_16UC4 CV_MAKETYPE(CV_16U,4)
#define CV_16UC(n) CV_MAKETYPE(CV_16U,(n))

#define CV_16SC1 CV_MAKETYPE(CV_16S,1)
#define CV_16SC2 CV_MAKETYPE(CV_16S,2)
#define CV_16SC3 CV_MAKETYPE(CV_16S,3)
#define CV_16SC4 CV_MAKETYPE(CV_16S,4)
#define CV_16SC(n) CV_MAKETYPE(CV_16S,(n))

#define CV_32SC1 CV_MAKETYPE(CV_32S,1)
#define CV_32SC2 CV_MAKETYPE(CV_32S,2)
#define CV_32SC3 CV_MAKETYPE(CV_32S,3)
#define CV_32SC4 CV_MAKETYPE(CV_32S,4)
#define CV_32SC(n) CV_MAKETYPE(CV_32S,(n))

#define CV_32FC1 CV_MAKETYPE(CV_32F,1)
#define CV_32FC2 CV_MAKETYPE(CV_32F,2)
#define CV_32FC3 CV_MAKETYPE(CV_32F,3)
#define CV_32FC4 CV_MAKETYPE(CV_32F,4)
#define CV_32FC(n) CV_MAKETYPE(CV_32F,(n))

#define CV_64FC1 CV_MAKETYPE(CV_64F,1)
#define CV_64FC2 CV_MAKETYPE(CV_64F,2)
#define CV_64FC3 CV_MAKETYPE(CV_64F,3)
#define CV_64FC4 CV_MAKETYPE(CV_64F,4)
#define CV_64FC(n) CV_MAKETYPE(CV_64F,(n))

#define CV_AUTO_STEP &H7fffffff
#define CV_WHOLE_ARR cv_Slice( 0, &H3fffffff )

#define CV_MAT_CN_MASK          ((CV_CN_MAX - 1) shl CV_CN_SHIFT)
#define CV_MAT_CN(flags)        ((((flags) and CV_MAT_CN_MASK) shr CV_CN_SHIFT) + 1)
#define CV_MAT_TYPE_MASK        (CV_DEPTH_MAX*CV_CN_MAX - 1)
#define CV_MAT_TYPE(flags)      ((flags) and CV_MAT_TYPE_MASK)
#define CV_MAT_CONT_FLAG_SHIFT  14
#define CV_MAT_CONT_FLAG        (1 shl CV_MAT_CONT_FLAG_SHIFT)
#define CV_IS_MAT_CONT(flags)   ((flags) and CV_MAT_CONT_FLAG)
#define CV_IS_CONT_MAT          CV_IS_MAT_CONT
#define CV_SUBMAT_FLAG_SHIFT    15
#define CV_SUBMAT_FLAG          (1 shl CV_SUBMAT_FLAG_SHIFT)
#define CV_IS_SUBMAT(flags)     ((flags) and CV_MAT_SUBMAT_FLAG)

#define CV_MAGIC_MASK    &HFFFF0000
#define CV_MAT_MAGIC_VAL &H42420000
#define CV_TYPE_NAME_MAT "opencv-matrix"

union _CvMatData
  as ubyte ptr _ptr ' renamed: ptr
  as short ptr  s
  as long ptr   i
  as single ptr fl
  as double ptr db
end union

type CvMat
  as long type
  as long step
  ' for internal use only
  as long ptr refcount
  as long hdr_refcount
  as _CvMatData data
  as long rows
  as long cols
end type

#define CV_IS_MAT_HDR(mat) _
    ( (mat) <> NULL andalso _
      (cptr(const CvMat ptr,mat)->type and CV_MAGIC_MASK = CV_MAT_MAGIC_VAL) andalso _
      (cptr(const CvMat ptr,mat)->cols>0 andalso cptr(const CvMat ptr,mat)->rows>0) )

#define CV_IS_MAT_HDR_Z(mat) _
    ( (mat) <> NULL andalso _
      (cptr(const CvMat ptr,mat)->type and CV_MAGIC_MASK = CV_MAT_MAGIC_VAL) andalso _
      (cptr(const CvMat ptr,mat)->cols>0 andalso cptr(const CvMat ptr,mat)->rows>0) )
#define CV_IS_MAT(mat) _
    (CV_IS_MAT_HDR(mat) andalso (cptr(const CvMat ptr,mat)->data._ptr<>NULL))

#define CV_IS_MASK_ARR(mat) _
    (((mat)->type and (CV_MAT_TYPE_MASK and not CV_8SC1)) = 0)
    
#define CV_ARE_TYPES_EQ(mat1, mat2) _
    ((((mat1)->type xor (mat2)->type) and CV_MAT_TYPE_MASK) = 0)

#define CV_ARE_CNS_EQ(mat1, mat2) _
    ((((mat1)->type xor (mat2)->type) and CV_MAT_CN_MASK) = 0)

#define CV_ARE_DEPTHS_EQ(mat1, mat2) _
    ((((mat1)->type xor (mat2)->type) and CV_MAT_DEPTH_MASK) = 0)

#define CV_ARE_SIZES_EQ(mat1, mat2) _
    ((mat1)->rows = (mat2)->rows andalso (mat1)->cols = (mat2)->cols)

#define CV_IS_MAT_CONST(mat) _
    (((mat)->rows or (mat)->cols) = 1)

' Size of each channel item
' &H124489 = 1000 0100 0100 0010 0010 0001 0001 ~ array of sizeof(arr_type_elem)
#define CV_ELEM_SIZE1(type) _
    ((((sizeof(size_t) shl 28) or &H8442211) or CV_MAT_DEPTH(type)*4) and 15)

' &H3a50 = 11 10 10 01 01 00 00 ~ array of log2(sizeof(arr_type_elem))
#define CV_ELEM_SIZE(type) _
    (CV_MAT_CN(type) shl ((((sizeof(size_t)/4+1)*16384 or &H3a50) shr CV_MAT_DEPTH(type)*2) and 3))

#define IPL2CV_DEPTH(depth) _
    ((((CV_8U)+(CV_16U shl 4)+(CV_32F shl 8) + (CV_64F shl 16) + (CV_8S shl 20) + _
      (CV_16S shl 24)+(CV_32S shl 28)) shr _
      ((((depth) & &HF0) shr 2) + iif(depth and IPL_DEPTH_SIGN, 20 , 0))) and 15)


' constructor: No data is allocated internally!!!
' (Use together with cvCreateData, or use cvCreateMat instead to get a matrix with allocated data):
' renamed: cvMat
function cv_Mat( rows as long, cols as long, _type as long, _data as any ptr CV_DEFAULT(NULL) ) as CvMat
  dim as CvMat m=any
  _type = CV_MAT_TYPE(_type)
  m.type = CV_MAT_MAGIC_VAL or CV_MAT_CONT_FLAG or _type
  m.cols = cols
  m.rows = rows
  m.step = m.cols*CV_ELEM_SIZE(_type)
  m.data._ptr = _data
  m.refcount = NULL
  m.hdr_refcount = 0
  return m
end function
 
 #define CV_MAT_ELEM_PTR_FAST( mat, row, col, pix_size ) _
    ( (mat).data._ptr + cast(size_t,(mat).step*(row) + (pix_size)*(col)) )

#define CV_MAT_ELEM_PTR( mat, row, col ) _
    CV_MAT_ELEM_PTR_FAST( mat, row, col, CV_ELEM_SIZE((mat).type) )

#define CV_MAT_ELEM( mat, elemtype, row, col ) _
    (*cptr( ementype ptr,CV_MAT_ELEM_PTR_FAST( mat, row, col, sizeof(elemtype))))

' cvmGet() Returns the particular element of single-channel floating-point matrix.
' The function is a fast replacement for cvGetReal2D 
' in the case of single-channel floating-point matrices. 
' It is faster because it does fewer checks for array type and array element type. 
' param mat: Input matrix
' param row: The zero-based index of row
' param col: The zero-based index of column
function  cvmGet( mat as const CvMat ptr, row as long, col as long) as double
  dim as long typ
  typ = CV_MAT_TYPE(mat->type)
  if( typ = CV_32FC1 ) then
    return cptr(single ptr,cptr(any ptr,mat->data._ptr + cast(size_t,mat->step*row)))[col] 
  else
    return cptr(double ptr,cptr(any ptr,mat->data._ptr + cast(size_t,mat->step*row)))[col]
  end if
end function
' cvmSet() Sets a specific element of a single-channel floating-point matrix.
' The function is a fast replacement for cvSetReal2D in the case of single-channel floating-point matrices.
' It is faster because it does fewer checks for array type and array element type.
' param mat  : The matrix
' param row  : The zero-based index of row
' param col  : The zero-based index of column
' param value: The new value of the matrix element
sub cvmSet( mat as CvMat ptr, row as long, col as long, value as double )
  dim as long  typ
  typ = CV_MAT_TYPE(mat->type)
  if( typ = CV_32FC1 ) then
   ' ((float*)(void*)(mat->data.ptr + (size_t)mat->step*row))[col] = (float)value;
    cptr(single ptr,cptr(any ptr,mat->data._ptr + cast(size_t,mat->step*row)))[col]=csng(value) 
  else
   ' (double*)(void*)(mat->data.ptr + (size_t)mat->step*row))[col] = value;
   cptr(double ptr,cptr(any ptr,mat->data._ptr + cast(size_t,mat->step*row)))[col] = value
  end if
end sub

function cvIplDepth(typ as long) as long
  dim as long depth = CV_MAT_DEPTH(typ)
  return CV_ELEM_SIZE1(depth)*8 or (iif(depth = CV_8S,1,0) or iif(depth = CV_16S,1,0) orelse _
           iif(depth = CV_32S , IPL_DEPTH_SIGN , 0) )
end function


' ###########################################
' # Multi-dimensional dense array (CvMatND) #
' ###########################################

#define CV_MATND_MAGIC_VAL     &H42430000
#define CV_TYPE_NAME_MATND    "opencv-nd-matrix"

#define CV_MAX_DIM      32
#define CV_MAX_DIM_HEAP 1024

union _CvMatNDData
  as ubyte  ptr _ptr ' renamed: ptr
  as single ptr fl
  as double ptr db
  as long   ptr i
  as short  ptr s
end union
type _CvMatNDDim
  as long size
  as long step
end type  
type CvMatND
  as long         type
  as long         dims
  as long     ptr refcount
  as long         hdr_refcount
  as _CvMatNDData data
  as _CvMatNDDim  dim(CV_MAX_DIM-1)
end type
#define CV_IS_MATND_HDR(mat) _
    ( (mat) <> NULL andalso (cptr(const CvMatND ptr,mat)->type and CV_MAGIC_MASK) = CV_MATND_MAGIC_VAL)

#define CV_IS_MATND(mat) _
    (CV_IS_MATND_HDR(mat) andalso (cptr(const CvMatND ptr,mat)->data._ptr <> NULL))


' ################################################
' # Multi-dimensional sparse array (CvSparseMat) #
' ################################################

#define CV_SPARSE_MAT_MAGIC_VAL &H42440000
#define CV_TYPE_NAME_SPARSE_MAT "opencv-sparse-matrix"

type _CvSet as CvSet

type CvSparseMat
  as long        type
  as long        dims
  as long    ptr refcount
  as long        hdr_refcount
  as _CvSet  ptr heap
  as any ptr ptr hashtable
  as long        hashsize
  as long        valoffset
  as long        idxoffset
  as long        size(CV_MAX_DIM-1)
end type

#define CV_IS_SPARSE_MAT_HDR(mat) _
    ((mat) <> NULL andalso _
    (cptr( const CvSparseMat ptr,mat)->type and CV_MAGIC_MASK) = CV_SPARSE_MAT_MAGIC_VAL)

#define CV_IS_SPARSE_MAT(mat) CV_IS_SPARSE_MAT_HDR(mat)

' iteration through a sparse array
type CvSparseNode
  as ulong            hashval
  as CvSparseNode ptr next
end type

type CvSparseMatIterator
  as CvSparseMat  ptr mat
  as CvSparseNode ptr node
  as long curidx
end type
#define CV_NODE_VAL(mat,node) ( cptr(any ptr,(cptr(ubyte ptr,node) + (mat)->valoffset) )
#define CV_NODE_IDX(mat,node) ( cptr(long ptr,(cptr(ubyte ptr,node) + (mat)->idxoffset) )

' #############
' # Histogram #
' #############

type CvHistType as long

#define CV_HIST_MAGIC_VAL      &H42450000
#define CV_HIST_UNIFORM_FLAG  (1 shl 10)

' indicates whether bin ranges are set already or not
#define CV_HIST_RANGES_FLAG   (1 shl 11)

#define CV_HIST_ARRAY         0
#define CV_HIST_SPARSE        1
#define CV_HIST_TREE          CV_HIST_SPARSE

' should be used as a parameter only, it turns to CV_HIST_UNIFORM_FLAG of hist->type
#define CV_HIST_UNIFORM       1

type CvHistogram
  as long           type
  as CvArr ptr      bins
  as single         thresh(CV_MAX_DIM-1,1)  ' For uniform histograms.
  as single ptr ptr thresh2                 ' For non-uniform histograms.
  as CvMatND mat                            ' Embedded matrix header for array histograms.
end type
#define CV_IS_HIST( hist ) _
    ((hist)<>NULL andalso _
     ((cptr( as CvHistogram ptr,hist)->type and CV_MAGIC_MASK) = CV_HIST_MAGIC_VAL andalso _
     (hist)->bins <> NULL)

#define CV_IS_UNIFORM_HIST( hist ) _
    (((hist)->type and CV_HIST_UNIFORM_FLAG) <> 0)

#define CV_IS_SPARSE_HIST( hist ) _
    CV_IS_SPARSE_MAT((hist)->bins)

#define CV_HIST_HAS_RANGES( hist ) _
    (((hist)->type and CV_HIST_RANGES_FLAG) <> 0)

' #############################################
' # Other supplementary data type definitions #
' #############################################

' ##########
' # CvRect #
' ##########
type CvRect
  as long x
  as long y
  as long width
  as long height
end type
' renamed: cvRect
function cv_Rect( x as long, y as long, _width as long, height as long ) as CvRect
  dim as CvRect r=any
  r.x = x
  r.y = y
  r.width = _width
  r.height = height
  return r
end function

function cvRectToROI( rect as CvRect, coi as long ) as IplROI
  dim as IplROI roi = any
  roi.xOffset = rect.x
  roi.yOffset = rect.y
  roi.width = rect.width
  roi.height = rect.height
  roi.coi = coi
  return roi
end function

function cvROIToRect( roi as IplROI ) as CvRect
  return cv_Rect( roi.xOffset, roi.yOffset, roi.width, roi.height )
end function

' ##################
' # CvTermCriteria #
' ##################

#define CV_TERMCRIT_ITER    1
#define CV_TERMCRIT_NUMBER  CV_TERMCRIT_ITER
#define CV_TERMCRIT_EPS     2

type CvTermCriteria
  as long   type    ' may be combination of CV_TERMCRIT_ITER CV_TERMCRIT_EPS
  as long   max_iter
  as double epsilon
end type
' renamed: cvTermCriteria
function cv_TermCriteria( _type as long, max_iter as long, epsilon as double ) as CvTermCriteria 
  dim as CvTermCriteria t = any
  t.type     = _type
  t.max_iter = max_iter
  t.epsilon  = epsilon
  return t
end function

' ########################
' # CvPoint and variants #
' ########################

type CvPoint
  as long x
  as long y
end type

' renamed: cvPoint
function cv_Point( x as long, y as long ) as CvPoint
  dim as CvPoint p = any
  p.x = x
  p.y = y
  return p
end function

type CvPoint2D32f
  as single x
  as single y
end type

' renamed: cvPoint2D32f
function cv_Point2D32f( x as double, y as double ) as CvPoint2D32f
  dim as CvPoint2D32f p = any
  p.x = x
  p.y = y
  return p
end function

function cvPointTo32f( _point as CvPoint ) as CvPoint2D32f
  return cv_Point2D32f( _point.x, _point.y )
end function

function cvPointFrom32f( _point as CvPoint2D32f ) as CvPoint 
  dim as CvPoint ipt = any
  ipt.x = cvRound(_point.x)
  ipt.y = cvRound(_point.y)
  return ipt
end function

type CvPoint3D32f
  as single x
  as single y
  as single z
end type
' renamed: cvPoint3D32f
function cv_Point3D32f( x as double, y as double, z as double ) as CvPoint3D32f
  dim as CvPoint3D32f p = any
  p.x = x
  p.y = y
  p.z = z
  return p
end function

type CvPoint2D64f
  as double x
  as double y
end type
' renamed: cvPoint2D64f
function cv_Point2D64f( x as double, y as double ) as CvPoint2D64f
  dim as CvPoint2D64f p = any
  p.x = x
  p.y = y
  return p
end function

type CvPoint3D64f
  as double x
  as double y
  as double z
end type
' renamed: cvPoint3D64f
function cv_Point3D64f( x as double, y as double, z as double ) as CvPoint3D64f
  dim as CvPoint3D64f p = any
  p.x = x
  p.y = y
  p.z = z
  return p
end function 

' ######################
' # CvSize's and CvBox #
' ######################

type CvSize
  as long width
  as long height
end type
' renamed: cvSize
function cv_Size( _width as long, height as long ) as CvSize
  dim as CvSize s = any
  s.width  = _width
  s.height = height
  return s
end function

type CvSize2D32f
  as single width
  as single height
end type
' renamed: cvSize2D32f
function cv_Size2D32f( _width as double, height as double ) as CvSize2D32f
  dim as CvSize2D32f s = any
  s.width  = _width
  s.height = height
  return s
end function

type CvBox2D
  as CvPoint2D32f center ' Center of the box
  as CvSize2D32f  size   ' Box width and length.
  as single angle        ' Angle between the horizontal axis and the first side (i.e. length) in degrees
end type

' Line iterator state:
type CvLineIterator
  ' Pointer to the current point:
  as ubyte ptr _ptr ' renamed: ptr
  ' Bresenham algorithm state:
  as long  err
  as long  plus_delta
  as long  minus_delta
  as long  plus_step
  as long  minus_step
end type


' ###########
' # CvSlice #
' ###########

type CvSlice
  as long  start_index, end_index
end type
' renamed: cv_Slice
function cv_Slice( start as long, end_ as long) as CvSlice
  dim as CvSlice slice = any
  slice.start_index = start
  slice.end_index   = end_
  return slice
end function
#define CV_WHOLE_SEQ_END_INDEX  &H3fffffff
#define CV_WHOLE_SEQ  cv_Slice(0, CV_WHOLE_SEQ_END_INDEX)

' ############
' # CvScalar #
' ############

type CvScalar
  as double val(2)
end type
' renamed: cvScalar
function cv_Scalar( val0 as double, val1 as double CV_DEFAULT(0), _
                    val2 as double CV_DEFAULT(0), val3 as double  CV_DEFAULT(0)) as CvScalar
  dim as CvScalar scalar = any
  scalar.val(0) = val0: scalar.val(1) = val1
  scalar.val(2) = val2: scalar.val(3) = val3
  return scalar
end function

function cvRealScalar( val0 as double ) as CvScalar
  dim as CvScalar scalar
  scalar.val(0) = val0
  return scalar
end function

function cvScalarAll( val0 as double ) as CvScalar
  dim as CvScalar scalar = any
  scalar.val(0) = val0: scalar.val(1) = val0
  scalar.val(2) = val0: scalar.val(3) = val0
  return scalar
end function

' ###########################
' # Dynamic Data structures #
' ###########################

' ##################
' # Memory storage #
' ##################
type CvMemBlock
  as CvMemBlock ptr prev
  as CvMemBlock ptr next
end type
#define CV_STORAGE_MAGIC_VAL &H42890000

type CvMemStorage
  as long             signature
  as CvMemBlock ptr   bottom     ' First allocated block.
  as CvMemBlock ptr   top        ' Current memory block - top of the stack.
  as CvMemStorage ptr parent     ' We get new blocks from parent as needed.
  as long             block_size ' Block size.
  as long             free_space ' Remaining free space in current block.
end type
#define CV_IS_STORAGE(storage) _
    ((storage) <> NULL andalso  _
     (cptr( as CvMemStorage ptr,storage)->signature and CV_MAGIC_MASK) = CV_STORAGE_MAGIC_VAL)

type CvMemStoragePos
  as CvMemBlock ptr top
  as long           free_space
end type

' ############
' # Sequence #
' ############
type CvSeqBlock
  as CvSeqBlock ptr prev        ' Previous sequence block.
  as CvSeqBlock ptr next        ' Next sequence block.
  as long           start_index ' Index of the first element in the block + sequence->first->start_index.
  as long           count       ' Number of elements in the block.
  as byte       ptr data        ' Pointer to the first element of the block.
end type

#macro CV_TREE_NODE_FIELDS(node_type)
  as long          flags       ' Miscellaneous flags.
  as long          header_size ' Size of sequence header.
  as node_type ptr h_prev      ' Previous sequence.
  as node_type ptr h_next      ' Next sequence.
  as node_type ptr v_prev      ' 2nd previous sequence.
  as node_type ptr v_next      ' 2nd next sequence.
#endmacro

' Read/Write sequence.
' Elements can be dynamically inserted to or deleted from the sequence.
#macro CV_SEQUENCE_FIELDS
  CV_TREE_NODE_FIELDS(CvSeq)
  as long             total       ' Total number of elements.
  as long             elem_size   ' Size of sequence element in bytes.
  as byte         ptr block_max   ' Maximal bound of the last block.
  as byte         ptr _ptr        ' renamed: ptr Current write pointer.
  as long             delta_elems ' Grow seq this many at a time.
  as CvMemStorage ptr storage     ' Where the seq is stored.
  as CvSeqBlock   ptr free_blocks ' Free blocks list.
  as CvSeqBlock   ptr first       ' Pointer to the first sequence block.
#endmacro

type CvSeq
  CV_SEQUENCE_FIELDS
end type

#define CV_TYPE_NAME_SEQ             "opencv-sequence"
#define CV_TYPE_NAME_SEQ_TREE        "opencv-sequence-tree"

' #######
' # Set #
' #######
' Order is not preserved. There can be gaps between sequence elements.
' After the element has been inserted it stays in the same place all the time.
' The MSB(most-significant or sign bit) of the first field (flags) is 0 iff the element exists.
#macro CV_SET_ELEM_FIELDS(elem_type)
  as long          flags
  as elem_type ptr next_free
#endmacro
type CvSetElem
  CV_SET_ELEM_FIELDS(CvSetElem)
end type

#macro CV_SET_FIELDS
  CV_SEQUENCE_FIELDS
  as CvSetElem ptr free_elems
  as long          active_count
#endmacro

type CvSet
  CV_SET_FIELDS
end type
#define CV_SET_ELEM_IDX_MASK   ( (1 shl 26) - 1 )
#define CV_SET_ELEM_FREE_FLAG  ( 1 shl ( sizeof(long)*8-1 ) )

' Checks whether the element pointed by ptr belongs to a set or not
#define CV_IS_SET_ELEM(_ptr)  (cptr(CvSetElem ptr,_ptr)->flags >= 0)

' #########
' # Graph #
' #########

' We represent a graph as a set of vertices.
' Vertices contain their adjacency lists (more exactly, pointers to first incoming or
' outcoming edge (or 0 if isolated vertex)). Edges are stored in another set.
' There is a singly-linked list of incoming/outcoming edges for each vertex.

' Each edge consists of

'    o   Two pointers to the starting and ending vertices
'        (vtx[0] and vtx[1] respectively).

' A graph may be oriented or not. In the latter case, edges between
' vertex i to vertex j are not distinguished during search operations.

'    o   Two pointers to next edges for the starting and ending vertices, where
'         next[0] points to the next edge in the vtx[0] adjacency list and
'         next[1] points to the next edge in the vtx[1] adjacency list.

type _CvGraphVtx as CvGraphVtx

#macro CV_GRAPH_EDGE_FIELDS
  as long            flags
  as single          weight
  as CvGraphEdge ptr next(1)
  as _CvGraphVtx ptr vtx(1)
#endmacro

#macro CV_GRAPH_VERTEX_FIELDS
  as long flags
  as CvGraphEdge ptr first
#endmacro

type CvGraphEdge
  CV_GRAPH_EDGE_FIELDS
end type

type CvGraphVtx
  CV_GRAPH_VERTEX_FIELDS
end type

type vGraphVtx2D
  CV_GRAPH_VERTEX_FIELDS
  as CvPoint2D32f ptr _ptr ' renamed: ptr
end type

' Graph is "derived" from the set (this is set a of vertices) and includes another set (edges)
#macro CV_GRAPH_FIELDS
  CV_SET_FIELDS
  as CvSet ptr edges
#endmacro  
type CvGraph
  CV_GRAPH_FIELDS
end type

#define CV_TYPE_NAME_GRAPH "opencv-graph"

' ##################
' # Chain/Countour #
' ##################

type CvChain
  CV_SEQUENCE_FIELDS
  as CvPoint  origin
end type

#macro CV_CONTOUR_FIELDS
  CV_SEQUENCE_FIELDS
  as CvRect rect
  as long   color
  as long   reserved(2)
#endmacro

type CvContour
  CV_CONTOUR_FIELDS
end type
type CvPoint2DSeq as CvContour

' ##################
' # Sequence types #
' ##################

#define CV_SEQ_MAGIC_VAL &H42990000
#define CV_IS_SEQ(seq) _
    ((seq)<>NULL andalso ((cptr(CvSeq ptr,seq)->flags and CV_MAGIC_MASK) = CV_SEQ_MAGIC_VAL)

#define CV_SET_MAGIC_VAL &H42980000
#define CV_IS_SET(set) _
    ((set)<>NULL andalso ((cptr(CvSeq ptr,set)->flags and CV_MAGIC_MASK) = CV_SET_MAGIC_VAL)

#define CV_SEQ_ELTYPE_BITS           12
#define CV_SEQ_ELTYPE_MASK           ((1 shl CV_SEQ_ELTYPE_BITS) - 1)

#define CV_SEQ_ELTYPE_POINT          CV_32SC2  ' (x,y)
#define CV_SEQ_ELTYPE_CODE           CV_8UC1   ' freeman code: 0..7
#define CV_SEQ_ELTYPE_GENERIC        0
#define CV_SEQ_ELTYPE_PTR            CV_MAKE_TYPE(CV_8U,8)
#define CV_SEQ_ELTYPE_PPOINT         CV_SEQ_ELTYPE_PTR  ' @(x,y) 
#define CV_SEQ_ELTYPE_INDEX          CV_32SC1  ' #(x,y)
#define CV_SEQ_ELTYPE_GRAPH_EDGE     0  ' &next_o, &next_d, &vtx_o, &vtx_d
#define CV_SEQ_ELTYPE_GRAPH_VERTEX   0  ' first_edge, &(x,y)
#define CV_SEQ_ELTYPE_TRIAN_ATR      0  ' vertex of the binary tree
#define CV_SEQ_ELTYPE_CONNECTED_COMP 0  ' connected component
#define CV_SEQ_ELTYPE_POINT3D CV_32FC3  ' (x,y,z)

#define CV_SEQ_KIND_BITS        2
#define CV_SEQ_KIND_MASK        ( ( (1 shl CV_SEQ_KIND_BITS) - 1 ) shl CV_SEQ_ELTYPE_BITS)

' types of sequences
#define CV_SEQ_KIND_GENERIC     (0 shl CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_CURVE       (1 shl CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_BIN_TREE    (2 shl CV_SEQ_ELTYPE_BITS)

' types of sparse sequences (sets)
#define CV_SEQ_KIND_GRAPH       (1 shl CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_SUBDIV2D    (2 shl CV_SEQ_ELTYPE_BITS)

#define CV_SEQ_FLAG_SHIFT       (CV_SEQ_KIND_BITS + CV_SEQ_ELTYPE_BITS)

' flags for curves
#define CV_SEQ_FLAG_CLOSED     (1 shl CV_SEQ_FLAG_SHIFT)
#define CV_SEQ_FLAG_SIMPLE     CV_SEQ_FLAG_SHIFT
#define CV_SEQ_FLAG_CONVEX     CV_SEQ_FLAG_SHIFT
#define CV_SEQ_FLAG_HOLE       (2 shl CV_SEQ_FLAG_SHIFT)

' flags for graphs
#define CV_GRAPH_FLAG_ORIENTED (1 shl CV_SEQ_FLAG_SHIFT)

#define CV_GRAPH               CV_SEQ_KIND_GRAPH
#define CV_ORIENTED_GRAPH      (CV_SEQ_KIND_GRAPH or CV_GRAPH_FLAG_ORIENTED)

' point sets
#define CV_SEQ_POINT_SET       (CV_SEQ_KIND_GENERIC or  CV_SEQ_ELTYPE_POINT)
#define CV_SEQ_POINT3D_SET     (CV_SEQ_KIND_GENERIC or  CV_SEQ_ELTYPE_POINT3D)
#define CV_SEQ_POLYLINE        (CV_SEQ_KIND_CURVE  or CV_SEQ_ELTYPE_POINT)
#define CV_SEQ_POLYGON         (CV_SEQ_FLAG_CLOSED or CV_SEQ_POLYLINE )
#define CV_SEQ_CONTOUR         CV_SEQ_POLYGON
#define CV_SEQ_SIMPLE_POLYGON  (CV_SEQ_FLAG_SIMPLE or CV_SEQ_POLYGON  )

' chain-coded curves
#define CV_SEQ_CHAIN           (CV_SEQ_KIND_CURVE  or CV_SEQ_ELTYPE_CODE)
#define CV_SEQ_CHAIN_CONTOUR   (CV_SEQ_FLAG_CLOSED or CV_SEQ_CHAIN)

' binary tree for the contour
#define CV_SEQ_POLYGON_TREE    (CV_SEQ_KIND_BIN_TREE or CV_SEQ_ELTYPE_TRIAN_ATR)

' sequence of the connected components
#define CV_SEQ_CONNECTED_COMP  (CV_SEQ_KIND_GENERIC or CV_SEQ_ELTYPE_CONNECTED_COMP)

' sequence of the integer numbers
#define CV_SEQ_INDEX           (CV_SEQ_KIND_GENERIC or CV_SEQ_ELTYPE_INDEX)

#define CV_SEQ_ELTYPE( seq )   ((seq)->flags and CV_SEQ_ELTYPE_MASK)
#define CV_SEQ_KIND( seq )     ((seq)->flags and CV_SEQ_KIND_MASK )

' flag checking
#define CV_IS_SEQ_INDEX( seq )      ( (CV_SEQ_ELTYPE(seq) = CV_SEQ_ELTYPE_INDEX) andalso (CV_SEQ_KIND(seq) = CV_SEQ_KIND_GENERIC) )

#define CV_IS_SEQ_CURVE( seq )      (CV_SEQ_KIND(seq) = CV_SEQ_KIND_CURVE)
#define CV_IS_SEQ_CLOSED( seq )     (((seq)->flags and CV_SEQ_FLAG_CLOSED) <> 0)
#define CV_IS_SEQ_CONVEX( seq )     0
#define CV_IS_SEQ_HOLE( seq )       (((seq)->flags and CV_SEQ_FLAG_HOLE) <> 0)
#define CV_IS_SEQ_SIMPLE( seq )     1

' type checking macros
#define CV_IS_SEQ_POINT_SET( seq ) _
    ((CV_SEQ_ELTYPE(seq) = CV_32SC2 orelse CV_SEQ_ELTYPE(seq) = CV_32FC2))

#define CV_IS_SEQ_POINT_SUBSET( seq ) _
    (CV_IS_SEQ_INDEX( seq ) orelse CV_SEQ_ELTYPE(seq) = CV_SEQ_ELTYPE_PPOINT)

#define CV_IS_SEQ_POLYLINE( seq ) _
    (CV_SEQ_KIND(seq) orelse CV_SEQ_KIND_CURVE andalso CV_IS_SEQ_POINT_SET(seq))

#define CV_IS_SEQ_POLYGON( seq ) _
    (CV_IS_SEQ_POLYLINE(seq) andalso CV_IS_SEQ_CLOSED(seq))

#define CV_IS_SEQ_CHAIN( seq ) _
    (CV_SEQ_KIND(seq) = CV_SEQ_KIND_CURVE andalso (seq)->elem_size = 1)

#define CV_IS_SEQ_CONTOUR( seq ) _
    (CV_IS_SEQ_CLOSED(seq) andalso (CV_IS_SEQ_POLYLINE(seq) orelse CV_IS_SEQ_CHAIN(seq)))

#define CV_IS_SEQ_CHAIN_CONTOUR( seq ) _
    (CV_IS_SEQ_CHAIN( seq ) andalso CV_IS_SEQ_CLOSED( seq ))

#define CV_IS_SEQ_POLYGON_TREE( seq ) _
    ( (CV_SEQ_ELTYPE (seq) = CV_SEQ_ELTYPE_TRIAN_ATR) andalso _
      (CV_SEQ_KIND( seq ) = CV_SEQ_KIND_BIN_TREE) )

#define CV_IS_GRAPH( seq ) _
    (CV_IS_SET(seq) andalso CV_SEQ_KIND(cptr(CvSet ptr, seq) = CV_SEQ_KIND_GRAPH)

#define CV_IS_GRAPH_ORIENTED( seq ) _
    (((seq)->flags and CV_GRAPH_FLAG_ORIENTED) <> 0)

#define CV_IS_SUBDIV2D( seq )  _
    (CV_IS_SET(seq) andalso CV_SEQ_KIND(cptr(CvSet ptr,seq) = CV_SEQ_KIND_SUBDIV2D)

' ##############################
' # Sequence writer and reader #
' ##############################

#macro CV_SEQ_WRITER_FIELDS
  as long           header_size
  as CvSeq      ptr seq         ' the sequence written
  as CvSeqBlock ptr block       ' current block
  as byte       ptr _ptr        ' renamed: ptr pointer to free space
  as byte       ptr block_min   ' pointer to the beginning of block
  as byte       ptr block_max   ' pointer to the end of block
#endmacro
type CvSeqWriter
  CV_SEQ_WRITER_FIELDS
end type

#macro CV_SEQ_READER_FIELDS
  as long           header_size
  as CvSeq      ptr seq         ' sequence, beign read
  as CvSeqBlock ptr block       ' current block
  as byte       ptr _ptr        ' renamed: ptr pointer to element be read next
  as byte       ptr block_min   ' pointer to the beginning of block
  as byte       ptr block_max   ' pointer to the end of block
  as long           delta_index ' = seq->first->start_index
  as byte       ptr prev_elem   ' pointer to previous element
#endmacro
type CvSeqReader
  CV_SEQ_READER_FIELDS
end type

' ###########################
' # Operations on sequences #
' ###########################

#define CV_SEQ_ELEM( seq, elem_type, index ) _
( iif(cptr(elem_type ptr,seq)->first andalso cuint(index) < cuint((seq)->first->count), _
    (seq)->first->data + (index) * sizeof(elem_type) , _
    cvGetSeqElem( cptr(CvSeq ptr,seq), (index) )) )

#define CV_GET_SEQ_ELEM( elem_type, seq, index ) CV_SEQ_ELEM( (seq), elem_type, (index) )

' Add element to sequence:
#macro CV_WRITE_SEQ_ELEM_VAR( elem_ptr, writer ) 
  if( (writer)._ptr >= (writer).block_max ) then
    cvCreateSeqBlock(@writer)
  end if
  memcpy((writer)._ptr, elem_ptr, (writer).seq->elem_size)
  (writer).ptr += (writer).seq->elem_size
#endmacro

#macro CV_WRITE_SEQ_ELEM( elem, writer )
  if( (writer).ptr >= (writer).block_max ) then
   cvCreateSeqBlock(@qwriter)
  end if
  memcpy((writer)._ptr, @(elem), sizeof(elem))
 (writer).ptr += sizeof(elem)
#endmacro

' Move reader position forward:
#macro CV_NEXT_SEQ_ELEM( elem_size, reader )
  (reader)._ptr += (elem_size)
  if ( (reader)._ptr >= (reader).block_max ) then
    cvChangeSeqBlock( @(reader), 1 )                     
  end if
#endmacro

' Move reader position backward:
#macro CV_PREV_SEQ_ELEM( elem_size, reader )
  (reader)._ptr -= (elem_size)
  if ( (reader)._ptr < (reader).block_min ) then
    cvChangeSeqBlock( @(reader), -1 )
  end if
#endmacro

' Read element and move read position forward:
#macro CV_READ_SEQ_ELEM( elem, reader )
  memcpy(  @(elem), (reader)._ptr, sizeof((elem)))
  CV_NEXT_SEQ_ELEM( sizeof(elem), reader )
#endmacro

' Read element and move read position backward:
#macro CV_REV_READ_SEQ_ELEM( elem, reader )
  memcpy( @(elem), (reader).ptr, sizeof((elem)))
  CV_PREV_SEQ_ELEM( sizeof(elem), reader )
#endmacro


#macro CV_READ_CHAIN_POINT( _pt, reader )
  (_pt) = (reader).pt
  if( (reader)._ptr ) then
    CV_READ_SEQ_ELEM( (reader).code, (reader))
    (reader).pt.x += (reader).deltas(clng((reader).code),0)
    (reader).pt.y += (reader).deltas(clng((reader).code),1)
  end if
#endmacro

#define CV_CURRENT_POINT( reader ) *cptr(CvPoint ptr,(reader)._ptr)
#define CV_PREV_POINT( reader )    *cptr(CvPoint ptr,(reader).prev_elem)


' ################
' # Graph macros #
' ################

' Return next graph edge for given vertex:
#define  CV_NEXT_GRAPH_EDGE( edge, vertex ) (edge)->next((edge)->vtx(1) = (vertex)))

' #######################################################################
' # Data structures for persistence (a.k.a serialization) functionality #
' #######################################################################

' "black box" file storage
type CvFileStorage as any

' Storage flags:
#define CV_STORAGE_READ          0
#define CV_STORAGE_WRITE         1
#define CV_STORAGE_WRITE_TEXT    CV_STORAGE_WRITE
#define CV_STORAGE_WRITE_BINARY  CV_STORAGE_WRITE
#define CV_STORAGE_APPEND        2
#define CV_STORAGE_MEMORY        4
#define CV_STORAGE_FORMAT_MASK   (7 shl 3)
#define CV_STORAGE_FORMAT_AUTO   0
#define CV_STORAGE_FORMAT_XML    8
#define CV_STORAGE_FORMAT_YAML  16

' List of attributes:
type CvAttrList
  as const zstring ptr ptr attr ' NULL-terminated array of (attribute_name,attribute_value) pairs.
  as CvAttrList        ptr next ' Pointer to next chunk of the attributes list.
end type
' renamed: cvAttrList
function cv_AttrList( ppAttr as const zstring ptr ptr CV_DEFAULT(NULL), _
                      pNext  as CvAttrList ptr CV_DEFAULT(NULL) ) as CvAttrList
  dim as CvAttrList l = any
  l.attr = ppAttr
  l.next = pNext
  return l
end function

type _CvTypeInfo as CvTypeInfo

#define CV_NODE_NONE      0
#define CV_NODE_INT       1
#define CV_NODE_INTEGER   CV_NODE_INT
#define CV_NODE_REAL      2
#define CV_NODE_FLOAT     CV_NODE_REAL
#define CV_NODE_STR       3
#define CV_NODE_STRING    CV_NODE_STR
#define CV_NODE_REF       4 ' not used
#define CV_NODE_SEQ       5
#define CV_NODE_MAP       6
#define CV_NODE_TYPE_MASK 7

#define CV_NODE_TYPE(flags)  ((flags) and CV_NODE_TYPE_MASK)

' file node flags
#define CV_NODE_FLOW   8  ' Used only for writing structures in YAML format.
#define CV_NODE_USER  16
#define CV_NODE_EMPTY 32
#define CV_NODE_NAMED 64

#define CV_NODE_IS_INT(flags)        (CV_NODE_TYPE(flags) = CV_NODE_INT)
#define CV_NODE_IS_REAL(flags)       (CV_NODE_TYPE(flags) = CV_NODE_REAL)
#define CV_NODE_IS_STRING(flags)     (CV_NODE_TYPE(flags) = CV_NODE_STRING)
#define CV_NODE_IS_SEQ(flags)        (CV_NODE_TYPE(flags) = CV_NODE_SEQ)
#define CV_NODE_IS_MAP(flags)        (CV_NODE_TYPE(flags) = CV_NODE_MAP)
#define CV_NODE_IS_COLLECTION(flags) (CV_NODE_TYPE(flags) >= CV_NODE_SEQ)
#define CV_NODE_IS_FLOW(flags)       ( ((flags) and CV_NODE_FLOW)  <> 0)
#define CV_NODE_IS_EMPTY(flags)      ( ((flags) and CV_NODE_EMPTY) <> 0)
#define CV_NODE_IS_USER(flags)       ( ((flags) and CV_NODE_USER)  <> 0)
#define CV_NODE_HAS_NAME(flags)      ( ((flags) and CV_NODE_NAMED) <> 0)

#define CV_NODE_SEQ_SIMPLE 256
#define CV_NODE_SEQ_IS_SIMPLE(seq) ( ((seq)->flags and CV_NODE_SEQ_SIMPLE) <> 0 )

type CvString
  as long        len
  as zstring ptr _ptr ' renamed: ptr
end type

' All the keys (names) of elements in the readed file storage
' are stored in the hash to speed up the lookup operations:
type CvStringHashNode
  as ulong                hashval
  as CvString             str
  as CvStringHashNode ptr next
end type

type CvFileNodeHash as CvGenericHash

' Basic element of the file storage - scalar or collection:
union _CvFileNodeData
  as double             f     ' scalar floating-point number
  as long               i     ' scalar integer number
  as CvString             str ' text string
  as CvSeq            ptr seq ' sequence (ordered collection of file nodes)
  as CvFileNodeHash   ptr map ' map (collection of named file nodes)
end union
type CvFileNode
  as long                 tag
  as _CvTypeInfo      ptr info ' type information (only for user-defined object, for others it is 0)
  as _CvFileNodeData      data
end type

type CvIsInstanceFunc as function CV_CDECL (pStruct  as const any ptr) as long
type CvReleaseFunc    as sub      CV_CDECL (ppStruct as any ptr ptr)
type CvReadFunc       as function CV_CDECL (storage as CvFileStorage ptr, node as CvFileNode ptr) as any ptr
type CvWriteFunc      as sub      CV_CDECL (storage as CvFileStorage ptr, name_ as const zstring ptr, pStruct as const any ptr, attributes as CvAttrList)
type CvCloneFunc      as function CV_CDECL (pStruct as const any ptr) as any ptr 

type CvTypeInfo
  as long                 flags
  as long                 header_size
  as CvTypeInfo       ptr prev
  as CvTypeInfo       ptr next
  as const zstring    ptr type_name
  as CvIsInstanceFunc     is_instance
  as CvReleaseFunc        release
  as CvReadFunc           read
  as CvWriteFunc          write
  as CvCloneFunc          clone
end type

'  System data types
type CvPluginFuncInfo
  as any          ptr ptr func_addr
  as any              ptr default_func_addr
  as const zstring    ptr func_names
  as long                 search_modules
  as long                 loaded_from
end type

type CvModuleInfo
  as CvModuleInfo     ptr next
  as const zstring    ptr name
  as const zstring    ptr version
  as CvPluginFuncInfo ptr func_tab
end type

end extern ' "C"
