#pragma once

#ifndef __OPENCV__
  #error 666: do not include this file manually include only 'opencv.bi' !
#endif

extern "C"

' #######################
' # Basic GUI functions #
' #######################

' For font
enum 
  CV_FONT_LIGHT           = 25, ' QFont::Light,
  CV_FONT_NORMAL          = 50, ' QFont::Normal,
  CV_FONT_DEMIBOLD        = 63, ' QFont::DemiBold,
  CV_FONT_BOLD            = 75, ' QFont::Bold,
  CV_FONT_BLACK           = 87  ' QFont::Black
end enum

enum 
  CV_STYLE_NORMAL         = 0, ' QFont::StyleNormal,
  CV_STYLE_ITALIC         = 1, ' QFont::StyleItalic,
  CV_STYLE_OBLIQUE        = 2  ' QFont::StyleOblique
end enum

' for color cvScalar(blue, green, red [, alpha_component])
' and alpha= 0 <-> &HFF (not transparent <-> transparent)
declare function cvFontQt(nameFont  as const zstring ptr, _
                          pointSize as long CV_DEFAULT(-1), _
                          colour    as CvScalar CV_DEFAULT(cvScalarAll(0)), _
                          weight    as long CV_DEFAULT(CV_FONT_NORMAL), _
                          style     as long CV_DEFAULT(CV_STYLE_NORMAL), _
                          spacing   as long CV_DEFAULT(0)) as CvFont

declare sub      cvAddText(image as const CvArr ptr, _
                           text  as const zstring ptr, _
                           org   as CvPoint, _
                           font  as CvFont ptr)

declare sub      cvDisplayOverlay  ( window_name   as const zstring ptr, _
                                     text    as const zstring ptr, _
                                     delayms as long CV_DEFAULT(0) )
declare sub      cvDisplayStatusBar( window_name   as const zstring ptr, _
                                     text    as const zstring ptr, _
                                     delayms as long CV_DEFAULT(0) )

declare sub      cvSaveWindowParameters( window_name as const zstring ptr )
declare sub      cvLoadWindowParameters( window_name as const zstring ptr )
declare function cvStartLoop( pt2Func as function (argc as long, argv as zstring ptr ptr) as long, _
                              argc as long, argv as zstring ptr ptr ) as long
declare sub      cvStopLoop()


#ifdef HAVE_QT
type CvButtonCallback as sub CV_CDECL ( state as long, userdata as any ptr )
enum 
  CV_PUSH_BUTTON = 0, 
  CV_CHECKBOX    = 1, 
  CV_RADIOBOX    = 2
end enum  
declare function cvCreateButton( button_name  as const zstring ptr CV_DEFAULT(NULL), _
                                 callback     as CvButtonCallback CV_DEFAULT(NULL), _
                                 userdata     as any ptr CV_DEFAULT(NULL), _
                                 button_type  as long CV_DEFAULT(CV_PUSH_BUTTON), _
                                 button_state as long CV_DEFAULT(0)) as long
#endif

#ifndef HAVE_WIN32UI ' Linux only
' this function is used to set some external parameters in case of X Window
declare function cvInitSystem( argc as long, argv as zstring ptr ptr ) as long
#endif

#ifdef HAVE_GTHREAD
declare function cvStartWindowThread() as long
#endif

enum
  ' These flags are used by cvSet/GetWindowProperty
  CV_WND_PROP_FULLSCREEN  = 0 ' to change/get window's fullscreen property
  CV_WND_PROP_AUTOSIZE    = 1 ' to change/get window's autosize property
  CV_WND_PROP_ASPECTRATIO = 2 ' to change/get window's aspectratio property
#ifdef HAVE_GL  
  CV_WND_PROP_OPENGL      = 3 ' to change/get window's opengl support
#endif
  ' These flags are used by cvNamedWindow and cvSet/GetWindowProperty
  CV_WINDOW_NORMAL        = &H00000000 ' the user can resize the window (no constraint)  / also use to switch a fullscreen window to a normal size
  CV_WINDOW_AUTOSIZE      = &H00000001 ' the user cannot resize the window, the size is constrainted by the image displayed

#ifdef HAVE_GL
  CV_WINDOW_OPENGL        = &H00001000 ' window with opengl support
#endif

#ifdef HAVE_QT
  CV_GUI_EXPANDED         = &H00000000 ' status bar and tool bar
  CV_GUI_NORMAL           = &H00000010 ' old fashious way
#endif

  ' These flags are used by cvNamedWindow and cvSet/GetWindowProperty
  CV_WINDOW_FULLSCREEN    = &H00000001 ' change the window to fullscreen
  CV_WINDOW_FREERATIO     = &H00000100 ' the image expends as much as it can (no ratio constraint)
  CV_WINDOW_KEEPRATIO     = &H00000000 ' the ration image is respected.
end enum

' create window
declare function cvNamedWindow( window_name as const zstring ptr, _
                                flags as long CV_DEFAULT(CV_WINDOW_AUTOSIZE) ) as long

' Set and Get Property of the window
declare sub      cvSetWindowProperty( window_name as const zstring ptr, prop_id as long, prop_value as double)
declare function cvGetWindowProperty( window_name as const zstring ptr, prop_id as long) as double

' display image within window (highgui windows remember their content)
declare sub      cvShowImage( window_name as const zstring ptr, image as const CvArr ptr )

' resize/move window
declare sub      cvResizeWindow( window_name as const zstring ptr, _width as long, height as long)
declare sub      cvMoveWindow( window_name as const zstring ptr, x as long, y as long )


' get native window handle (HWND in case of Win32 and Widget in case of X Window)
declare function cvGetWindowHandle( window_name as const zstring ptr ) as any ptr
' get name of highgui window given its native handle
declare function cvGetWindowName( window_handle as any ptr ) as const zstring ptr

' destroy window and all the trackers associated with it
declare sub      cvDestroyWindow( window_name as const zstring ptr )
' destroy all windows and all the trackers associated with it
declare sub      cvDestroyAllWindows()

type CvTrackbarCallback as sub CV_CDECL(tbarpos as long)

' create trackbar and display it on top of given window, set callback
declare function cvCreateTrackbar( trackbar_name as const zstring ptr, _
                                   window_name   as const zstring ptr, _
                                   value         as long ptr, _
                                   count         as long, _
                                   callback      as CvTrackbarCallback CV_DEFAULT(NULL) ) as long

type CvTrackbarCallback2 as sub CV_CDECL(tbarpos as long, userdata as any ptr)

declare function cvCreateTrackbar2( trackbar_name as const zstring ptr, _
                                    window_name   as const zstring ptr, _
                                    value         as long ptr, _
                                    count         as long, _
                                    callback      as CvTrackbarCallback2, _
                                    userdata      as any ptr CV_DEFAULT(NULL) ) as long

' retrieve or set trackbar position
declare function cvGetTrackbarPos(trackbar_name as const zstring ptr, _
                                  window_name   as const zstring ptr ) as long
                                  
declare sub      cvSetTrackbarPos(trackbar_name as const zstring ptr, _
                                  window_name   as const zstring ptr, _
                                  tbarpos       as long)
                                  
declare sub      cvSetTrackbarMax(trackbar_name as const zstring ptr, _
                                  window_name   as const zstring ptr, _
                                  maxval        as long)

enum
  CV_EVENT_MOUSEMOVE      = 0
  CV_EVENT_LBUTTONDOWN    = 1
  CV_EVENT_RBUTTONDOWN    = 2
  CV_EVENT_MBUTTONDOWN    = 3
  CV_EVENT_LBUTTONUP      = 4
  CV_EVENT_RBUTTONUP      = 5
  CV_EVENT_MBUTTONUP      = 6
  CV_EVENT_LBUTTONDBLCLK  = 7
  CV_EVENT_RBUTTONDBLCLK  = 8
  CV_EVENT_MBUTTONDBLCLK  = 9
end enum

enum
  CV_EVENT_FLAG_LBUTTON   =  1
  CV_EVENT_FLAG_RBUTTON   =  2
  CV_EVENT_FLAG_MBUTTON   =  4
  CV_EVENT_FLAG_CTRLKEY   =  8
  CV_EVENT_FLAG_SHIFTKEY  = 16
  CV_EVENT_FLAG_ALTKEY    = 32
end enum

type CvMouseCallback as sub CV_CDECL( event as long, x as long, y as long, flags as long, param as any ptr )

' assign callback for mouse events
declare sub      cvSetMouseCallback( window_name as const zstring ptr, _
                                     callback    as CvMouseCallback, _
                                     param       as any ptr CV_DEFAULT(NULL))

enum
  CV_LOAD_IMAGE_UNCHANGED  =-1 ' 8bit, color or not
  CV_LOAD_IMAGE_GRAYSCALE  = 0 ' 8bit, gray
  CV_LOAD_IMAGE_COLOR      = 1 ' ?, color
  CV_LOAD_IMAGE_ANYDEPTH   = 2 ' any depth, ?
  CV_LOAD_IMAGE_ANYCOLOR   = 4 ' ?, any color
end enum

' load image from file
' iscolor can be a combination of above flags 
' where CV_LOAD_IMAGE_UNCHANGED overrides the other flags
' using CV_LOAD_IMAGE_ANYCOLOR alone is equivalent to CV_LOAD_IMAGE_UNCHANGED
' unless CV_LOAD_IMAGE_ANYDEPTH is specified images are converted to 8bit
declare function cvLoadImage ( filename as const zstring ptr, _
                               iscolor  as long CV_DEFAULT(CV_LOAD_IMAGE_COLOR) ) as IplImage ptr
declare function cvLoadImageM( filename as const zstring ptr, _
                               iscolor  as long CV_DEFAULT(CV_LOAD_IMAGE_COLOR) ) as CvMat ptr
enum
  CV_IMWRITE_JPEG_QUALITY              = 1
  
  CV_IMWRITE_PNG_STRATEGY_DEFAULT      = 0
  CV_IMWRITE_PNG_STRATEGY_FILTERED     = 1
  CV_IMWRITE_PNG_STRATEGY_HUFFMAN_ONLY = 2
  CV_IMWRITE_PNG_STRATEGY_RLE          = 3
  CV_IMWRITE_PNG_STRATEGY_FIXED        = 4
  
  CV_IMWRITE_PNG_COMPRESSION           = 16
  CV_IMWRITE_PNG_STRATEGY              = 17
  CV_IMWRITE_PNG_BILEVEL               = 18
  
  CV_IMWRITE_PXM_BINARY                = 32
end enum

' save image to file
declare function cvSaveImage( filename as const zstring ptr, image as const CvArr ptr, _
                              params   as const long ptr CV_DEFAULT(NULL) ) as long

' decode image stored in the buffer
declare function cvDecodeImage ( buf as const CvMat ptr, _
                                 iscolor as long CV_DEFAULT(CV_LOAD_IMAGE_COLOR) ) as IplImage ptr 

declare function cvDecodeImageM( buf as const CvMat ptr, _
                                 iscolor as long CV_DEFAULT(CV_LOAD_IMAGE_COLOR) ) as CvMat ptr

' encode image and store the result as a byte vector (single-row 8uC1 matrix)
declare function cvEncodeImage( ext as const ubyte ptr, image as const CvArr ptr, _
                                params as const long ptr CV_DEFAULT(NULL) ) as CvMat ptr

enum
  CV_CVTIMG_FLIP    = 1
  CV_CVTIMG_SWAP_RB = 2
end enum

' utility function: convert one image to another with optional vertical flip
declare sub      cvConvertImage( src as const CvArr ptr, dst as CvArr ptr, flags as long CV_DEFAULT(0) )

' wait for key event infinitely (delay<=0) or for "delay" milliseconds
declare function cvWaitKey( delay as long CV_DEFAULT(0) ) as long

' OpenGL support
#ifdef HAVE_OPENGL
type CvOpenGlDrawCallback as sub CV_CDECL( userdata as any ptr )
declare sub      cvSetOpenGlDrawCallback( window_name as const zstring ptr, _
                                          callback as CvOpenGlDrawCallback, _
                                          userdata as any ptr CV_DEFAULT(NULL))
declare sub      cvSetOpenGlContext(window_name as const zstring ptr)
declare sub      cvUpdateWindow(window_name as const zstring ptr)
#endif

' ########################################
' # Working with Video Files and Cameras #
' ########################################

' "black box" capture structure
type CvCapture as any

' start capturing frames from video file
declare function cvCreateFileCapture( filename as const zstring ptr ) as CvCapture ptr

enum
  CV_CAP_ANY           =   0   ' autodetect

  CV_CAP_MIL           = 100   ' MIL proprietary drivers

  CV_CAP_VFW           = 200   ' platform native
  CV_CAP_V4L           = 200
  CV_CAP_V4L2          = 200

  CV_CAP_FIREWARE      = 300   ' IEEE 1394 drivers
  CV_CAP_FIREWIRE      = 300
  CV_CAP_IEEE1394      = 300
  CV_CAP_DC1394        = 300
  CV_CAP_CMU1394       = 300

  CV_CAP_STEREO        = 400   ' TYZX proprietary drivers
  CV_CAP_TYZX          = 400
  CV_TYZX_LEFT         = 400
  CV_TYZX_RIGHT        = 401
  CV_TYZX_COLOR        = 402
  CV_TYZX_Z            = 403

  CV_CAP_QT            = 500   ' QuickTime

  CV_CAP_UNICAP        = 600   ' Unicap drivers

  CV_CAP_DSHOW         = 700   ' DirectShow (via videoInput)
  
  CV_CAP_PVAPI         = 800   ' PvAPI, Prosilica GigE SDK

  CV_CAP_OPENNI        = 900   ' OpenNI (for Kinect)
  CV_CAP_OPENNI_ASUS   = 910   ' OpenNI (for Asus Xtion)
  
  CV_CAP_XIAPI         = 1100   ' XIMEA Camera API

  CV_CAP_AVFOUNDATION  = 1200  ' AVFoundation framework for iOS (OS X Lion will have the same API)

  CV_CAP_GIGANETIX     = 1300  ' Smartek Giganetix GigEVisionSDK
  
  CV_CAP_MSMF          = 1400  ' Microsoft Media Foundation (via videoInput)

  CV_CAP_INTELPERC     = 1500   ' Intel Perceptual Computing SDK
end enum

' start capturing frames from camera: index = camera_index + domain_offset (CV_CAP_*)
declare function cvCreateCameraCapture( index as long ) as CvCapture ptr

' grab a frame, return 1 on success, 0 on fail.
' this function is thought to be fast
declare function cvGrabFrame( capture as CvCapture ptr ) as long

' get the frame grabbed with cvGrabFrame(..)
' This function may apply some frame processing like
' frame decompression, flipping etc.
' !!!DO NOT RELEASE or MODIFY the retrieved frame!!!
declare function cvRetrieveFrame( capture as CvCapture ptr, streamIdx as long CV_DEFAULT(0) ) as IplImage ptr

' Just a combination of cvGrabFrame and cvRetrieveFrame
' !!!DO NOT RELEASE or MODIFY the retrieved frame!!!
declare function cvQueryFrame( capture as CvCapture ptr ) as IplImage ptr

' stop capturing/reading and free resources
declare sub      cvReleaseCapture( capture as CvCapture ptr ptr )

enum
  ' modes of the controlling registers (can be: auto, manual, auto single push, absolute Latter allowed with any other mode)
  ' every feature can have only one mode turned on at a time
  CV_CAP_PROP_DC1394_OFF                = -4 ' turn the feature off (not controlled manually nor automatically)
  CV_CAP_PROP_DC1394_MODE_MANUAL        = -3 ' set automatically when a value of the feature is set by the user
  CV_CAP_PROP_DC1394_MODE_AUTO          = -2
  CV_CAP_PROP_DC1394_MODE_ONE_PUSH_AUTO = -1
  
  CV_CAP_PROP_POS_MSEC        = 0
  CV_CAP_PROP_POS_FRAMES      = 1
  CV_CAP_PROP_POS_AVI_RATIO   = 2
  CV_CAP_PROP_FRAME_WIDTH     = 3
  CV_CAP_PROP_FRAME_HEIGHT    = 4
  CV_CAP_PROP_FPS             = 5
  CV_CAP_PROP_FOURCC          = 6
  CV_CAP_PROP_FRAME_COUNT     = 7
  CV_CAP_PROP_FORMAT          = 8
  CV_CAP_PROP_MODE            = 9
  CV_CAP_PROP_BRIGHTNESS      = 10
  CV_CAP_PROP_CONTRAST        = 11
  CV_CAP_PROP_SATURATION      = 12
  CV_CAP_PROP_HUE             = 13
  CV_CAP_PROP_GAIN            = 14
  CV_CAP_PROP_EXPOSURE        = 15
  CV_CAP_PROP_CONVERT_RGB     = 16
  CV_CAP_PROP_WHITE_BALANCE_U = 17
  CV_CAP_PROP_RECTIFICATION   = 18
  CV_CAP_PROP_MONOCROME       = 19
  CV_CAP_PROP_SHARPNESS       = 20
  CV_CAP_PROP_AUTO_EXPOSURE   = 21 ' exposure control done by camera,
                                   ' user can adjust refernce level using this feature
  CV_CAP_PROP_GAMMA           = 22
  CV_CAP_PROP_TEMPERATURE     = 23
  CV_CAP_PROP_TRIGGER         = 24
  CV_CAP_PROP_TRIGGER_DELAY   = 25
  CV_CAP_PROP_WHITE_BALANCE_V = 26
  CV_CAP_PROP_ZOOM            = 27
  CV_CAP_PROP_FOCUS           = 28
  CV_CAP_PROP_GUID            = 29
  CV_CAP_PROP_ISO_SPEED       = 30
  CV_CAP_PROP_MAX_DC1394      = 31
  CV_CAP_PROP_BACKLIGHT       = 32
  CV_CAP_PROP_PAN             = 33
  CV_CAP_PROP_TILT            = 34
  CV_CAP_PROP_ROLL            = 35
  CV_CAP_PROP_IRIS            = 36
  CV_CAP_PROP_SETTINGS        = 37
  CV_CAP_PROP_BUFFERSIZE      = 38

  CV_CAP_PROP_AUTOGRAB        = 1024 ' property for highgui class CvCapture_Android only
  CV_CAP_PROP_SUPPORTED_PREVIEW_SIZES_STRING = 1025 ' readonly, tricky property, returns cpnst char* indeed
  CV_CAP_PROP_PREVIEW_FORMAT  = 1026 ' readonly, tricky property, returns cpnst char* indeed

  ' OpenNI map generators
  CV_CAP_OPENNI_IMAGE_GENERATOR = 1 shl 30
  CV_CAP_OPENNI_DEPTH_GENERATOR = 1 shl 31
  CV_CAP_OPENNI_GENERATORS_MASK = CV_CAP_OPENNI_DEPTH_GENERATOR + CV_CAP_OPENNI_IMAGE_GENERATOR

  ' Properties of cameras available through OpenNI interfaces
  CV_CAP_PROP_OPENNI_OUTPUT_MODE     = 100
  CV_CAP_PROP_OPENNI_FRAME_MAX_DEPTH = 101 ' in mm
  CV_CAP_PROP_OPENNI_BASELINE        = 102 ' in mm
  CV_CAP_PROP_OPENNI_FOCAL_LENGTH    = 103 ' in pixels
  CV_CAP_PROP_OPENNI_REGISTRATION    = 104 ' flag
  CV_CAP_PROP_OPENNI_REGISTRATION_ON = CV_CAP_PROP_OPENNI_REGISTRATION ' flag that synchronizes the remapping depth map to image map
                                                                        ' by changing depth generator's view point (if the flag is "on") or
                                                                        ' sets this view point to its normal one (if the flag is "off").
  CV_CAP_PROP_OPENNI_APPROX_FRAME_SYNC = 105
  CV_CAP_PROP_OPENNI_MAX_BUFFER_SIZE   = 106
  CV_CAP_PROP_OPENNI_CIRCLE_BUFFER     = 107
  CV_CAP_PROP_OPENNI_MAX_TIME_DURATION = 108

  CV_CAP_PROP_OPENNI_GENERATOR_PRESENT = 109

  CV_CAP_OPENNI_IMAGE_GENERATOR_PRESENT         = CV_CAP_OPENNI_IMAGE_GENERATOR + CV_CAP_PROP_OPENNI_GENERATOR_PRESENT
  CV_CAP_OPENNI_IMAGE_GENERATOR_OUTPUT_MODE     = CV_CAP_OPENNI_IMAGE_GENERATOR + CV_CAP_PROP_OPENNI_OUTPUT_MODE
  CV_CAP_OPENNI_DEPTH_GENERATOR_BASELINE        = CV_CAP_OPENNI_DEPTH_GENERATOR + CV_CAP_PROP_OPENNI_BASELINE
  CV_CAP_OPENNI_DEPTH_GENERATOR_FOCAL_LENGTH    = CV_CAP_OPENNI_DEPTH_GENERATOR + CV_CAP_PROP_OPENNI_FOCAL_LENGTH
  CV_CAP_OPENNI_DEPTH_GENERATOR_REGISTRATION    = CV_CAP_OPENNI_DEPTH_GENERATOR + CV_CAP_PROP_OPENNI_REGISTRATION
  CV_CAP_OPENNI_DEPTH_GENERATOR_REGISTRATION_ON = CV_CAP_OPENNI_DEPTH_GENERATOR_REGISTRATION

  ' Properties of cameras available through GStreamer interface
  CV_CAP_GSTREAMER_QUEUE_LENGTH   = 200 ' default is 1

  ' PVAPI
  CV_CAP_PROP_PVAPI_MULTICASTIP           = 300 ' ip for anable multicast master mode. 0 for disable multicast
  CV_CAP_PROP_PVAPI_FRAMESTARTTRIGGERMODE = 301 ' FrameStartTriggerMode: Determines how a frame is initiated
  CV_CAP_PROP_PVAPI_DECIMATIONHORIZONTAL  = 302 ' Horizontal sub-sampling of the image
  CV_CAP_PROP_PVAPI_DECIMATIONVERTICAL    = 303 ' Vertical sub-sampling of the image
  CV_CAP_PROP_PVAPI_BINNINGX              = 304 ' Horizontal binning factor
  CV_CAP_PROP_PVAPI_BINNINGY              = 305 ' Vertical binning factor
  CV_CAP_PROP_PVAPI_PIXELFORMAT           = 306 ' Pixel format

  ' Properties of cameras available through XIMEA SDK interface
  CV_CAP_PROP_XI_DOWNSAMPLING  = 400 ' Change image resolution by binning or skipping.
  CV_CAP_PROP_XI_DATA_FORMAT   = 401 ' Output data format.
  CV_CAP_PROP_XI_OFFSET_X      = 402 ' Horizontal offset from the origin to the area of interest (in pixels).
  CV_CAP_PROP_XI_OFFSET_Y      = 403 ' Vertical offset from the origin to the area of interest (in pixels).
  CV_CAP_PROP_XI_TRG_SOURCE    = 404 ' Defines source of trigger.
  CV_CAP_PROP_XI_TRG_SOFTWARE  = 405 ' Generates an internal trigger. PRM_TRG_SOURCE must be set to TRG_SOFTWARE.
  CV_CAP_PROP_XI_GPI_SELECTOR  = 406 ' Selects general purpose input
  CV_CAP_PROP_XI_GPI_MODE      = 407 ' Set general purpose input mode
  CV_CAP_PROP_XI_GPI_LEVEL     = 408 ' Get general purpose level
  CV_CAP_PROP_XI_GPO_SELECTOR  = 409 ' Selects general purpose output
  CV_CAP_PROP_XI_GPO_MODE      = 410 ' Set general purpose output mode
  CV_CAP_PROP_XI_LED_SELECTOR  = 411 ' Selects camera signalling LED
  CV_CAP_PROP_XI_LED_MODE      = 412 ' Define camera signalling LED functionality
  CV_CAP_PROP_XI_MANUAL_WB     = 413 ' Calculates White Balance(must be called during acquisition)
  CV_CAP_PROP_XI_AUTO_WB       = 414 ' Automatic white balance
  CV_CAP_PROP_XI_AEAG          = 415 ' Automatic exposure/gain
  CV_CAP_PROP_XI_EXP_PRIORITY  = 416 ' Exposure priority (0.5 - exposure 50%, gain 50%).
  CV_CAP_PROP_XI_AE_MAX_LIMIT  = 417 ' Maximum limit of exposure in AEAG procedure
  CV_CAP_PROP_XI_AG_MAX_LIMIT  = 418 ' Maximum limit of gain in AEAG procedure
  CV_CAP_PROP_XI_AEAG_LEVEL    = 419 ' Average intensity of output signal AEAG should achieve(in %)
  CV_CAP_PROP_XI_TIMEOUT       = 420 ' Image capture timeout in milliseconds

  ' Properties of cameras available through Smartek Giganetix Ethernet Vision interface
  CV_CAP_PROP_GIGA_FRAME_OFFSET_X   = 10001
  CV_CAP_PROP_GIGA_FRAME_OFFSET_Y   = 10002
  CV_CAP_PROP_GIGA_FRAME_WIDTH_MAX  = 10003
  CV_CAP_PROP_GIGA_FRAME_HEIGH_MAX  = 10004
  CV_CAP_PROP_GIGA_FRAME_SENS_WIDTH = 10005
  CV_CAP_PROP_GIGA_FRAME_SENS_HEIGH = 10006

  CV_CAP_PROP_INTELPERC_PROFILE_COUNT               = 11001
  CV_CAP_PROP_INTELPERC_PROFILE_IDX                 = 11002
  CV_CAP_PROP_INTELPERC_DEPTH_LOW_CONFIDENCE_VALUE  = 11003
  CV_CAP_PROP_INTELPERC_DEPTH_SATURATION_VALUE      = 11004
  CV_CAP_PROP_INTELPERC_DEPTH_CONFIDENCE_THRESHOLD  = 11005
  CV_CAP_PROP_INTELPERC_DEPTH_FOCAL_LENGTH_HORZ     = 11006
  CV_CAP_PROP_INTELPERC_DEPTH_FOCAL_LENGTH_VERT     = 11007

  ' Intel PerC streams
  CV_CAP_INTELPERC_DEPTH_GENERATOR = 1 shl 29
  CV_CAP_INTELPERC_IMAGE_GENERATOR = 1 shl 28
  CV_CAP_INTELPERC_GENERATORS_MASK = CV_CAP_INTELPERC_DEPTH_GENERATOR + CV_CAP_INTELPERC_IMAGE_GENERATOR
end enum

enum
  ' Data given from depth generator.
  CV_CAP_OPENNI_DEPTH_MAP         = 0 ' Depth values in mm (CV_16UC1)
  CV_CAP_OPENNI_POINT_CLOUD_MAP   = 1 ' XYZ in meters (CV_32FC3)
  CV_CAP_OPENNI_DISPARITY_MAP     = 2 ' Disparity in pixels (CV_8UC1)
  CV_CAP_OPENNI_DISPARITY_MAP_32F = 3 ' Disparity in pixels (CV_32FC1)
  CV_CAP_OPENNI_VALID_DEPTH_MASK  = 4 ' CV_8UC1

  ' Data given from RGB image generator.
  CV_CAP_OPENNI_BGR_IMAGE         = 5
  CV_CAP_OPENNI_GRAY_IMAGE        = 6
end enum

' Supported output modes of OpenNI image generator
enum
  CV_CAP_OPENNI_VGA_30HZ     = 0
  CV_CAP_OPENNI_SXGA_15HZ    = 1
  CV_CAP_OPENNI_SXGA_30HZ    = 2
  CV_CAP_OPENNI_QVGA_30HZ    = 3
  CV_CAP_OPENNI_QVGA_60HZ    = 4
end enum

enum
  CV_CAP_INTELPERC_DEPTH_MAP   = 0 ' Each pixel is a 16-bit integer. The value indicates the distance from an object to the camera's XY plane or the Cartesian depth.
  CV_CAP_INTELPERC_UVDEPTH_MAP = 1 ' Each pixel contains two 32-bit floating point values in the range of 0-1, representing the mapping of depth coordinates to the color coordinates.
  CV_CAP_INTELPERC_IR_MAP      = 2 ' Each pixel is a 16-bit integer. The value indicates the intensity of the reflected laser beam.
  CV_CAP_INTELPERC_IMAGE       = 3
end enum

' retrieve or set capture properties
declare function cvGetCaptureProperty( capture as CvCapture ptr, property_id as long) as double
declare function cvSetCaptureProperty( capture as CvCapture ptr, property_id as long, value as double) as long

' Return the type of the capturer (eg, CV_CAP_V4W, CV_CAP_UNICAP), which is unknown if created with CV_CAP_ANY
declare function cvGetCaptureDomain( capture as CvCapture ptr ) as long

' "black box" video file writer structure
type CvVideoWriter as any

#define CV_FOURCC_MACRO(c1, c2, c3, c4) (((c1) and 255) + (((c2) ans 255) shl 8) and (((c3) and 255) shl 16) + (((c4) and 255) shl 24))

#define CV_FOURCC(c1,c2,c3,c4) CV_FOURCC_MACRO( asc(#c1), asc(#c2), asc(#c3), asc(#c4) )

#ifdef HAVE_WIN32UI
 ' Open Codec Selection Dialog (Windows only)
 #define CV_FOURCC_PROMPT -1
#endif

#ifdef __FB_LINUX__
  ' Use default codec for specified filename (Linux only)
 #define CV_FOURCC_DEFAULT CV_FOURCC(I,Y,U,V) 
#endif

' initialize video file writer
declare function cvCreateVideoWriter( filename   as const zstring ptr, _
                                      fourcc     as long, _
                                      fps        as double, _
                                      frame_size as CvSize, _
                                      is_color   as long CV_DEFAULT(1) ) as CvVideoWriter ptr
' write frame to video file
declare function cvWriteFrame( writer as CvVideoWriter ptr, _
                               image  as const IplImage ptr ) as long

' close video file writer
declare sub      cvReleaseVideoWriter( writer as CvVideoWriter ptr ptr )

#ifdef HAVE_WIN32UI ' windows only
declare sub cvSetPreprocessFuncWin32_ ( cb as const any ptr )
declare sub cvSetPostprocessFuncWin32_( cb as const any ptr )
#define cvSetPreprocessFuncWin32(callback)  cvSetPreprocessFuncWin32_ (cptr(const any ptr,callback))
#define cvSetPostprocessFuncWin32(callback) cvSetPostprocessFuncWin32_(cptr(const any ptr,callback))

#endif

end extern ' "C"


