#pragma once

#ifndef __OPENCV__
  #error 666: do not include this file manually include only 'opencv.bi' !
#endif


'#include "opencv_imgproc_types.bi"

extern "C"

' ######################################
' # Background statistics accumulation #
' ######################################

' Adds image to accumulator
' see also: cv::accumulate
declare sub  cvAcc( image as const CvArr ptr, sum as CvArr ptr, mask as const CvArr ptr = NULL )

' Adds squared image to accumulator
' see also: cv::accumulateSquare
declare sub cvSquareAcc( image as const CvArr ptr, sqsum as CvArr ptr, mask as const CvArr ptr = NULL )

' Adds a product of two images to accumulator
' see also: cv::accumulateProduct
declare sub cvMultiplyAcc( image1 as const CvArr ptr, image2 as const CvArr ptr, acc as CvArr ptr, _
                           mask as const CvArr ptr = NULL )

' Adds image to accumulator with weights: acc = acc*(1-alpha) + image*alpha
' see also: cv::accumulateWeighted
declare sub cvRunningAvg( image as const CvArr ptr, acc as CvArr ptr, alpha as double, mask as const CvArr ptr = NULL )

' ####################
' # Image Processing #
' ####################

' Copies source 2D array inside of the larger destination array and
' makes a border of the specified type (IPL_BORDER_*) around the copied area.
declare sub cvCopyMakeBorder( src as const CvArr ptr, dst as CvArr ptr, offset as CvPoint, _
                              bordertype as long, value as CvScalar = cvScalarAll(0) )

' Smooths the image in one of several ways.
'  param src        The source image
'  param dst        The destination image
'  param smoothtype Type of the smoothing, see SmoothMethod_c

'  param size1      The first parameter of the smoothing operation, the aperture width.
'                   Must be a positive odd number (1, 3, 5, ...)

'  param size2      The second parameter of the smoothing operation, the aperture height.
'	                  Ignored by CV_MEDIAN and CV_BILATERAL methods.
'							      In the case of simple scaled/non-scaled and Gaussian blur if size2 is zero, 
'							      it is set to size1. Otherwise it must be a positive odd number.

'  param sigma1     In the case of a Gaussian parameter this parameter may specify 
'	                  Gaussian sigma (standard deviation). 
'								    If it is zero, it is calculated from the kernel size: sigma = 0.3 (n/2 - 1) + 0.8  
'                   Using standard sigma for small kernels times 3 to times 7 gives better speed.
'								    If sigma1 is not zero, while size1 and size2 are zeros, 
'								    the kernel size is calculated from the sigma (to provide accurate enough operation).
'  param sigma2     additional parameter for bilateral filtering
'  see also: cv::GaussianBlur, cv::blur, cv::medianBlur, cv::bilateralFilter.
declare sub cvSmooth( src as const CvArr ptr, _
                      dst as CvArr ptr, _
                      smoothtype as long = CV_GAUSSIAN, _
                      size1   as long   = 3, _
                      size2   as long   = 0, _
                      sigma1  as double = 0, _
                      sigma2  as double = 0)

' Convolves an image with the kernel.
'  param src    input image.
'  param dst    output image of the same size and the same number of channels as src.
'  param kernel convolution kernel (or rather a correlation kernel), 
'	              a single-channel floating point matrix; 
'								if you want to apply different kernels to different channels, 
'								split the image into separate color planes using split and process them individually.
'  param anchor anchor of the kernel that indicates the relative position of a filtered point within
'                the kernel; the anchor should lie within the kernel; 
'								default value (-1,-1) means that the anchor is at the kernel center.
'  see also: cv::filter2D
declare sub cvFilter2D( src as const CvArr ptr, dst as CvArr ptr, kernel as const CvMat ptr, _
                        anchor as CvPoint = cv_Point(-1,-1))

' Finds integral image: SUM(X,Y) = sum(x<X,y<Y)I(x,y)
' see also: cv::integral
declare sub cvIntegral( image as const CvArr ptr, sum as CvArr ptr, _
                       sqsum as CvArr ptr = NULL, _
                       tilted_sum as CvArr ptr = NULL)

' Smoothes the input image with gaussian kernel and then down-samples it.
'   dst_width  = floor(src_width/2)[+1],
'   dst_height = floor(src_height/2)[+1]
'   see also: cv::pyrDown
declare sub cvPyrDown( src as const CvArr ptr, dst as CvArr ptr, _
                       filter as long = CV_GAUSSIAN_5x5 )

' Up-samples image and smoothes the result with gaussian kernel.
' dst_width = src_width*2,
' dst_height = src_height*2
' see also: cv::pyrUp
declare sub cvPyrUp( src as const CvArr ptr, dst as CvArr ptr, _
                     filter as long = CV_GAUSSIAN_5x5 )

' Builds pyramid for an image
' see also: buildPyramid
declare function cvCreatePyramid( img as const CvArr ptr, extra_layers as long, rate as double, _
                                  layer_sizes as const CvSize ptr = NULL, _
                                  bufarr as CvArr ptr = NULL, _
                                  calc as long = 1, filter as long = CV_GAUSSIAN_5x5 ) as CvMat ptr ptr

' Releases pyramid
declare sub cvReleasePyramid( pyramid as CvMat ptr ptr ptr, extra_layers as long)


' Filters image using meanshift algorithm
' see also: cv::pyrMeanShiftFiltering
declare sub cvPyrMeanShiftFiltering( src as const CvArr ptr, dst as CvArr ptr, _
                                    sp as double, sr as double, max_level as long = 1, _
                                    termcrit as CvTermCriteria = cv_TermCriteria(CV_TERMCRIT_ITER+CV_TERMCRIT_EPS,5,1) )

' Segments image using seed "markers"
' see also: cv::watershed
declare sub cvWatershed( image as const CvArr ptr, markers as CvArr ptr)

' Calculates an image derivative using generalized Sobel
' (aperture_size = 1,3,5,7) or Scharr (aperture_size = -1) operator.
' Scharr can be used only for the first dx or dy derivative
' see also: cv::Sobel
declare sub cvSobel( src as const CvArr ptr, dst as CvArr ptr, _
                    xorder as long, yorder as long, _
                    aperture_size as long = 3 )

' Calculates the image Laplacian: (d2/dx + d2/dy)I
' see also: cv::Laplacian
declare sub cvLaplace( src as const CvArr ptr, dst as CvArr ptr, _
                      aperture_size as long = 3 )

' Converts input array pixels from one color space to another
' see also: cv::cvtColor
declare sub cvCvtColor( src as const CvArr ptr, dst as CvArr ptr, code as long )

' Resizes image (input array is resized to fit the destination array)
' see also: cv::resize
declare sub cvResize( src as const CvArr ptr, dst as CvArr ptr, _
                      interpolation as long = CV_INTER_LINEAR )

' Warps image with affine transform
' note ::cvGetQuadrangleSubPix is similar to ::cvWarpAffine, but the outliers are extrapolated using
' replication border mode.
' see also: cv::warpAffine
declare sub cvWarpAffine( src as const CvArr ptr, dst as CvArr ptr, map_matrix as const CvMat ptr, _
                          flags as long = (CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS), _
                          fillval as CvScalar = cvScalarAll(0) )

' Computes affine transform matrix for mapping src[i] to dst[i] (i=0,1,2)
' see also: cv::getAffineTransform
declare function cvGetAffineTransform( src as const CvPoint2D32f ptr, _
                                       dst as const CvPoint2D32f ptr, _
                                       map_matrix as CvMat ptr) as CvMat ptr

' Computes rotation_matrix matrix 
' see also: cv::getRotationMatrix2D
declare function cv2DRotationMatrix( center as CvPoint2D32f,  angle as double, _
                                     scale as double, map_matrix as CvMat ptr ) as CvMat ptr

' Warps image with perspective (projective) transform
' see also: cv::warpPerspective
declare sub cvWarpPerspective( src as const CvArr ptr, dst as CvArr ptr, map_matrix as const CvMat ptr, _
                               flags as long = (CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS), _
                               fillval as CvScalar = cvScalarAll(0) )

' Computes perspective transform matrix for mapping src[i] to dst[i] (i=0,1,2,3)
' see also: cv::getPerspectiveTransform
declare function cvGetPerspectiveTransform( src as const CvPoint2D32f ptr, _
                                            dst as const CvPoint2D32f ptr, _
                                            map_matrix as CvMat ptr ) as CvMat ptr

' Performs generic geometric transformation using the specified coordinate maps
' see also: cv::remap
declare sub cvRemap( src as const CvArr ptr, dst as CvArr ptr, _
                     mapx as const CvArr ptr, mapy as const CvArr ptr, _
                     flags as long = (CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS), _
                     fillval as CvScalar = cvScalarAll(0) )

' Converts mapx & mapy from floating-point to integer formats for cvRemap
' see also: cv::convertMaps
declare sub cvConvertMaps( mapx as const CvArr ptr, mapy as const CvArr ptr, _
                           mapxy as CvArr ptr, mapalpha as CvArr ptr )

' Performs forward or inverse log-polar image transform
' see also: cv::warpPolar
declare sub cvLogPolar( src as const CvArr ptr, dst as CvArr ptr, _
                        center as CvPoint2D32f, M as double, _
                        flags as long = (CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS) )

' Performs forward or inverse linear-polar image transform
' see also: cv::warpPolar
declare sub cvLinearPolar( src as const CvArr ptr,dst as CvArr ptr, _
                           center as CvPoint2D32f, maxRadius as double, _
                           flags as long = (CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS) )

' Returns a structuring element of the specified size and shape for morphological operations.
'  note the created structuring element IplConvKernel* element must be released in the end using
' 'cvReleaseStructuringElement(@element)'.

'  param cols     Width of the structuring element
'  param rows     Height of the structuring element
'  param anchor_x x-coordinate of the anchor
'  param anchor_y y-coordinate of the anchor
'  param shape    element shape that could be one of the cv::MorphShapes_c
'  param values   integer array of cols*rows elements that specifies the custom shape 
'	                of the structuring element, when shape=CV_SHAPE_CUSTOM.
' see also: cv::getStructuringElement
declare function cvCreateStructuringElementEx( cols as long, rows as long, anchor_x as long, anchor_y as long, _
                                               shape as long, values  as long ptr = NULL ) as IplConvKernel

' releases structuring element
' see also: cvCreateStructuringElementEx
declare sub cvReleaseStructuringElement( element as IplConvKernel ptr ptr )

' erodes input image (applies minimum filter) one or more times.
' If element pointer is NULL, 3x3 rectangular element is used
' see also: cv::erode
declare sub cvErode( src as const CvArr ptr, dst as CvArr ptr, _
                     element as IplConvKernel ptr = NULL, _
                     iterations as long = 1 )

' dilates input image (applies maximum filter) one or more times.
'   If element pointer is NULL, 3x3 rectangular element is used
' see also: cv::dilate
declare sub cvDilate( src as const CvArr ptr, dst as CvArr ptr, _
                      element as IplConvKernel ptr = NULL, _
                      iterations as long = 1 )

' Performs complex morphological transformation
' see also: cv::morphologyEx
declare sub cvMorphologyEx( src as const CvArr ptr, dst as CvArr ptr, _
                            temp as CvArr ptr, element as IplConvKernel ptr, _
                            operation as long,  iterations as long = 1 )

' Calculates all spatial and central moments up to the 3rd order
' see also: cv::moments
declare sub cvMoments( arr as const CvArr ptr, moments as CvMoments ptr, _binary as long = 0 )

' Retrieve spatial moments
declare function cvGetSpatialMoment( moments as CvMoments ptr, x_order as long, y_order as long ) as double
' Retrieve central moments
declare function cvGetCentralMoment( moments as CvMoments ptr, x_order as long, y_order as long ) as double
' Retrieve normalized central moments
declare function cvGetNormalizedCentralMoment( moments as CvMoments ptr, x_order as long, y_order as long ) as double

' Calculates 7 Hu's invariants from precalculated spatial and central moments
' see also: cv::HuMoments
declare sub cvGetHuMoments( moments as CvMoments ptr, hu_moments as CvHuMoments ptr )

' #################
' # data sampling #
' #################

' Fetches pixels that belong to the specified line segment and stores them to the buffer.
' Returns the number of retrieved points.
' see also: cv::LineSegmentDetector
declare function cvSampleLine( image as const CvArr ptr, p1 as CvPoint, p2 as CvPoint, buffer as any ptr, _
                               connectivity as long = 8) as long

' Retrieves the rectangular image region with specified center from the input array.
' dst(x,y) <- src(x + center.x - dst_width/2, y + center.y - dst_height/2).
' Values of pixels with fractional coordinates are retrieved using bilinear interpolation
' see also: cv::getRectSubPix
declare sub cvGetRectSubPix( src as const CvArr ptr, dst as CvArr ptr, center as CvPoint2D32f )

' Retrieves quadrangle from the input array.
'    matrixarr = ( a11  a12 | b1 )   dst(x,y) <- src(A[x y]' + b)
'                ( a21  a22 | b2 )   (bilinear interpolation is used to retrieve pixels
'                                     with fractional coordinates)
' see also: cvWarpAffine
declare sub cvGetQuadrangleSubPix( src as const CvArr ptr, dst as CvArr ptr, _
                                   map_matrix  as const CvMat ptr)

' Measures similarity between template and overlapped windows in the source image
' and fills the resultant image with the measurements
' see also: cv::matchTemplate
declare sub cvMatchTemplate( image as const CvArr ptr, templ as const CvArr ptr, _
                             result as CvArr ptr, method as long )

' Computes earth mover distance between
' two weighted point sets (called signatures)
' see also: cv::EMD
declare function cvCalcEMD2( signature1 as const CvArr ptr, signature2 as const CvArr ptr, _
                             distance_type as long, distance_func as CvDistanceFunction = NULL, _
                             cost_matrix as const CvArr ptr = NULL, _
                             flow as CvArr ptr = NULL, _
                             lower_bound as single ptr = NULL, _
                             userdata as any ptr = NULL) as single

' #######################
' # Contours retrieving #
' #######################

' Retrieves outer and optionally inner boundaries of white (non-zero) connected
' components in the black (zero) background
' see also: cv::findContours, cvStartFindContours, cvFindNextContour, cvSubstituteContour, cvEndFindContours
declare function cvFindContours( image as CvArr ptr, storage as CvMemStorage ptr, first_contour as CvSeq ptr ptr, _
                                 header_size as long = sizeof(CvContour), _
                                 mode as long = CV_RETR_LIST, _
                                 method as long = CV_CHAIN_APPROX_SIMPLE, _
                                 offset as CvPoint = cv_Point(0,0) ) as long

' Initializes contour retrieving process.
' Calls cvStartFindContours.
' Calls cvFindNextContour until null pointer is returned
' or some other condition becomes true.
' Calls cvEndFindContours at the end.
' see also: cvFindContours
declare function cvStartFindContours( image as CvArr ptr, storage as CvMemStorage ptr, _
                                      header_size as long = sizeof(CvContour), _
                                      mode as long = CV_RETR_LIST, _
                                      method as long = CV_CHAIN_APPROX_SIMPLE, _
                                      offset as CvPoint = cv_Point(0,0) ) as CvContourScanner

' Retrieves next contour
' see also: cvFindContours
declare function cvFindNextContour( scanner as CvContourScanner ) as CvSeq ptr

' Substitutes the last retrieved contour with the new one
' (if the substitutor is null, the last retrieved contour is removed from the tree)
' see also: cvFindContours
declare sub cvSubstituteContour( scanner as CvContourScanner, new_contour as CvSeq ptr)

' Releases contour scanner and returns pointer to the first outer contour
' see also: cvFindContours
declare function cvEndFindContours( scanner as CvContourScanner ptr ) as CvSeq ptr

' Approximates Freeman chain(s) with a polygonal curve.

' This is a standalone contour approximation routine, not represented in the new interface. When
' cvFindContours retrieves contours as Freeman chains, it calls the function to get approximated
' contours, represented as polygons.

' param src_seq Pointer to the approximated Freeman chain that can refer to other chains.
' param storage Storage location for the resulting polylines.
' param method Approximation method (see the description of the function :ocvFindContours ).
' param parameter Method parameter (not used now).
' param minimal_perimeter Approximates only those contours whose perimeters are not less than
' minimal_perimeter . Other chains are removed from the resulting structure.
' param recursive Recursion flag. If it is non-zero, the function approximates all chains that can
' be obtained from chain by using the h_next or v_next links. Otherwise, the single input chain is
' approximated.
' see also: cvStartReadChainPoints, cvReadChainPoint
declare function cvApproxChains( src_seq as CvSeq ptr, storage as CvMemStorage ptr, _
                                 method as long = CV_CHAIN_APPROX_SIMPLE, _
                                 parameter as double = 0, _
                                 minimal_perimeter as long = 0, _
                                 recursive as long = 0) as CvSeq ptr

' Initializes Freeman chain reader.
' The reader is used to iteratively get coordinates of all the chain points.
' If the Freeman codes should be read as is, a simple sequence reader should be used
' see also: cvApproxChains
declare sub cvStartReadChainPoints( _chain as CvChain ptr, reader as CvChainPtReader ptr )

' Retrieves the next chain point
' see also: cvApproxChains
declare function cvReadChainPoint( reader as CvChainPtReader ptr) as CvPoint

' #########################################
' # Contour Processing and Shape Analysis #
' #########################################

' Approximates a single polygonal curve (contour) or
' a tree of polygonal curves (contours)
' see also: cv::approxPolyDP
declare function cvApproxPoly( src_seq as const any ptr, _
                               header_size as long, storage as CvMemStorage ptr, _
                               method as long, eps as double, _
                               recursive as long = 0 ) as CvSeq ptr

' Calculates perimeter of a contour or length of a part of contour
' see also: cv::arcLength
declare function cvArcLength( curve as const any ptr, _
                              slice as CvSlice = CV_WHOLE_SEQ, _
                              is_closed as long = -1) as double

' same as cvArcLength for closed contour
function cvContourPerimeter( contour as const any ptr ) as double
  return cvArcLength( contour, CV_WHOLE_SEQ, 1 )
end function


' Calculates contour bounding rectangle (update=1) or
' just retrieves pre-calculated rectangle (update=0)
' see also: cv::boundingRect
declare function cvBoundingRect( points as CvArr ptr, update as long = 0 ) as CvRect

' Calculates area of a contour or contour segment
' see also: cv::contourArea
declare function cvContourArea( contour as const CvArr ptr, _
                                slice as CvSlice = CV_WHOLE_SEQ, _
                                oriented as long = 0) as double

' Finds minimum area rotated rectangle bounding a set of points
' see also: cv::minAreaRect
declare function cvMinAreaRect2( points as const CvArr ptr, _
                                 storage as CvMemStorage ptr = NULL) as CvBox2D

' Finds minimum enclosing circle for a set of points
' see also: cv::minEnclosingCircle
declare function cvMinEnclosingCircle( points as const CvArr ptr, _
                                       center as CvPoint2D32f ptr, radius as single ptr ) as long

' Compares two contours by matching their moments
' see also: cv::matchShapes
declare function cvMatchShapes( object1 as const any ptr, object2 as const any ptr, _
                                method as long, parameter as double = 0) as double

' Calculates exact convex hull of 2d point set
' see also: cv::convexHull
declare function cvConvexHull2( _input as const CvArr ptr, _
                                hull_storage as any ptr = NULL, _
                                orientation as long = CV_CLOCKWISE, _
                                return_points as long = 0) as CvSeq ptr

' Checks whether the contour is convex or not (returns 1 if convex, 0 if not)
' see also: cv::isContourConvex
declare function cvCheckContourConvexity( contour as const CvArr ptr ) as long


' Finds convexity defects for the contour
' see also: cv::convexityDefects
declare function cvConvexityDefects( contour as const CvArr ptr, convexhull as const CvArr ptr, _
                                     storage as CvMemStorage ptr = NULL) as CvSeq ptr 

' Fits ellipse into a set of 2d points
' see also: cv::fitEllipse
declare function cvFitEllipse2( points as const CvArr ptr ) as CvBox2D

' Finds minimum rectangle containing two given rectangles
declare function cvMaxRect( rect1 as const CvRect ptr, rect2 as const CvRect ptr ) as CvRect

' Finds coordinates of the box vertices
declare sub cvBoxPoints( box as CvBox2D, as CvPoint2D32f ptr ) ' (4)

' Initializes sequence header for a matrix (column or row vector) of points
' a wrapper for cvMakeSeqHeaderForArray (it does not initialize bounding rectangle!!!)
declare function cvPointSeqFromMat( seq_kind as long, mat as const CvArr ptr, _
                                    contour_header as CvContour ptr, block as CvSeqBlock ptr ) as CvSeq ptr

' Checks whether the point is inside polygon, outside, on an edge (at a vertex).
' Returns positive, negative or zero value, correspondingly.
' Optionally, measures a signed distance between
' the point and the nearest polygon edge (measure_dist=1)
' see also: cv::pointPolygonTest
declare function cvPointPolygonTest( contour as const CvArr ptr, _
                                     pt as CvPoint2D32f,  measure_dist as long ) as double

' #######################
' # Histogram functions #
' #######################

/'  Creates a histogram.

The function creates a histogram of the specified size and returns a pointer to the created
histogram. If the array ranges is 0, the histogram bin ranges must be specified later via the
function cvSetHistBinRanges. Though cvCalcHist and cvCalcBackProject may process 8-bit images
without setting bin ranges, they assume they are equally spaced in 0 to 255 bins.

@param dims Number of histogram dimensions.
@param sizes Array of the histogram dimension sizes.
@param type Histogram representation format. CV_HIST_ARRAY means that the histogram data is
represented as a multi-dimensional dense array CvMatND. CV_HIST_SPARSE means that histogram data
is represented as a multi-dimensional sparse array CvSparseMat.
@param ranges Array of ranges for the histogram bins. Its meaning depends on the uniform parameter
value. The ranges are used when the histogram is calculated or backprojected to determine which
histogram bin corresponds to which value/tuple of values from the input image(s).
@param uniform Uniformity flag. If not zero, the histogram has evenly spaced bins and for every
\f$0<=i<cDims\f$ ranges[i] is an array of two numbers: lower and upper boundaries for the i-th
histogram dimension. The whole range [lower,upper] is then split into dims[i] equal parts to
determine the i-th input tuple value ranges for every histogram bin. And if uniform=0 , then the
i-th element of the ranges array contains dims[i]+1 elements: \f$\texttt{lower}_0,
\texttt{upper}_0, \texttt{lower}_1, \texttt{upper}_1 = \texttt{lower}_2,
...
\texttt{upper}_{dims[i]-1}\f$ where \f$\texttt{lower}_j\f$ and \f$\texttt{upper}_j\f$ are lower
and upper boundaries of the i-th input tuple value for the j-th bin, respectively. In either
case, the input values that are beyond the specified range for a histogram bin are not counted
by cvCalcHist and filled with 0 by cvCalcBackProject. '/
declare function cvCreateHist( dims as long, sizes as long ptr, _type as long, _
                               ranges as single ptr ptr = NULL, _
                               uniform as long = 1) as CvHistogram ptr

/'  Sets the bounds of the histogram bins.

This is a standalone function for setting bin ranges in the histogram. For a more detailed
description of the parameters ranges and uniform, see the :ocvCalcHist function that can initialize
the ranges as well. Ranges for the histogram bins must be set before the histogram is calculated or
the backproject of the histogram is calculated.

@param hist Histogram.
@param ranges Array of bin ranges arrays. See :ocvCreateHist for details.
@param uniform Uniformity flag. See :ocvCreateHist for details.
 '/
declare sub cvSetHistBinRanges( hist as CvHistogram ptr, ranges as single ptr ptr, _
                                uniform as long = 1 )

/'  Makes a histogram out of an array.

The function initializes the histogram, whose header and bins are allocated by the user.
cvReleaseHist does not need to be called afterwards. Only dense histograms can be initialized this
way. The function returns hist.

@param dims Number of the histogram dimensions.
@param sizes Array of the histogram dimension sizes.
@param hist Histogram header initialized by the function.
@param data Array used to store histogram bins.
@param ranges Histogram bin ranges. See cvCreateHist for details.
@param uniform Uniformity flag. See cvCreateHist for details. '/
declare function cvMakeHistHeaderForArray( dims as long, sizes as long ptr, hist as CvHistogram ptr, _
                                           data as single ptr, ranges as single ptr ptr = NULL, _
                                           uniform as long = 1) as CvHistogram ptr

/' Releases the histogram.
   The function releases the histogram (header and the data). The pointer to the histogram is cleared
   by the function. If \*hist pointer is already NULL, the function does nothing.
   @param hist Double pointer to the released histogram. '/
declare sub cvReleaseHist( hist as CvHistogram ptr ptr )

/' Clears the histogram.
   The function sets all of the histogram bins to 0 in case of a dense histogram and removes all
   histogram bins in case of a sparse array.
   @param hist Histogram.  '/
declare sub cvClearHist( hist as CvHistogram ptr )

/' Finds the minimum and maximum histogram bins.

The function finds the minimum and maximum histogram bins and their positions. All of output
arguments are optional. Among several extremas with the same value the ones with the minimum index
(in the lexicographical order) are returned. In case of several maximums or minimums, the earliest
in the lexicographical order (extrema locations) is returned.

@param hist Histogram.
@param min_value Pointer to the minimum value of the histogram.
@param max_value Pointer to the maximum value of the histogram.
@param min_idx Pointer to the array of coordinates for the minimum.
@param max_idx Pointer to the array of coordinates for the maximum. '/
declare sub cvGetMinMaxHistValue( hist as const CvHistogram ptr, _
                                  min_value as single ptr, max_value as single ptr, _
                                  min_idx as long ptr = NULL, _
                                  max_idx as long ptr = NULL)

/' Normalizes the histogram.
   The function normalizes the histogram bins by scaling them so that the sum of the bins becomes equal
   to factor.
   @param hist Pointer to the histogram.
   @param factor Normalization factor. '/
declare sub cvNormalizeHist( hist as CvHistogram ptr, factor as double)


/' Thresholds the histogram.
   The function clears histogram bins that are below the specified threshold.
   @param hist Pointer to the histogram.
   @param threshold Threshold level. '/
declare sub cvThreshHist( hist as CvHistogram ptr, threshold as double)

' Compares two histogram
declare function cvCompareHist( hist1 as const CvHistogram ptr, _
                                hist2 as const CvHistogram ptr, _
                                method as long) as double

/' Copies a histogram.
   The function makes a copy of the histogram. If the second histogram pointer \*dst is NULL, a new
   histogram of the same size as src is created. Otherwise, both histograms must have equal types and
   sizes. Then the function copies the bin values of the source histogram to the destination histogram
   and sets the same bin value ranges as in src.
   @param src Source histogram.
   @param dst Pointer to the destination histogram.  '/
declare sub cvCopyHist( src as const CvHistogram ptr, dst as CvHistogram ptr ptr )


' Calculates bayesian probabilistic histograms
' (each or src and dst is an array of _number_ histograms
declare sub cvCalcBayesianProb( src as CvHistogram ptr ptr, number as long, _
                                dst as CvHistogram ptr ptr)

' Calculates array histogram
' see also: cv::calcHist
declare sub cvCalcArrHist( ar as CvArr ptr ptr, hist as CvHistogram ptr, _
                           accumulate as long = 0, _
                           mask as const CvArr ptr = NULL )

' overloaded
sub cvCalcHist( image as IplImage ptr ptr, hist as CvHistogram ptr, _
                accumulate as long = 0, _
                mask as const CvArr ptr = NULL ) 
  cvCalcArrHist( cptr (CvArr ptr ptr,image), hist, accumulate, mask )
end sub

' Calculates back project
' see also: cvCalcBackProject, cv::calcBackProject
declare sub cvCalcArrBackProject( image as CvArr ptr ptr, dst as CvArr ptr, _
                                  hist as const CvHistogram ptr )

#define  cvCalcBackProject(image, dst, hist) cvCalcArrBackProject((CvArr**)image, dst, hist)

/' Locates a template within an image by using a histogram comparison.

The function calculates the back projection by comparing histograms of the source image patches with
the given histogram. The function is similar to matchTemplate, but instead of comparing the raster
patch with all its possible positions within the search window, the function CalcBackProjectPatch
compares histograms. See the algorithm diagram below:

@param image Source images (though, you may pass CvMat as well).
@param dst Destination image.
@param range
@param hist Histogram.
@param method Comparison method passed to cvCompareHist (see the function description).
@param factor Normalization factor for histograms that affects the normalization scale of the
destination image. Pass 1 if not sure.
see also: cvCalcBackProjectPatch  '/
declare sub cvCalcArrBackProjectPatch( image as CvArr ptr ptr, dst as CvArr ptr, range as CvSize, _
                                        hist as CvHistogram ptr, method as long, _
                                        factor as double )

#define  cvCalcBackProjectPatch( image, dst, range, hist, method, factor ) _
  cvCalcArrBackProjectPatch( ptr( CvArr ptr ptr,image), dst, range, hist, method, factor )


/' Divides one histogram by another.
   The function calculates the object probability density from two histograms as:
   \f[\texttt{disthist} (I)= \forkthree{0}{if \(\texttt{hist1}(I)=0\)}{\texttt{scale}}{if \(\texttt{hist1}(I) \ne 0\) and \(\texttt{hist2}(I) > \texttt{hist1}(I)\)}{\frac{\texttt{hist2}(I) \cdot \texttt{scale}}{\texttt{hist1}(I)}}{if \(\texttt{hist1}(I) \ne 0\) and \(\texttt{hist2}(I) \le \texttt{hist1}(I)\)}\f]

@param hist1 First histogram (the divisor).
@param hist2 Second histogram.
@param dst_hist Destination histogram.
@param scale Scale factor for the destination histogram.
 '/
declare sub cvCalcProbDensity( hist1 as const CvHistogram ptr,  hist2 as const CvHistogram ptr, _
                               dst_hist as CvHistogram ptr, scale as double = 255 )

' equalizes histogram of 8-bit single-channel image
' see also: cv::equalizeHist
declare sub cvEqualizeHist( src as const CvArr ptr, dst as CvArr ptr )


' Applies distance transform to binary image
' see also: cv::distanceTransform
declare sub cvDistTransform( src as const CvArr ptr, dst as CvArr ptr, _
                             distance_type as long = CV_DIST_L2, _
                             mask_size as long = 3, _
                             mask as const single ptr = NULL, _
                             labels as CvArr ptr = NULL, _
                             labelType as long = CV_DIST_LABEL_CCOMP)


/' Applies fixed-level threshold to grayscale image.
  This is a basic operation applied before retrieving contours
  see also: cv::threshold '/
declare function cvThreshold( src as const CvArr ptr, dst as CvArr ptr, _
                              threshold as double, max_value as double, _
                              threshold_type as long) as double

/' Applies adaptive threshold to grayscale image.
  The two parameters for methods CV_ADAPTIVE_THRESH_MEAN_C and
  CV_ADAPTIVE_THRESH_GAUSSIAN_C are:
  neighborhood size (3, 5, 7 etc.),
  and a constant subtracted from mean (...,-3,-2,-1,0,1,2,3,...)
  see also: cv::adaptiveThreshold '/
declare sub cvAdaptiveThreshold( src as const CvArr ptr, _
                                 dst as CvArr ptr, _
                                 max_value as double, _
                                 adaptive_method as long = CV_ADAPTIVE_THRESH_MEAN_C, _
                                 threshold_type  as long = CV_THRESH_BINARY, _
                                 block_size as long = 3, _
                                 param1  as double  = 5)

/' Fills the connected component until the color difference gets large enough
  see also: cv::floodFill '/
declare sub cvFloodFill( image as CvArr ptr, seed_point as CvPoint, _
                         new_val as CvScalar, lo_diff as CvScalar = cvScalarAll(0), _
                         up_diff as CvScalar = cvScalarAll(0), _
                         comp as CvConnectedComp ptr = NULL, _
                         flags as long = 4, _
                         mask as CvArr ptr = NULL)

' #####################
' # Feature detection #
' #####################

' Runs canny edge detector
' see also: cv::Canny
declare sub cvCanny( image as const CvArr ptr, edges as CvArr ptr, _
                     threshold1 as double, threshold2 as double, _
                     aperture_size as long = 3 )

/' Calculates constraint image for corner detection
   Dx^2 * Dyy + Dxx * Dy^2 - 2 * Dx * Dy * Dxy.
   Applying threshold to the result gives coordinates of corners
   see also: cv::preCornerDetect '/
declare sub cvPreCornerDetect( image as const CvArr ptr, corners as CvArr ptr, _
                               aperture_size as long = 3 )

/' Calculates eigen values and vectors of 2x2
  gradient covariation matrix at every image pixel
  see also: cv::cornerEigenValsAndVecs '/
declare sub cvCornerEigenValsAndVecs( image as const CvArr ptr, eigenvv as CvArr ptr, _
                                      block_size as long, aperture_size as long = 3)

' Calculates minimal eigenvalue for 2x2 gradient covariation matrix at every image pixel
' see also: cv::cornerMinEigenVal
declare sub cvCornerMinEigenVal( image as const CvArr ptr, eigenval as CvArr ptr, _
                                 block_size as long, aperture_size as long = 3 )

/' Harris corner detector:
   Calculates det(M) - k*(trace(M)^2), where M is 2x2 gradient covariation matrix for each pixel 
   see also: cv::cornerHarris '/
declare sub cvCornerHarris( image as const CvArr ptr, harris_response as CvArr ptr, _
                            block_size as long, aperture_size as long = 3, _
                            k as double = 0.04 )

' Adjust corner position using some sort of gradient search
' see also: cv::cornerSubPix
declare sub cvFindCornerSubPix( image as const CvArr ptr, corners as CvPoint2D32f ptr, _
                                count as long, win as CvSize, zero_zone as CvSize, _
                                criteria as CvTermCriteria)

/' Finds a sparse set of points within the selected region
  that seem to be easy to track
  see also: cv::goodFeaturesToTrack '/
declare sub cvGoodFeaturesToTrack( image as const CvArr ptr, eig_image as CvArr ptr, _
                                   temp_image as CvArr ptr, corners as CvPoint2D32f ptr, _
                                   corner_count as long ptr, quality_level as long, _
                                   min_distance as double, mask as const CvArr ptr = NULL, _
                                   block_size as long = 3, use_harris as long = 0, _
                                   k as double = 0.04 )

/' Finds lines on binary image using one of several methods.
   line_storage is either memory storage or 1 x _max number of lines_ CvMat,
   its number of columns is changed by the function.
   method is one of CV_HOUGH_*  rho, theta and threshold are used for each of those methods
   param1 ~ line length, 
   param2 ~ line gap - for probabilistic,
   param1 ~ srn, 
   param2 ~ stn - for multi-scale
   see also: cv::HoughLines '/
declare function cvHoughLines2( image as CvArr ptr, line_storage as any ptr, method as long, _
                                rho as double, theta as double, threshold as long, _
                                param1 as double = 0, param2 as double = 0, _
                                min_theta as double = 0, max_theta as double = CV_PI) as CvSeq ptr

/' Finds circles in the image
  see also: cv::HoughCircles '/
declare function cvHoughCircles( image as CvArr ptr, circle_storage as any ptr, _
                                 method as long, dp as double, min_dist as double, _
                                 param1 as double = 100, param2 as double = 100, _
                                 min_radius as long = 0, max_radius as long = 0 ) as CvSeq ptr

/' Fits a line into set of 2d or 3d points in a robust way (M-estimator technique)
   see also: cv::fitLine '/
declare sub cvFitLine( points as const CvArr ptr, dist_type as long, _
                       param as double, reps as double, aeps as double, pline as single ptr)

end extern ' "C"


