#pragma once

#ifndef __OPENCV__
  #error 666: do not include this file manually include only 'opencv.bi' !
#endif


'#include "opencv_imgproc.bi"
'#include "opencv_features2d.bi
'#include "opencv_calib3d.bi"
'#include "opencv_ml.bi"


extern "C"

declare function cvSegmentImage( srcarr as const CvArr ptr, dstarr as CvArr ptr, _
                                 canny_threshold as double, _
                                 ffill_threshold as double , _
                                 storage as CvMemStorage ptr  ) as CvSeq ptr 

' #################
' # Eigen objects #
' #################

type CvCallback as function CV_CDECL( index as long, buffer as any ptr, user_data as any ptr) as long
union CvInput
  as CvCallback callback
  as any ptr data
end union

#define CV_EIGOBJ_NO_CALLBACK     0
#define CV_EIGOBJ_INPUT_CALLBACK  1
#define CV_EIGOBJ_OUTPUT_CALLBACK 2
#define CV_EIGOBJ_BOTH_CALLBACK   3

' Calculates covariation matrix of a set of arrays
declare sub      cvCalcCovarMatrixEx( nObjects as long, pinput as any ptr, ioFlags as long, _
                                      ioBufSize as long, buffer as ubyte ptr, userData as any ptr, _
                                      avg as IplImage ptr, covarMatrix as single ptr)

' Calculates eigen values and vectors of covariation matrix of a set of arrays
declare sub      cvCalcEigenObjects( nObjects as long, pinput as any ptr, poutput as any ptr, _
                                     ioFlags as long, ioBufSize as long, userData as any ptr, _
                                     calcLimit as CvTermCriteria ptr, avg as IplImage ptr, _
                                     eigVals as single ptr)

' Calculates dot product (obj - avg) * eigObj (i.e. projects image to eigen vector)
declare function cvCalcDecompCoeff( obj as IplImage ptr, eigObj as IplImage ptr, avg as IplImage ptr ) as double

' Projects image to eigen space (finds all decomposion coefficients
declare sub      cvEigenDecomposite( obj as IplImage ptr, nEigObjs as long, eigInput as any ptr, _
                                     ioFlags as long, userData as any ptr, avg as IplImage ptr, _
                                     coeffs as single ptr)

' Projects original objects used to calculate eigen space basis to that space
declare sub      cvEigenProjection( eigInput as any ptr, nEigObjs as long, ioFlags as long, _
                                    userData as any ptr, coeffs as single ptr, avg as IplImage ptr, _
                                    proj as IplImage ptr)

' #############
' # 1D/2D HMM #
' #############
type CvImgObsInfo
  as long       obs_x
  as long       obs_y
  as long       obs_size
  as single ptr obs      ' consequtive observations
  as long ptr   state    ' arr of pairs superstate/state to which observation belong
  as long ptr   mix      ' number of mixture to which observation belong
end type

type CvEHMMState
  as long       num_mix     ' number of mixtures in this state
  as single ptr mu          ' mean vectors corresponding to each mixture
  as single ptr inv_var     ' square root of inversed variances corresp. to each mixture
  as single ptr log_var_val ' sum of 0.5 (LN2PI + ln(variance[i]) ) for i=1,n
  as single ptr weight      ' array of mixture weights. Summ of all weights in state is 1.
end type

type _CvEHMM as CvEHMM
union _CvEHMMunion
  as CvEHMMState ptr state ' if level == 0 points to real states array,
                           ' if not - points to embedded hmms
  as _CvEHMM     ptr ehmm  ' pointer to an embedded model or NULL, if it is a leaf
end union

type CvEHMM
  as long           level      ' 0 - lowest(i.e its states are real states), .....
  as long           num_states ' number of HMM states
  as single ptr     transP     ' transition probab. matrices for states
  as single ptr ptr obsProb    ' if level == 0 - array of brob matrices corresponding to hmm
                               ' if level == 1 - martix of matrices
  as _CvEHMMunion u
end type

' #################
' # Embedded HMMs #
' #################

' Creates 2D HMM
declare function cvCreate2DHMM( stateNumber as long ptr, numMix as long ptr, obsSize as long ) as CvEHMM ptr

' Releases HMM
declare sub      cvRelease2DHMM( hmm as CvEHMM ptr ptr )

#macro CV_COUNT_OBS(roi, win, delta, numObs )
   (numObs)->width  =((roi)->width  -(win)->width  +(delta)->width)/(delta)->width
   (numObs)->height =((roi)->height -(win)->height +(delta)->height)/(delta)->height
#endmacro

' Creates storage for observation vectors
declare function cvCreateObsInfo( numObs as CvSize, obsSize as long ) as CvImgObsInfo ptr

' Releases storage for observation vectors
declare sub      cvReleaseObsInfo( obs_info as CvImgObsInfo ptr ptr )

' The function takes an image on input and and returns the sequnce of observations
' to be used with an embedded HMM; Each observation is top-left block of DCT coefficient matrix
declare sub      cvImgToObs_DCT( arr as const CvArr ptr, obs as single ptr, dctSize as CvSize, obsSize as CvSize, delta as CvSize)


' Uniformly segments all observation vectors extracted from image
declare sub      cvUniformImgSegm( obs_info as CvImgObsInfo ptr, ehmm as CvEHMM ptr )

' Does mixture segmentation of the states of embedded HMM
declare sub      cvInitMixSegm( obs_info_array as CvImgObsInfo ptr ptr, num_img as long, ehmm as CvEHMM ptr )

' Function calculates means, variances, weights of every Gaussian mixture
' of every low-level state of embedded HMM
declare sub      cvEstimateHMMStateParams( obs_info_array as CvImgObsInfo ptr ptr, num_img as long, ehmm as CvEHMM ptr )

' Function computes transition probability matrices of embedded HMM given observations segmentation
declare sub      cvEstimateTransProb( obs_info_array as CvImgObsInfo ptr ptr, num_img as long, ehmm as CvEHMM ptr )

' Function computes probabilities of appearing observations at any state
' (i.e. computes P(obs|state) for every pair(obs,state))
declare sub      cvEstimateObsProb( obs_info as CvImgObsInfo ptr, ehmm as CvEHMM ptr )

' Runs Viterbi algorithm for embedded HMM
declare function cvEViterbi( obs_info as CvImgObsInfo ptr, ehmm as CvEHMM ptr ) as single

' Function clusters observation vectors from several images given observations segmentation.
' Euclidean distance used for clustering vectors.
' Centers of clusters are given means of every mixture
declare sub      cvMixSegmL2( obs_info_array as CvImgObsInfo ptr ptr, num_img as long, ehmm as CvEHMM ptr )

' #################################################################
' # A few functions from old stereo gesture recognition demosions #
' #################################################################

' Creates hand mask image given several points on the hand
declare sub      cvCreateHandMask( hand_points as CvSeq ptr, img_mask as IplImage ptr, roi as CvRect ptr)

' Finds hand region in range image data
declare sub      cvFindHandRegion ( points as CvPoint3D32f ptr, count as long, _
                                    indexs as CvSeq ptr, _
                                    pline as single ptr, size as CvSize2D32f, flag as long, _
                                    center as CvPoint3D32f ptr, _
                                    storage as CvMemStorage ptr, numbers as CvSeq ptr ptr)

' Finds hand region in range image data (advanced version)
declare sub      cvFindHandRegionA( points as CvPoint3D32f ptr, count as long, _
                                    indexs as CvSeq ptr, _
                                    pline as single ptr, size as CvSize2D32f, jc as long, _
                                    center as CvPoint3D32f ptr, _
                                    storage as CvMemStorage ptr, numbers as CvSeq ptr)

' Calculates the cooficients of the homography matrix
declare sub      cvCalcImageHomography( pline as single ptr, center as CvPoint3D32f ptr, _
                                        intrinsic as single ptr, homography as single ptr)

' ################################
' # More operations on sequences #
' ################################

#define CV_CURRENT_INT( reader ) *(cptr(long ptr,(reader)._ptr))
#define CV_PREV_INT( reader ) *(cptr(long ptr,(reader).prev_elem))

#define  CV_GRAPH_WEIGHTED_VERTEX_FIELDS as single weight

#define  CV_GRAPH_WEIGHTED_EDGE_FIELDS CV_GRAPH_EDGE_FIELDS

type CvGraphWeightedVtx
  CV_GRAPH_WEIGHTED_VERTEX_FIELDS
end type

type CvGraphWeightedEdge
  CV_GRAPH_WEIGHTED_EDGE_FIELDS
end type

enum CvGraphWeightType
  CV_NOT_WEIGHTED,
  CV_WEIGHTED_VTX,
  CV_WEIGHTED_EDGE,
  CV_WEIGHTED_ALL
end enum


' Calculates histogram of a contour
declare sub      cvCalcPGH( contour as const CvSeq ptr, hist as CvHistogram ptr )

#define CV_DOMINANT_IPAN 1

' Finds high-curvature points of the contour
declare function cvFindDominantPoints( contour    as CvSeq ptr, _
                                       storage    as CvMemStorage ptr, _
                                       method     as long CV_DEFAULT(CV_DOMINANT_IPAN), _
                                       parameter1 as double CV_DEFAULT(0), _
                                       parameter2 as double CV_DEFAULT(0), _
                                       parameter3 as double CV_DEFAULT(0), _
                                       parameter4 as double CV_DEFAULT(0) ) as CvSeq ptr 

' #########################
' # Stereo correspondence #
' #########################

#define CLIQUE_TIME_OFF 2
#define CLIQUE_FOUND 1
#define CLIQUE_END   0

#define CV_UNDEF_SC_PARAM         12345 ' default value of parameters

#define CV_IDP_BIRCHFIELD_PARAM1  25
#define CV_IDP_BIRCHFIELD_PARAM2  5
#define CV_IDP_BIRCHFIELD_PARAM3  12
#define CV_IDP_BIRCHFIELD_PARAM4  15
#define CV_IDP_BIRCHFIELD_PARAM5  25


#define  CV_DISPARITY_BIRCHFIELD  0



' Name:    cvFindStereoCorrespondence
' Purpose: find stereo correspondence on stereo-pair
' Context:
' Parameters:
'   leftImage - left image of stereo-pair (format 8uC1).
'   rightImage - right image of stereo-pair (format 8uC1).
'   mode - mode of correspondence retrieval (now CV_DISPARITY_BIRCHFIELD only)
'   dispImage - destination disparity image
'   maxDisparity - maximal disparity
'   param1, param2, param3, param4, param5 - parameters of algorithm
' Returns:
' Notes:
'   Images must be rectified.
'   All images must have format 8uC1.
declare sub      cvFindStereoCorrespondence( leftImage  as const CvArr ptr, _
                                             rightImage as const CvArr ptr, _
                                             mode as long, _
                                             dispImage as CvArr ptr, _
                                             maxDisparity as long, _
                                             param1 as double CV_DEFAULT(CV_UNDEF_SC_PARAM), _
                                             param2 as double CV_DEFAULT(CV_UNDEF_SC_PARAM), _
                                             param3 as double CV_DEFAULT(CV_UNDEF_SC_PARAM), _
                                             param4 as double CV_DEFAULT(CV_UNDEF_SC_PARAM), _
                                             param5 as double CV_DEFAULT(CV_UNDEF_SC_PARAM) )
' #####################
' # Epiline functions #
' #####################
type CvStereoLineCoeff
  as double Xcoef
  as double XcoefA
  as double XcoefB
  as double XcoefAB

  as double Ycoef
  as double YcoefA
  as double YcoefB
  as double YcoefAB

  as double Zcoef
  as double ZcoefA
  as double ZcoefB
  as double ZcoefAB
end type

type CvCamera
  as single imgSize(1)    ' size of the camera view, used during calibration 
  as single matrix(8)     ' intinsic camera parameters:  [ fx 0 cx; 0 fy cy; 0 0 1 ]
  as single distortion(3) ' distortion coefficients - two coefficients for radial distortion
                          ' and another two for tangential: [ k1 k2 p1 p2 ]
  as single rotMatr(8)
  as single transVect(2)  ' rotation matrix and transition vector relatively
                          ' to some reference point in the space.
end type

type CvStereoCamera
  as CvCamera ptr camera(1)       ' two individual camera parameters
  as single       fundMatr(8)     ' fundamental matrix

  ' New part for stereo
  as CvPoint3D32f epipole(1)
  as CvPoint2D32f quad(1,3)       ' coordinates of destination quadrangle after
                                  ' epipolar geometry rectification
  as double       coeffs(1,2,2)   ' coefficients for transformation
  as CvPoint2D32f border(1,3)
  as CvSize       warpSize
  as CvStereoLineCoeff ptr lineCoeffs
  as long         needSwapCameras ' flag set to 1 if need to swap cameras for good reconstruction
  as single       rotMatrix(8)
  as single       transVector(2)
end type

type CvContourOrientation
  as single egvals(1)
  as single egvects(3)
  as single max, min    ' minimum and maximum projections
  as long imax, imin
end type


#define CV_CAMERA_TO_WARP 1
#define CV_WARP_TO_CAMERA 2

declare function icvConvertWarpCoordinates( coeffs(any,any) as double, _
                                            cameraPoint as CvPoint2D32f ptr, _
                                            warpPoint   as CvPoint2D32f ptr, _
                                            directionas as long ) as long

declare function icvGetSymPoint3D( pointCorner as CvPoint3D64f, _
                                   point1      as CvPoint3D64f, _
                                   point2      as CvPoint3D64f, _
                                   pointSym2   as CvPoint3D64f ptr ) as long

declare sub      icvGetPieceLength3D( point1 as CvPoint3D64f, _
                                      point2 as CvPoint3D64f, _
                                      dist   as double ptr)

declare function icvCompute3DPoint( alpha_ as double, _
                                    betta  as double, _
                                    coeffs as CvStereoLineCoeff ptr, _
                                    ppoint as CvPoint3D64f ptr ) as long

declare function icvCreateConvertMatrVect( rotMatr1 as double ptr, _
                                           transVect1 as double ptr, _
                                           rotMatr2 as double ptr, _
                                           transVect2 as double ptr, _
                                           convRotMatr as double ptr, _
                                           convTransVect as double ptr) as long

declare function icvConvertPointSystem( M1 as CvPoint3D64f, _
                                        M2 as CvPoint3D64f ptr, _
                                        rotMatr   as double ptr, _
                                        transVect as double ptr) as long

declare function icvComputeCoeffForStereo( stereoCamera as CvStereoCamera ptr) as long

declare function icvGetCrossPieceVector( p1_start as CvPoint2D32f, _
                                         p1_end   as CvPoint2D32f, _
                                         v2_start as CvPoint2D32f, _
                                         v2_end   as CvPoint2D32f, _
                                         cross    as CvPoint2D32f ptr ) as long
declare function icvGetCrossLineDirect( p1 as CvPoint2D32f, _
                                        p2 as CvPoint2D32f, _
                                        a  as single, b as single, c as single, _
                                        vros as CvPoint2D32f ptr ) as long
declare function icvDefinePointPosition( point1 as CvPoint2D32f, _
                                         point2 as CvPoint2D32f, _
                                         point_ as CvPoint2D32f ) as single
declare function icvStereoCalibration( numImages as long, _
                                       nums  as long ptr, _
                                       imageSize as CvSize, _
                                       imagePoints1 as CvPoint2D32f ptr, _
                                       imagePoints2 as CvPoint2D32f ptr, _
                                       objectPoints as CvPoint2D32f ptr, _
                                       stereoparams as CvStereoCamera ptr ) as long


declare function icvComputeRestStereoParams(stereoparams as CvStereoCamera ptr) as long

declare sub      cvComputePerspectiveMap( coeffs(any,any) as const double, rectMapX as CvArr ptr, rectMapY as CvArr ptr )

declare function icvComCoeffForLine( point1 as CvPoint2D64f, _
                                     point2 as CvPoint2D64f, _
                                     point3 as CvPoint2D64f, _
                                     point4 as CvPoint2D64f, _
                                     camMatr1 as double ptr, _
                                     rotMatr1 as double ptr, _
                                     transVect1 as double ptr, _
                                     camMatr2 as double ptr, _
                                     rotMatr2 as double ptr, _
                                     transVect2 as double ptr, _
                                     coeffs as CvStereoLineCoeff ptr, _
                                     needSwapCameras as long ptr ) as long

declare function icvGetDirectionForPoint( point_  as CvPoint2D64f, _
                                          camMatr as double ptr, _
                                          direct  as CvPoint3D64f ptr ) as long

declare function icvGetCrossLines( point11  as CvPoint3D64f, _
                                   point12  as CvPoint3D64f, _
                                   point21  as CvPoint3D64f, _
                                   point22  as CvPoint3D64f, _
                                   midPoint as CvPoint3D64f ptr ) as long

declare function icvComputeStereoLineCoeffs( pointA as CvPoint3D64f, _
                                             pointB as CvPoint3D64f, _
                                             pointCam1 as CvPoint3D64f, _
                                             gamma as double, _
                                             coeffs as CvStereoLineCoeff ptr) as long


declare function icvGetAngleLine( startPoint as CvPoint2D64f,  _
                                  imageSize  as CvSize, _
                                  point1     as CvPoint2D64f ptr, _
                                  point2     as CvPoint2D64f ptr) as long

declare sub      icvGetCoefForPiece( p_start as CvPoint2D64f, p_end as CvPoint2D64f, _
                                     a as double, b as double ptr, c as double ptr, result as long ptr)


declare sub      icvComputeeInfiniteProject1( rotMatr  as double ptr, _
                                              camMatr1 as double ptr, _
                                              camMatr2 as double ptr, _
                                              point1   as CvPoint2D32f, _
                                              point2   as CvPoint2D32f ptr)

declare sub      icvComputeeInfiniteProject2( rotMatr  as double ptr, _
                                              camMatr1 as double ptr, _
                                              camMatr2 as double ptr, _
                                              point1   as CvPoint2D32f ptr, _
                                              point2   as CvPoint2D32f)

declare sub      icvGetCrossDirectDirect( direct1 as double ptr, _
                                          direct2 as double ptr, _
                                          cross   as CvPoint2D64f ptr, _
                                          result  as long ptr)

declare sub      icvGetCrossPieceDirect( p_start as CvPoint2D64f, p_end as CvPoint2D64f, _
                                         a as double, b as double, c as double, _
                                         cross as CvPoint2D64f ptr, result as long ptr)

declare sub      icvGetCrossPiecePiece( p_start as CvPoint2D64f, p_end as CvPoint2D64f, _
                                        p2_start as CvPoint2D64f, p2_end as CvPoint2D64f, _
                                        cross as CvPoint2D64f ptr, result as long ptr)

declare sub      icvGetPieceLength(point1 as CvPoint2D64f, point2 as CvPoint2D64f, dist as double ptr)

declare sub      icvGetCrossRectDirect( imageSize as CvSize, _
                                        a as double, b as double, c as double, _
                                        start  as CvPoint2D64f ptr, _
                                        end_   as CvPoint2D64f ptr, _
                                        result as long ptr)

declare sub      icvProjectPointToImage( point_    as CvPoint3D64f, _
                                         camMatr   as double ptr, _
                                         rotMatr   as double ptr, _
                                         transVect as double ptr, _
                                         projPoint as CvPoint2D64f ptr )

declare sub      icvGetQuadsTransform( imageSize  as CvSize, _
                                       camMatr1   as double ptr, _
                                       rotMatr1   as double ptr, _
                                       transVect1 as double ptr, _
                                       camMatr2   as double ptr, _
                                       rotMatr2   as double ptr, _
                                       transVect2 as double ptr, _
                                       warpSize   as CvSize ptr, _
                                       quad1(any,any) as double, _
                                       quad2(any,any) as double, _
                                       fundMatr   as double ptr, _
                                       epipole1   as CvPoint3D64f ptr, _
                                       epipole2   as CvPoint3D64f ptr)

declare sub      icvGetQuadsTransformStruct( stereoCamera as CvStereoCamera ptr )

declare sub      icvComputeStereoParamsForCameras( stereoCamera as CvStereoCamera ptr )

declare sub      icvGetCutPiece( areaLineCoef1 as double ptr, _
                                 areaLineCoef2 as double ptr, _
                                 epipole       as CvPoint2D64f, _
                                 imageSize     as CvSize, _
                                 point11       as CvPoint2D64f ptr, _
                                 point12       as CvPoint2D64f ptr, _
                                 point21       as CvPoint2D64f ptr, _
                                 point22       as CvPoint2D64f ptr, _
                                 result        as long ptr)

declare sub      icvGetMiddleAnglePoint( basePoint as CvPoint2D64f, _
                                         point1    as CvPoint2D64f, _
                                         point2    as CvPoint2D64f, _
                                         midPoint  as CvPoint2D64f ptr)

declare sub      icvGetNormalDirect( direct     as double ptr, _
                                     point_     as CvPoint2D64f, _
                                     normDirect as double ptr )

declare function icvGetVect( basePoint as CvPoint2D64f, _
                             point1    as CvPoint2D64f, _
                             point2    as CvPoint2D64f) as double

declare sub      icvProjectPointToDirect( point_       as CvPoint2D64f, _
                                          lineCoeff    as double ptr, _
                                          projectPoint as CvPoint2D64f ptr)

declare sub      icvGetDistanceFromPointToDirect( point_   as CvPoint2D64f, _
                                                  lineCoef as double ptr, _
                                                  dist     as double ptr)

declare function icvCreateIsometricImage( src as IplImage ptr, _
                                          dst as IplImage ptr, _
                                          desired_depth as long, _
                                          desired_num_channels as long) as IplImage ptr

declare sub      cvDeInterlace( frame as const CvArr ptr, fieldEven as CvArr ptr, fieldOdd as CvArr ptr)


' ################
' # Contour Tree #
' ################

' Contour tree header
type CvContourTree
  CV_SEQUENCE_FIELDS
  as CvPoint p1      ' the first point of the binary tree root segment
  as CvPoint p2      ' the last point of the binary tree root segment
end type

' Builds hierarhical representation of a contour
declare function cvCreateContourTree( contour as const CvSeq ptr, _
                                      storage as CvMemStorage ptr, _
                                      threshold as double ) as CvContourTree ptr

' Reconstruct (completelly or partially) contour a from contour tree
declare function cvContourFromContourTree( tree as const CvContourTree ptr, _
                                           storage as CvMemStorage ptr, _
                                           criteria as CvTermCriteria ) as CvSeq ptr

' Compares two contour trees
enum 
  CV_CONTOUR_TREES_MATCH_I1 = 1
end enum  

declare function cvMatchContourTrees( tree1 as const CvContourTree ptr, _
                                      tree2 as const CvContourTree ptr, _
                                      method as long, threshold as double ) as double

' ####################
' # Contour Morphing #
' ####################
' finds correspondence between two contours
declare function cvCalcContoursCorrespondence( contour1 as const CvSeq ptr, _
                                               contour2 as const CvSeq ptr, _
                                               storage  as CvMemStorage ptr ) as CvSeq ptr

' morphs contours using the pre-calculated correspondence:
' alpha=0 ~ contour1, alpha=1 ~ contour2
declare function cvMorphContours( contour1 as const CvSeq ptr, _
                                  contour2 as const CvSeq ptr, _
                                  corr     as CvSeq ptr, _
                                  alpha_   as double, _
                                  storage  as CvMemStorage ptr ) as CvSeq ptr

' ###################
' # Active Contours #
' ###################

#define  CV_VALUE  1
#define  CV_ARRAY  2
' Updates active contour in order to minimize its cummulative
' (internal and external) energy.
declare sub      cvSnakeImage( image as const IplImage ptr, _
                               points as CvPoint ptr, _
                               length as long, _
                               alpha_ as single ptr, _
                               betha  as single ptr, _
                               gamma  as single ptr, _
                               coeff_usage as long, win as CvSize, _
                               criteria as CvTermCriteria, _
                               calc_gradient as long CV_DEFAULT(1) )


' #######################
' # Texture Descriptors #
' #######################

#define CV_GLCM_OPTIMIZATION_NONE                   -2
#define CV_GLCM_OPTIMIZATION_LUT                    -1
#define CV_GLCM_OPTIMIZATION_HISTOGRAM              0

#define CV_GLCMDESC_OPTIMIZATION_ALLOWDOUBLENEST    10
#define CV_GLCMDESC_OPTIMIZATION_ALLOWTRIPLENEST    11
#define CV_GLCMDESC_OPTIMIZATION_HISTOGRAM          4

#define CV_GLCMDESC_ENTROPY                         0
#define CV_GLCMDESC_ENERGY                          1
#define CV_GLCMDESC_HOMOGENITY                      2
#define CV_GLCMDESC_CONTRAST                        3
#define CV_GLCMDESC_CLUSTERTENDENCY                 4
#define CV_GLCMDESC_CLUSTERSHADE                    5
#define CV_GLCMDESC_CORRELATION                     6
#define CV_GLCMDESC_CORRELATIONINFO1                7
#define CV_GLCMDESC_CORRELATIONINFO2                8
#define CV_GLCMDESC_MAXIMUMPROBABILITY              9

#define CV_GLCM_ALL                                 0
#define CV_GLCM_GLCM                                1
#define CV_GLCM_DESC                                2

type CvGLCM as any

declare function cvCreateGLCM( srcImage          as const IplImage ptr, _
                               stepMagnitude     as long, _
                               stepDirections    as const long ptr CV_DEFAULT(0), _
                               numStepDirections as long CV_DEFAULT(0), _
                               optimizationType  as long CV_DEFAULT(CV_GLCM_OPTIMIZATION_NONE) ) as CvGLCM ptr

declare sub      cvReleaseGLCM( GLCMas as CvGLCM ptr ptr, flag as long CV_DEFAULT(CV_GLCM_ALL) )

declare sub      cvCreateGLCMDescriptors( destGLCM as CvGLCM ptr, _
                                          descriptorOptimizationType as long CV_DEFAULT(CV_GLCMDESC_OPTIMIZATION_ALLOWDOUBLENEST) )

declare function cvGetGLCMDescriptor( GLCMas as CvGLCM ptr, step_ as long, descriptor as long) as double

declare sub      cvGetGLCMDescriptorStatistics( GLCM as CvGLCM ptr, descriptor as long, _
                                                average as double ptr, standardDeviation as double ptr)

declare function cvCreateGLCMImage( GLCM as CvGLCM ptr, step_ as long ) as IplImage ptr

' ################################
' # Face eyes and mouth tracking #
' ################################

type CvFaceTracker as any 

#define CV_NUM_FACE_ELEMENTS    3
enum CV_FACE_ELEMENTS
  CV_FACE_MOUTH     = 0,
  CV_FACE_LEFT_EYE  = 1,
  CV_FACE_RIGHT_EYE = 2
end enum

declare function cvInitFaceTracker(pFaceTracking as CvFaceTracker ptr, _
                                        imgGray as const IplImage ptr, _
                                        pRects  as CvRect ptr, nRects as long) as CvFaceTracker ptr
declare function cvTrackFace( pFaceTracker as CvFaceTracker ptr, imgGray as IplImage ptr, _
                              pRects as CvRect ptr, nRects as long, _
                              ptRotate as CvPoint ptr, dbAngleRotate as double ptr) as long
declare sub      cvReleaseFaceTracker(ppFaceTracker as CvFaceTracker ptr ptr)

#if 0 ' internal use only
type CvFace
  as  CvRect MouthRect
  as  CvRect LeftEyeRect
  as  CvRect RightEyeRect
end type
type CvFaceData as CvFace
#endif
declare function cvFindFace(Image as IplImage ptr, storage as CvMemStorage ptr ) as CvSeq ptr 
declare function cvPostBoostingFindFace( Image as IplImage ptr, storage as CvMemStorage ptr ) as CvSeq ptr


' ##############
' # 3D Tracker #
' ##############

type CvBool as boolean ' as ubyte

type Cv3dTracker2dTrackedObject
  as long         id
  as CvPoint2D32f p  ' pgruebele: So we do not loose precision, this needs to be float
end type
' renamed: cv3dTracker2dTrackedObject
function cv_3dTracker2dTrackedObject(id as long, p as CvPoint2D32f) as Cv3dTracker2dTrackedObject
  dim as Cv3dTracker2dTrackedObject r
  r.id = id
  r.p  = p
  return r
end function

type Cv3dTrackerTrackedObject
  as long         id
  as CvPoint3D32f p ' location of the tracked object
end type
' renamed: cv3dTrackerTrackedObject
function cv_3dTrackerTrackedObject(id as long, p as CvPoint3D32f) as Cv3dTrackerTrackedObject
  dim as Cv3dTrackerTrackedObject r
  r.id = id
  r.p = p
  return r
end function

type Cv3dTrackerCameraInfo
  as CvBool valid
  as single mat(3,3)              ' maps camera coordinates to world coordinates
  as CvPoint2D32f principal_point ' copied from intrinsics so this structure has all the info we need
end type

type Cv3dTrackerCameraIntrinsics
  as CvPoint2D32f principal_point
  as single focal_length(1)
  as single distortion(3)
end type

declare function cv3dTrackerCalibrateCameras( num_cameras as long, _
                                              camera_intrinsics() as const Cv3dTrackerCameraIntrinsics, _' size is num_cameras
                                              etalon_size   as CvSize, _
                                              square_size   as single, _
                                              samples()     as IplImage ptr, _ ' size is num_cameras
                                              camera_info() as Cv3dTrackerCameraInfo ) as CvBool ' size is num_cameras

' tracking_info is a rectangular array
' one row per camera, num_objects elements per row.
' The id field of any unused slots must be -1. Ids need not be ordered or consecutive.
' On completion, the return value is the number of objects located
' i.e., the number of objects visible by more than one camera. 
' The id field of any unused slots in tracked objects is set to -1.

declare function cv3dTrackerLocateObjects( num_cameras as long, num_objects as long, _
                                           camera_info() as const Cv3dTrackerCameraInfo , _         ' size is num_cameras
                                           tracking_info() as const Cv3dTracker2dTrackedObject , _  ' size is num_objects * num_cameras
                                           tracked_objects() as Cv3dTrackerTrackedObject ) as long  ' size is num_objects



' #######################################
' # Skeletons and Linear-Contour Models #
' #######################################

enum CvLeeParameters
  CV_LEE_INT = 0,
  CV_LEE_FLOAT = 1,
  CV_LEE_DOUBLE = 2,
  CV_LEE_AUTO = -1,
  CV_LEE_ERODE = 0,
  CV_LEE_ZOOM = 1,
  CV_LEE_NON = 2
end enum

#define CV_NEXT_VORONOISITE2D( SITE ) ((SITE)->edge(0)->site(((SITE)->edge(0)->site(0) = (SITE))))
#define CV_PREV_VORONOISITE2D( SITE ) ((SITE)->edge(1)->site(((SITE)->edge(1)->site(0) = (SITE))))
#define CV_FIRST_VORONOIEDGE2D( SITE ) ((SITE)->edge(0))
#define CV_LAST_VORONOIEDGE2D( SITE ) ((SITE)->edge(1))
#define CV_NEXT_VORONOIEDGE2D( EDGE, SITE ) ((EDGE)->next((EDGE)->site(0) <> (SITE)))
#define CV_PREV_VORONOIEDGE2D( EDGE, SITE ) ((EDGE)->next(2 + ((EDGE)->site(0) <> (SITE))))
#define CV_VORONOIEDGE2D_BEGINNODE( EDGE, SITE ) ((EDGE)->node(((EDGE)->site(0) <> (SITE))))
#define CV_VORONOIEDGE2D_ENDNODE( EDGE, SITE ) ((EDGE)->node(((EDGE)->site(0) = (SITE))))
#define CV_TWIN_VORONOISITE2D( SITE, EDGE ) ( (EDGE)->site(((EDGE)->site(0) = (SITE))))

type _CvVoronoiNode2D as CvVoronoiNode2D
type _CvVoronoiEdge2D as CvVoronoiEdge2D

#macro CV_VORONOISITE2D_FIELDS
  as _CvVoronoiNode2D ptr node(1)
  as _CvVoronoiEdge2D ptr edge(1)
#endmacro

type CvVoronoiSite2D
  CV_VORONOISITE2D_FIELDS
  as CvVoronoiSite2D ptr next(1)
end type

#macro CV_VORONOIEDGE2D_FIELDS
  as _CvVoronoiNode2D ptr node(1)
  as CvVoronoiSite2D ptr site(1)
  as _CvVoronoiEdge2D ptr next(3)
#endmacro
type CvVoronoiEdge2D
  CV_VORONOIEDGE2D_FIELDS
end type

#macro CV_VORONOINODE2D_FIELDS
  CV_SET_ELEM_FIELDS(CvVoronoiNode2D)
  as CvPoint2D32f pt
  as single radius
#endmacro
type CvVoronoiNode2D
  CV_VORONOINODE2D_FIELDS
end type

#macro CV_VORONOIDIAGRAM2D_FIELDS
  CV_GRAPH_FIELDS
  as CvSet ptr sites
#endmacro
type CvVoronoiDiagram2D
  CV_VORONOIDIAGRAM2D_FIELDS
end type


' Computes Voronoi Diagram for given polygons with holes
declare function cvVoronoiDiagramFromContour(ContourSeq as CvSeq ptr, _
                                             VoronoiDiagram as CvVoronoiDiagram2D ptr ptr, _
                                             VoronoiStorage as CvMemStorage ptr, _
                                             contour_type as CvLeeParameters CV_DEFAULT(CV_LEE_INT), _
                                             contour_orientation as long CV_DEFAULT(-1), _
                                             attempt_number as long CV_DEFAULT(10)) as long

' Computes Voronoi Diagram for domains in given image
declare function cvVoronoiDiagramFromImage(pImage as IplImage ptr, _
                                           ContourSeq as CvSeq ptr ptr, _
                                           VoronoiDiagram as CvVoronoiDiagram2D ptr ptr, _
                                           VoronoiStorage as CvMemStorage ptr, _
                                           regularization_method as CvLeeParameters CV_DEFAULT(CV_LEE_NON), _
                                           approx_precision as single CV_DEFAULT(-1)) as long

' Deallocates the storage
declare sub      cvReleaseVoronoiStorage( VoronoiDiagram as CvVoronoiDiagram2D ptr, _
                                          pVoronoiStorage as CvMemStorage ptr )
' ########################
' # Linear-Contour Model #
' ########################

type CvLCMEdge
  CV_GRAPH_EDGE_FIELDS
  as CvSeq ptr chain
  as single    width
  as long      index1
  as long      index2
end type

type CvLCMNode
  CV_GRAPH_VERTEX_FIELDS
  as CvContour ptr contour
end type


' Computes hybrid model from Voronoi Diagram
declare function cvLinearContorModelFromVoronoiDiagram( VoronoiDiagram as CvVoronoiDiagram2D ptr, _
                                                        maxWidth as single ) as CvGraph ptr

' Releases hybrid model storage
declare function cvReleaseLinearContorModelStorage( Graph as CvGraph ptr ) as long


' two stereo-related functions (mat(3,3))
declare sub      cvInitPerspectiveTransform( size            as CvSize, _
                                             vertex()        as const CvPoint2D32f, _
                                             matrix(any,any) as double, _
                                             rectMap         as CvArr ptr  )

' ###########################
' # View Morphing Functions #
' ###########################

type CvMatrix3
  as single m(2,2)
end type

' The order of the function corresponds to the order 
' they should appear in the view morphing pipeline

' Finds ending points of scanlines on left and right images of stereo-pair
declare sub      cvMakeScanlines( matrix     as const CvMatrix3 ptr, _
                                  img_size   as CvSize, _
                                  scanlines1 as long ptr, _
                                  scanlines2 as long ptr, _
                                  lengths1   as long ptr, _
                                  lengths2   as long ptr, _
                                  line_count as long ptr )

' Grab pixel values from scanlines and stores them sequentially
' (some sort of perspective image transform)
declare sub      cvPreWarpImage( line_count as long, _
                                 img        as IplImage ptr, _
                                 dst        as ubyte ptr, _
                                 dst_nums   as long ptr, _
                                 scanlines  as long ptr)

' Approximate each grabbed scanline by a sequence of runs
' (lossy run-length compression)
declare sub      cvFindRuns( line_count    as long, _
                             prewarp1      as ubyte ptr, _
                             prewarp2      as ubyte ptr, _
                             line_lengths1 as long ptr, _
                             line_lengths2 as long ptr, _
                             runs1         as long ptr, _
                             runs2         as long ptr, _
                             num_runs1     as long ptr, _
                             num_runs2     as long ptr )

'Compares two sets of compressed scanlines
declare sub      cvDynamicCorrespondMulti( line_count  as long, _
                                           first       as long ptr, _
                                           first_runs  as long ptr, _
                                           second_     as long ptr, _
                                           second_runs as long ptr, _
                                           first_corr  as long ptr, _
                                           second_corr as long ptr)

' Finds scanline ending coordinates for some intermediate "virtual" camera position
declare sub      cvMakeAlphaScanlines( scanlines1 as long ptr, _
                                       scanlines2 as long ptr, _
                                       scanlinesA as long ptr, _
                                       lengths    as long ptr, _
                                       line_count as long, _
                                       alpha_     as single)

' Blends data of the left and right image scanlines 
' to get pixel values of "virtual" image scanlines
declare sub      cvMorphEpilinesMulti( line_count  as long, _
                                       first_pix   as ubyte ptr, _
                                       first_num   as long ptr, _
                                       second_pix  as ubyte ptr, _
                                       second_num  as long ptr, _
                                       dst_pix     as ubyte ptr, _
                                       dst_num     as long ptr, _
                                       alpha_      as single, _
                                       first       as long ptr, _
                                       first_runs  as long ptr, _
                                       second_     as long ptr, _
                                       second_runs as long ptr, _
                                       first_corr  as long ptr, _
                                       second_corr as long ptr)

' Does reverse warping of the morphing result 
' to make it fill the destination image rectangle
declare sub      cvPostWarpImage( line_count as long, _
                                  src        as ubyte ptr, _
                                  src_nums   as long ptr, _
                                  img        as IplImage ptr, _
                                  scanlines  as long ptr)

' Deletes Moire (missed pixels that appear due to discretization)
declare sub      cvDeleteMoire( img as IplImage ptr )

type CvConDensation
  as long            MP
  as long            DP
  as single ptr      DynamMatr    ' Matrix of the linear Dynamics system
  as single ptr      State        ' Vector of State
  as long            SamplesNum   ' Number of the Samples
  as single ptr ptr  flSamples    ' arr of the Sample Vectors
  as single ptr ptr  flNewSamples ' temporary array of the Sample Vectors
  as single ptr      flConfidence ' Confidence for each Sample
  as single ptr      flCumulative ' Cumulative confidence
  as single ptr      Temp         ' Temporary vector
  as single ptr      RandomSample ' RandomVector to update sample set
  as CvRandState ptr RandS        ' Array of structures to generate random vectors
end type

' Creates ConDensation filter state
declare function cvCreateConDensation( dynam_params   as long, _
                                       measure_params as long, _
                                       sample_count   as long ) as CvConDensation ptr

' Releases ConDensation filter state
declare sub      cvReleaseConDensation( condens as CvConDensation ptr ptr )

' Updates ConDensation filter by time (predict future state of the system)
declare sub      cvConDensUpdateByTime( condens as CvConDensation ptr )

' Initializes ConDensation filter samples
declare sub      cvConDensInitSampleSet( condens     as CvConDensation ptr, _
                                         lower_bound as CvMat ptr, _
                                         upper_bound as CvMat ptr )

function iplWidth( img as const IplImage ptr ) as long
  return iif( img=NULL,0, iif(img->roi=0,img->width,img->roi->width) )
end function

function iplHeight( img as const IplImage ptr ) as long
  return iif( img=NULL,0, iif(img->roi=0,img->height,img->roi->height) )
end function

' Splits color or grayscale image into multiple connected components
' of nearly the same color/brightness using modification of Burt algorithm.
' comp with contain a pointer to sequence (CvSeq)
' of connected components (CvConnectedComp
declare sub      cvPyrSegmentation( src        as IplImage ptr, _
                                    dst        as IplImage ptr, _
                                    storage    as CvMemStorage ptr, _
                                    comp       as CvSeq ptr ptr, _
                                    level      as long, _
                                    threshold1 as double, _
                                    threshold2 as double )


' #######################
' # Planar subdivisions #
' #######################

' Initializes Delaunay triangulation
declare sub      cvInitSubdivDelaunay2D( subdiv as CvSubdiv2D ptr, rect as CvRect )

' Creates new subdivision
declare function cvCreateSubdiv2D( subdiv_type as long, header_size as long, _
                                   vtx_size as long, quadedge_size as long, _
                                   storage as CvMemStorage ptr ) as CvSubdiv2D ptr

' ####################################
' # high-level subdivision functions #
' ####################################

' Simplified Delaunay diagram creation
function cvCreateSubdivDelaunay2D( rect as CvRect, storage as CvMemStorage ptr ) as CvSubdiv2D ptr
  dim as CvSubdiv2D ptr subdiv = cvCreateSubdiv2D( CV_SEQ_KIND_SUBDIV2D, sizeof(*subdiv), _
                                                   sizeof(CvSubdiv2DPoint), sizeof(CvQuadEdge2D), storage )

  cvInitSubdivDelaunay2D( subdiv, rect )
  return subdiv
end function

' Inserts new point to the Delaunay triangulation
declare function cvSubdivDelaunay2DInsert( subdiv as CvSubdiv2D ptr, pt as CvPoint2D32f) as CvSubdiv2DPoint ptr

' Locates a point within the Delaunay triangulation 
' finds the edge the point is left to or belongs to, 
' or the triangulation point the given point coinsides with
declare function cvSubdiv2DLocate( subdiv as CvSubdiv2D ptr, pt as CvPoint2D32f , _
                                   edge as CvSubdiv2DEdge ptr, _
                                   vertex as CvSubdiv2DPoint ptr ptr CV_DEFAULT(NULL) ) as CvSubdiv2DPointLocation ptr

' Calculates Voronoi tesselation (i.e. coordinates of Voronoi points)
declare sub      cvCalcSubdivVoronoi2D( subdiv as CvSubdiv2D ptr )


' Removes all Voronoi points from the tesselation
declare sub      cvClearSubdivVoronoi2D( subdiv as CvSubdiv2D ptr )


' Finds the nearest to the given point vertex in subdivision.
declare function cvFindNearestPoint2D( subdiv as CvSubdiv2D ptr, pt as CvPoint2D32f ) as CvSubdiv2DPoint ptr

' #############################################
' # Basic quad-edge navigation and operations #
' #############################################

' NOTE: CvSubdiv2DEdge is size_t
' NOTE: CvNextEdgeType is enum
function cvSubdiv2DNextEdge( edge as CvSubdiv2DEdge ) as CvSubdiv2DEdge 
  return  CV_SUBDIV2D_NEXT_EDGE(edge)
end function

function cvSubdiv2DRotateEdge( edge as CvSubdiv2DEdge , rotate as long ) as CvSubdiv2DEdge
  return (edge and not 3) + ((edge + rotate) and 3)
end function 

function cvSubdiv2DSymEdge( edge as CvSubdiv2DEdge  ) as CvSubdiv2DEdge
  return edge xor 2
end function 


function cvSubdiv2DGetEdge( edge as CvSubdiv2DEdge, type_ as CvNextEdgeType ) as CvSubdiv2DEdge 
  dim as CvQuadEdge2D ptr e = cptr( CvQuadEdge2D ptr,(edge and not 3) )
  'edge = e->next[(edge + (int)type_) & 3];
  edge = e->next((edge + clng(type_)) and 3)
  'return  (edge & ~3) + ((edge + ((int)type >> 4)) & 3);
  return  (edge and not 3) + ((edge + (clng(type_) shr 4)) and 3)
end function


function cvSubdiv2DEdgeOrg( edge as CvSubdiv2DEdge ) as CvSubdiv2DPoint ptr
  'CvQuadEdge2D* e = (CvQuadEdge2D*)(edge & ~3);
  'return (CvSubdiv2DPoint*)e->pt[edge & 3];
  dim as CvQuadEdge2D ptr e = cptr(CvQuadEdge2D ptr,edge and not 3)
  return cptr(CvSubdiv2DPoint ptr,e->pt(edge and 3))
end function 


function cvSubdiv2DEdgeDst( edge as CvSubdiv2DEdge ) as CvSubdiv2DPoint ptr
  'CvQuadEdge2D* e = (CvQuadEdge2D*)(edge & ~3);
  'return (CvSubdiv2DPoint*)e->pt[(edge + 2) & 3];
  dim as CvQuadEdge2D ptr e = cptr(CvQuadEdge2D ptr,edge and not 3)
  return cptr(CvSubdiv2DPoint ptr,e->pt((edge + 3) and 3))
end function 


' #########################################
' # Additional operations on Subdivisions #
' #########################################

' paints voronoi diagram: just demo function
declare sub      icvDrawMosaic( subdiv as CvSubdiv2D ptr, src as IplImage ptr, dst as IplImage ptr )

' checks planar subdivision for correctness. 
' It is not an absolute check, but it verifies some relations between quad-edges
declare function icvSubdiv2DCheck( subdiv as CvSubdiv2D ptr ) as long

' returns squared distance between two 2D points with floating-point coordinates.
function icvSqDist2D32f( pt1 as CvPoint2D32f, pt2 as CvPoint2D32f ) as double
  dim as double dx = pt1.x - pt2.x
  dim as double dy = pt1.y - pt2.y
  return dx*dx + dy*dy
end function 

function cvTriangleArea( a as CvPoint2D32f, b as CvPoint2D32f, c as CvPoint2D32f ) as double
  return cdbl(b.x - a.x)*cdbl(c.y - a.y) - cdbl(b.y - a.y)*cdbl(c.x - a.x)
end function


' Constructs kd-tree from set of feature descriptors
declare function cvCreateKDTree( desc as CvMat ptr ) as CvFeatureTree ptr

' Constructs spill-tree from set of feature descriptors
declare function cvCreateSpillTree( raw_data as const CvMat ptr, _
                                    naive as const long CV_DEFAULT(50), _
                                    rho as const double CV_DEFAULT(.7), _
                                    tau as const double CV_DEFAULT(.1) ) as CvFeatureTree ptr

' Release feature tree
declare sub      cvReleaseFeatureTree( tr as CvFeatureTree ptr )

' Searches feature tree for k nearest neighbors of given reference points,
' searching (in case of kd-tree/bbf) at most emax leaves.
declare sub      cvFindFeatures( tr as CvFeatureTree ptr, query_points as const CvMat ptr, _
                                 indices as CvMat ptr, dist as CvMat ptr, k as long, emax as long CV_DEFAULT(20) )

' Search feature tree for all points that are inlier to given rect region.
' Only implemented for kd trees
declare function cvFindFeaturesBoxed( tr as CvFeatureTree ptr, _
                                      bounds_min as CvMat ptr, bounds_max as CvMat ptr, _
                                      out_indices as CvMat ptr ) as long


' Construct a Locality Sensitive Hash (LSH) table, 
' for indexing d-dimensional vectors of given type. 
' Vectors will be hashed L times with k-dimensional p-stable (p=2) functions.
declare function cvCreateLSH( ops as CvLSHOperations ptr, d as long, _
                              L as long CV_DEFAULT(10), k as long CV_DEFAULT(10), _
                              type_ as long CV_DEFAULT(CV_64FC1), r as double CV_DEFAULT(4), _
                              seed as longint CV_DEFAULT(-1)) as CvLSH ptr

' Construct in-memory LSH table, with n bins.
declare function cvCreateMemoryLSH( d as long, n as long, L as long CV_DEFAULT(10), k as long CV_DEFAULT(10), _
                                    type_ as long CV_DEFAULT(CV_64FC1), r as double CV_DEFAULT(4), _
                                    seed as longint CV_DEFAULT(-1) ) as CvLSH ptr

' Free the given LSH structure.
declare sub      cvReleaseLSH( lsh as CvLSH ptr ptr )

' Return the number of vectors in the LSH.
declare function LSHSize( lsh as CvLSH ptr ) as ulong

' Add vectors to the LSH structure, optionally returning indices.
declare sub      cvLSHAdd( lsh as CvLSH ptr, data_ as const CvMat ptr, indices as CvMat ptr CV_DEFAULT(NULL) )

' Remove vectors from LSH, as addressed by given indices.
declare sub      cvLSHRemove( lsh as CvLSH ptr, indices as const CvMat ptr )

' Query the LSH n times for at most k nearest points; data is n x d,
' indices and dist are n x k. At most emax stored points will be accessed.
declare sub      cvLSHQuery( lsh as CvLSH ptr, query_points as const CvMat ptr, _
                             indices as CvMat ptr, dist as CvMat ptr, k as long, emax as long)

' Kolmogorov-Zabin stereo-correspondence algorithm (a.k.a. KZ1)
#define CV_STEREO_GC_OCCLUDED  SHRT_MAX

type CvStereoGCState
  as long Ithreshold
  as long interactionRadius
  as single K, lambda, lambda1, lambda2
  as long occlusionCost
  as long minDisparity
  as long numberOfDisparities
  as long maxIters
  as CvMat ptr left
  as CvMat ptr right
  as CvMat ptr dispLeft
  as CvMat ptr dispRight
  as CvMat ptr ptrLeft
  as CvMat ptr ptrRight
  as CvMat ptr vtxBuf
  as CvMat ptr edgeBuf
end type

declare function cvCreateStereoGCState( numberOfDisparities as long, maxIters as long ) as CvStereoGCState ptr
declare sub      cvReleaseStereoGCState( state as CvStereoGCState ptr ptr )
declare sub      cvFindStereoCorrespondenceGC( left_ as const CvArr ptr, right_ as const CvArr ptr, _
                                               disparityLeft as CvArr ptr, disparityRight as CvArr ptr, _
                                               state as CvStereoGCState ptr, _
                                               useDisparityGuess as long CV_DEFAULT(0) )

' Calculates optical flow for 2 images using classical Lucas & Kanade algorithm
declare sub      cvCalcOpticalFlowLK( prev as const CvArr ptr, curr as const CvArr ptr, _
                                      win_size as CvSize,  velx as CvArr ptr, vely as CvArr ptr )

' Calculates optical flow for 2 images using block matching algorithm
declare sub      cvCalcOpticalFlowBM( prev as const CvArr ptr, curr as const CvArr ptr, _
                                      block_size as CvSize, shift_size as CvSize, _
                                      max_range as CvSize, use_previous as long, _
                                      velx as CvArr ptr, vely as CvArr ptr )

' Calculates Optical flow for 2 images using Horn & Schunck algorithm
declare sub      cvCalcOpticalFlowHS( prev as const CvArr ptr, curr as const CvArr ptr, _
                                      use_previous as long, velx as CvArr ptr, vely as CvArr ptr, _
                                      lambda as double, criteria as CvTermCriteria )


' ######################################
' # Background/foreground segmentation #
' ######################################

' We discriminate between foreground and background pixels
' by building and maintaining a model of the background.
' Any pixel which does not fit this model is then deemed
' to be foreground.

' At present we support two core background models,
' one of which has two variations:

'  o CV_BG_MODEL_FGD: latest and greatest algorithm, described in

'	 Foreground Object Detection from Videos Containing Complex Background.
'	 Liyuan Li, Weimin Huang, Irene Y.H. Gu, and Qi Tian.
'	 ACM MM2003 9p

'  o CV_BG_MODEL_FGD_SIMPLE:
'       A code comment describes this as a simplified version of the above,
'       but the code is in fact currently identical

'  o CV_BG_MODEL_MOG: "Mixture of Gaussians", older algorithm, described in

'       Moving target classification and tracking from real-time video.
'       A Lipton, H Fujijoshi, R Patil
'       Proceedings IEEE Workshop on Application of Computer Vision pp 8-14 1998

'       Learning patterns of activity using real-time tracking
'       C Stauffer and W Grimson  August 2000
'       IEEE Transactions on Pattern Analysis and Machine Intelligence 22(8):747-757

#define CV_BG_MODEL_FGD		      0
#define CV_BG_MODEL_MOG		      1 ' "Mixture of Gaussians".
#define CV_BG_MODEL_FGD_SIMPLE	2

type _CvBGStatModel as CvBGStatModel

type CvReleaseBGStatModel as sub CV_CDECL ( bg_model as _CvBGStatModel ptr ptr )
type CvUpdateBGStatModel as function CV_CDECL ( curr_frame as IplImage ptr, _
                                                bg_model as _CvBGStatModel ptr, _
                                                learningRate as double) as long

#macro CV_BG_STAT_MODEL_FIELDS
  as long                 type               ' type of BG model
  as CvReleaseBGStatModel release
  as CvUpdateBGStatModel  update
  as IplImage ptr         background         ' 8UC3 reference background image
  as IplImage ptr         foreground         ' 8UC1 foreground image
  as IplImage ptr ptr     layers             ' 8UC3 reference background image, can be null
  as long                 layer_count        ' can be zero
  as CvMemStorage ptr     storage            ' storage for foreground_regions
  as CvSeq ptr            foreground_regions ' foreground object contours
#endmacro

type CvBGStatModel
  CV_BG_STAT_MODEL_FIELDS
end type

' Releases memory used by BGStatModel
declare sub      cvReleaseBGStatModel( bg_model as CvBGStatModel ptr )

' Updates statistical model and returns number of found foreground regions
declare function cvUpdateBGStatModel( current_frame as IplImage ptr, bg_model as CvBGStatModel ptr, _
                                      learningRate as double CV_DEFAULT(-1) ) as long

' Performs FG post-processing using segmentation
' (all pixels of a region will be classified as foreground if majority of pixels of the region are FG).
' parameters:
'  segments - pointer to result of segmentation (for example MeanShiftSegmentation)
'  bg_model - pointer to CvBGStatModel structure
declare sub      cvRefineForegroundMaskBySegm( segments as CvSeq ptr, bg_model as CvBGStatModel ptr )

' Common use change detection function
declare function cvChangeDetection( prev_frame  as IplImage ptr, _
                                    curr_frame  as IplImage ptr, _
                                    change_mask as IplImage ptr ) as long


' Interface of ACM MM2003 algorithm

' Default parameters of foreground detection algorithm:
#define  CV_BGFG_FGD_LC              128
#define  CV_BGFG_FGD_N1C             15
#define  CV_BGFG_FGD_N2C             25

#define  CV_BGFG_FGD_LCC             64
#define  CV_BGFG_FGD_N1CC            25
#define  CV_BGFG_FGD_N2CC            40

' Background reference image update parameter:
#define  CV_BGFG_FGD_ALPHA_1         0.1f

' stat model update parameter
' 0.002f ~ 1K frame(~45sec), 0.005 ~ 18sec (if 25fps and absolutely static BG)
#define  CV_BGFG_FGD_ALPHA_2         0.005f

' start value for alpha parameter (to fast initiate statistic model)
#define  CV_BGFG_FGD_ALPHA_3         0.1f

#define  CV_BGFG_FGD_DELTA           2

#define  CV_BGFG_FGD_T               0.9f

#define  CV_BGFG_FGD_MINAREA         15.f

#define  CV_BGFG_FGD_BG_UPDATE_TRESH 0.5f

' See the above-referenced Li/Huang/Gu/Tian paper
' for a full description of these background-model
' tuning parameters.

' Nomenclature:  'c'  == "color", a three-component red/green/blue vector.
'                         We use histograms of these to model the range of
'                         colors we've seen at a given background pixel.

'                'cc' == "color co-occurrence", a six-component vector giving
'                         RGB color for both this frame and preceding frame.
'                             We use histograms of these to model the range of
'                         color CHANGES we've seen at a given background pixel.
type CvFGDStatModelParams
  as long    Lc			' Quantized levels per 'color' component. Power of two, typically 32, 64 or 128.
  as long    N1c		' Number of color vectors used to model normal background color variation at a given pixel.
  as long    N2c		' Number of color vectors retained at given pixel.  Must be > N1c, typically ~ 5/3 of N1c.
    ' Used to allow the first N1c vectors to adapt over time to changing background.

  as long    Lcc		' Quantized levels per 'color co-occurrence' component.  Power of two, typically 16, 32 or 64.
  as long    N1cc		' Number of color co-occurrence vectors used to model normal background color variation at a given pixel.
  as long    N2cc		' Number of color co-occurrence vectors retained at given pixel.  Must be > N1cc, typically ~ 5/3 of N1cc.
    ' Used to allow the first N1cc vectors to adapt over time to changing background.

  as long    is_obj_without_holes ' If TRUE we ignore holes within foreground blobs. Defaults to TRUE.
  as long    perform_morphing 	  ' Number of erode-dilate-erode foreground-blob cleanup iterations.
    ' These erase one-pixel junk blobs and merge almost-touching blobs. Default value is 1.

  as single alpha1	' How quickly we forget old background pixel values seen.  Typically set to 0.1
  as single alpha2	' "Controls speed of feature learning". Depends on T. Typical value circa 0.005.
  as single alpha3	' Alternate to alpha2, used (e.g.) for quicker initial convergence. Typical value 0.1.

  as single delta		' Affects color and color co-occurrence quantization, typically set to 2.
  as single T			  ' "A percentage value which determines when new features can be recognized as new background." (Typically 0.9).
  as single minArea	' Discard foreground blobs whose bounding box is smaller than this threshold.
end type

type CvBGPixelCStatTable
  as single Pv, Pvb
  as ubyte  v(2)
end type

type CvBGPixelCCStatTable
  as single Pv, Pvb
  as ubyte  v(5)
end type

type CvBGPixelStat
  as single                   Pbc
  as single                   Pbcc
  as CvBGPixelCStatTable  ptr ctable
  as CvBGPixelCCStatTable ptr cctable
  as ubyte                    is_trained_st_model
  as ubyte                    is_trained_dyn_model
end type

type CvFGDStatModel
  CV_BG_STAT_MODEL_FIELDS
  as CvBGPixelStat ptr    pixel_stat
  as IplImage ptr         Ftd
  as IplImage ptr         Fbd
  as IplImage ptr         prev_frame
  as CvFGDStatModelParams params
end type

' Creates FGD model
declare function cvCreateFGDStatModel( first_frame as IplImage ptr, _
                                       parameters as CvFGDStatModelParams ptr CV_DEFAULT(NULL) ) as CvBGStatModel ptr


' Interface of Gaussian mixture algorithm

' "An improved adaptive background mixture model for real-time tracking with shadow detection"
' P. KadewTraKuPong and R. Bowden,
' Proc. 2nd European Workshp on Advanced Video-Based Surveillance Systems, 2001."
' http://personal.ee.surrey.ac.uk/Personal/R.Bowden/publications/avbs01/avbs01.pdf

' Note:  "MOG" == "Mixture Of Gaussians":

#define CV_BGFG_MOG_MAX_NGAUSSIANS 500

' default parameters of gaussian background detection algorithm
#define CV_BGFG_MOG_BACKGROUND_THRESHOLD     0.7     ' threshold sum of weights for background test
#define CV_BGFG_MOG_STD_THRESHOLD            2.5     ' lambda=2.5 is 99%
#define CV_BGFG_MOG_WINDOW_SIZE              200     ' Learning rate; alpha = 1/CV_GBG_WINDOW_SIZE
#define CV_BGFG_MOG_NGAUSSIANS               5       ' = K = number of Gaussians in mixture
#define CV_BGFG_MOG_WEIGHT_INIT              0.05
#define CV_BGFG_MOG_SIGMA_INIT               30
#define CV_BGFG_MOG_MINAREA                  15.f

#define CV_BGFG_MOG_NCOLORS                  3

type CvGaussBGStatModelParams
  as long    win_size               ' = 1/alpha
  as long    n_gauss
  as double  bg_threshold, std_threshold, minArea
  as double  weight_init, variance_init
end type

type CvGaussBGValues
  as long   match_sum
  as double weight
  as double variance(CV_BGFG_MOG_NCOLORS-1)
  as double mean(CV_BGFG_MOG_NCOLORS-1)
end type

type CvGaussBGPoint
  as CvGaussBGValues ptr g_values
end type

type CvGaussBGModel
  CV_BG_STAT_MODEL_FIELDS
  as CvGaussBGStatModelParams params
  as CvGaussBGPoint ptr       g_point
  as long                     countFrames
  as any ptr                  mog
end type

' Creates Gaussian mixture background model
declare function cvCreateGaussianBGModel( first_frame as IplImage ptr, _
                                          parameters as CvGaussBGStatModelParams ptr CV_DEFAULT(NULL) ) as CvBGStatModel ptr

type CvBGCodeBookElem
  as CvBGCodeBookElem ptr next
  as long tLastUpdate
  as long stale
  as ubyte boxMin(2)
  as ubyte boxMax(2)
  as ubyte learnMin(2)
  as ubyte learnMax(2)
end type

type CvBGCodeBookModel
  as CvSize size
  as long t
  as ubyte cbBounds(2)
  as ubyte modMin(2)
  as ubyte modMax(2)
  as CvBGCodeBookElem ptr ptr cbmap
  as CvMemStorage ptr storage
  as CvBGCodeBookElem ptr freeList
end type

declare function cvCreateBGCodeBookModel() as CvBGCodeBookModel ptr
declare sub      cvReleaseBGCodeBookModel( model as CvBGCodeBookModel ptr ptr )

declare sub      cvBGCodeBookUpdate( model as CvBGCodeBookModel ptr, image as const CvArr ptr, _
                                     roi   as CvRect CV_DEFAULT(cv_Rect(0,0,0,0)), _
                                     mask  as const CvArr ptr CV_DEFAULT(NULL) )

declare function cvBGCodeBookDiff( model as const CvBGCodeBookModel ptr, image as const CvArr ptr, _
                                   fgmask as CvArr ptr, roi as CvRect CV_DEFAULT(cv_Rect(0,0,0,0)) ) as long

declare sub      cvBGCodeBookClearStale( model as CvBGCodeBookModel ptr, staleThresh as long, _
                                         roi as CvRect CV_DEFAULT(cv_Rect(0,0,0,0)), _
                                         mask as const CvArr ptr CV_DEFAULT(NULL) )

declare function cvSegmentFGMask( fgmask as CvArr ptr, poly1Hull0 as long CV_DEFAULT(1), _
                                  perimScale as single CV_DEFAULT(4.f), _
                                  storage as CvMemStorage ptr CV_DEFAULT(NULL), _
                                  offset as CvPoint CV_DEFAULT(cv_Point(0,0)) ) as CvSeq ptr


end extern ' "C"

