#pragma once

#ifndef __OPENCV__
  #error 666: do not include this file manually include only 'opencv.bi' !
#endif

' #include "opencv_core.bi"

extern "C"

' ########################################
' # Haar-like Object Detection functions #
' ########################################

#define CV_HAAR_MAGIC_VAL    &H42500000
#define CV_TYPE_NAME_HAAR    "opencv-haar-classifier"

#define CV_IS_HAAR_CLASSIFIER( haar ) _
    ((haar) <> NULL andalso ( ((cptr(const CvHaarClassifierCascade ptr,haar))->flags and CV_MAGIC_MASK)=CV_HAAR_MAGIC_VAL )

#define CV_HAAR_FEATURE_MAX  3

type _CvHaarFeatureRect
  as CvRect r
  as single weight
end type

type CvHaarFeature
  as long tilted
  as _CvHaarFeatureRect rect(CV_HAAR_FEATURE_MAX-1)
end type

type CvHaarClassifier
  as long count
  as CvHaarFeature ptr haar_feature
  as single ptr threshold
  as long ptr   left
  as long ptr   right
  as single ptr alpha
end type

type CvHaarStageClassifier
  as long count
  as single threshold
  as CvHaarClassifier ptr classifier
  as long next
  as long child
  as long parent
end type

type CvHidHaarClassifierCascade as any 

type CvHaarClassifierCascade
  as long flags
  as long count
  as CvSize orig_window_size
  as CvSize real_window_size
  as double scale
  as CvHaarStageClassifier ptr stage_classifier
  as CvHidHaarClassifierCascade ptr hid_cascade
end type

type CvAvgComp
  as CvRect rect
  as long neighbors
end type

' Loads haar classifier cascade from a directory.
' It is obsolete: convert your cascade to xml and use cvLoad instead
declare function cvLoadHaarClassifierCascade( directory as const zstring ptr, orig_window_size as CvSize ) as CvHaarClassifierCascade ptr

declare sub      cvReleaseHaarClassifierCascade( cascade as CvHaarClassifierCascade ptr ptr )

#define CV_HAAR_DO_CANNY_PRUNING    1
#define CV_HAAR_SCALE_IMAGE         2
#define CV_HAAR_FIND_BIGGEST_OBJECT 4
#define CV_HAAR_DO_ROUGH_SEARCH     8

declare function cvHaarDetectObjects( image as const CvArr ptr, _
                                      cascade as CvHaarClassifierCascade ptr, _
                                      storage as CvMemStorage ptr, _
                                      scale_factor as double CV_DEFAULT(1.1), _
                                      min_neighbors as long CV_DEFAULT(3), _
                                      flags as long CV_DEFAULT(0), _
                                      min_size as CvSize CV_DEFAULT(cv_Size(0,0)), _
                                      max_size as CvSize CV_DEFAULT(cv_Size(0,0)) ) as CvSeq ptr

' sets images for haar classifier cascade
declare sub      cvSetImagesForHaarClassifierCascade( cascade as CvHaarClassifierCascade ptr, _
                                                      sum as const CvArr ptr, _
                                                      sqsum as const CvArr ptr, _
                                                      tilted_sum as const CvArr ptr, _
                                                      scale as double )

' runs the cascade on the specified window
declare function cvRunHaarClassifierCascade( cascade as const CvHaarClassifierCascade ptr, _
                                             pt as CvPoint, _
                                             start_stage as long CV_DEFAULT(0) ) as long


' #########################################
' # Latent SVM Object Detection functions #
' #########################################

' DataType: STRUCT position
' Structure describes the position of the filter in the feature pyramid
' l - level in the feature pyramid
' (x, y) - coordinate in level l
type CvLSVMFilterPosition
  as long x
  as long y
  as long l
end type

' DataType: STRUCT filterObject
' Description of the filter, which corresponds to the part of the object
' V               - ideal (penalty = 0) position of the partial filter
'                   from the root filter position (V_i in the paper)
' penaltyFunction - vector describes penalty function (d_i in the paper)
'                   pf[0] * x + pf[1] * y + pf[2] * x^2 + pf[3] * y^2
' FILTER DESCRIPTION
'   Rectangular map (sizeX x sizeY),
'   every cell stores feature vector (dimension = p)
' H               - matrix of feature vectors
'                   to set and get feature vectors (i,j)
'                   used formula H[(j * sizeX + i) * p + k], where
'                   k - component of feature vector in cell (i, j)
' END OF FILTER DESCRIPTION
type CvLSVMFilterObject
  as CvLSVMFilterPosition V
  as single fineFunction(3)
  as long sizeX
  as long sizeY
  as long numFeatures
  as single ptr H
end type

' data type: STRUCT CvLatentSvmDetector
' structure contains internal representation of trained Latent SVM detector
' num_filters          - total number of filters (root plus part) in model
' num_components       - number of components in model
' num_part_filters     - array containing number of part filters for each component
' filters              - root and part filters for all model components
' b                    - biases for all model components
' score_threshold      - confidence level threshold
type CvLatentSvmDetector
  as long num_filters
  as long num_components
  as long ptr num_part_filters
  as CvLSVMFilterObject ptr ptr filters
  as single ptr b
  as single score_threshold
end type

' data type: STRUCT CvObjectDetection
' structure contains the bounding box and confidence level for detected object
' rect                 - bounding box for a detected object
' score                - confidence level
type CvObjectDetection
  as CvRect rect
  as single score
end type

' #####################################
' # Object Detection using Latent SVM #
' #####################################



' load trained detector from a file

' API
' CvLatentSvmDetector* cvLoadLatentSvmDetector(const char* filename);
' INPUT
' filename             - path to the file containing the parameters of
'                      - trained Latent SVM detector
' OUTPUT
' trained Latent SVM detector in internal representation

declare function cvLoadLatentSvmDetector( filename as const zstring ptr ) as CvLatentSvmDetector ptr


' release memory allocated for CvLatentSvmDetector structure

' API
' void cvReleaseLatentSvmDetector(CvLatentSvmDetector** detector);
' INPUT
' detector             - CvLatentSvmDetector structure to be released
' OUTPUT
declare sub cvReleaseLatentSvmDetector( detector as CvLatentSvmDetector ptr )


' find rectangular regions in the given image that are likely
' to contain objects and corresponding confidence levels

' API
' CvSeq* cvLatentSvmDetectObjects(const IplImage* image,
'                                  CvLatentSvmDetector* detector,
'                                  CvMemStorage* storage,
'                                  float overlap_threshold = 0.5f,
'                                  int numThreads = -1);
' INPUT
' image                - image to detect objects in
' detector             - Latent SVM detector in internal representation
' storage              - memory storage to store the resultant sequence
'                          of the object candidate rectangles
' overlap_threshold    - threshold for the non-maximum suppression algorithm
'                      = 0.5f [here will be the reference to original paper]
' OUTPUT
' sequence of detected objects (bounding boxes and confidence levels stored in CvObjectDetection structures)

declare function cvLatentSvmDetectObjects( image as IplImage ptr, _
                                           detector as CvLatentSvmDetector ptr, _
                                           storage as CvMemStorage ptr, _
                                           overlap_threshold as single CV_DEFAULT(0.5f), _
                                           numThreads as long CV_DEFAULT(-1) ) as CvSeq ptr


end extern ' "C"

